Package: kstMatrix
Type: Package
Date: 2026-04-26
Version: 2.3-2
Title: Basic Functions in Knowledge Space Theory Using Matrix
        Representation
Description: Knowledge space theory by Doignon and Falmagne (1999)
   <doi:10.1007/978-3-642-58625-5> is a set- and order-theoretical
   framework, which proposes mathematical formalisms to operationalize
   knowledge structures in a particular domain. The 'kstMatrix' package 
   provides basic functionalities to generate, handle, and manipulate 
   knowledge structures and knowledge spaces. Opposed to the 'kst'
   package, 'kstMatrix' uses matrix representations for knowledge
   structures. Furthermore, 'kstMatrix' contains several knowledge spaces
   developed by the research group around Cornelia Dowling through
   querying experts.
Depends: R (>= 4.4.0)
Imports: stats, grDevices, sets, pks, tidyr, DiagrammeR, rsvg
Suggests: DiagrammeRsvg, litedown, markdown
Authors@R: c(person(given = "Cord", family = "Hockemeyer",
                    role = c("aut", "cre"),
                    email = "cord.hockemeyer@uni-graz.at"),
             person(given = "Peter", family = "Steiner",
                    role = c("aut"),
                    email = "Peter.Steiner@PHSG.ch"),
             person(given = "Wai", family = "Wong",
                    role = c("aut"),
                    email = "wai.wong@kuleuven.be")
             )
Maintainer: Cord Hockemeyer <cord.hockemeyer@uni-graz.at>
License: GPL-3
NeedsCompilation: yes
Repository: CRAN
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
VignetteBuilder: litedown
Packaged: 2026-04-26 11:34:06 UTC; cord
Author: Cord Hockemeyer [aut, cre],
  Peter Steiner [aut],
  Wai Wong [aut]
Date/Publication: 2026-04-26 12:20:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-27 00:01:23 UTC; unix
Archs: kstMatrix.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(kmbasis,kmsurmisefunction)
S3method(kmbasis,kmsurmiserelation)
S3method(kmbasis,matrix)
S3method(kmunionclosure,kmdata)
S3method(kmunionclosure,kmfamset)
S3method(kmunionclosure,kmstructure)
S3method(plot,kmfamset)
S3method(plot,kmneighbourhood)
S3method(plot,kmsurmiserelation)
export(kmSF2basis)
export(kmSR2basis)
export(kmSRvalidate)
export(kmassess)
export(kmassessbayesian)
export(kmassesshalfsplit)
export(kmassessinformative)
export(kmassessmentsimulation)
export(kmassessmultiplicative)
export(kmbasis)
export(kmbasisfringe)
export(kmbasisinnerfringe)
export(kmbasisneighbourhood)
export(kmbasisouterfringe)
export(kmcolors)
export(kmdist)
export(kmdoubleequal)
export(kmeqreduction)
export(kmfamset)
export(kmfringe)
export(kmgenerate)
export(kmgradations)
export(kmiita2SR)
export(kminnerfringe)
export(kmiswellgraded)
export(kmlearningpaths)
export(kmmaximalspace)
export(kmminimalspace)
export(kmneighbourhood)
export(kmnneighbourhood)
export(kmnotions)
export(kmouterfringe)
export(kmsassess)
export(kmsetdistance)
export(kmsetiselement)
export(kmsimulate)
export(kmspace)
export(kmstructure)
export(kmsurmisefunction)
export(kmsurmiserelation)
export(kmsymmsetdiff)
export(kmunionclosure)
export(kmvalidate)
importFrom(DiagrammeR,grViz)
importFrom(grDevices,cm.colors)
importFrom(grDevices,dev.off)
importFrom(grDevices,jpeg)
importFrom(grDevices,terrain.colors)
importFrom(pks,as.binmat)
importFrom(pks,as.pattern)
importFrom(rsvg,rsvg_png)
importFrom(sets,"%e%")
importFrom(sets,as.set)
importFrom(sets,set)
importFrom(sets,set_is_subset)
importFrom(stats,runif)
importFrom(tidyr,unnest)
useDynLib(kstMatrix)

# knitr::opts_chunk$set(
#   collapse = TRUE,
#   comment = "#>"
# )
library(kstMatrix)
library(grDevices)
library(sets)
library(pks)

# litedown::include_graphics("kstMatrixClasses.png")

plot(xpl$space)
plot(xpl$sr)

plot(xpl$sr, colors="#aaffaa", vertexshape="box")

plot(kmneighbourhood(c(1,1,0,0), xpl$space, include=TRUE), state=c(1,1,0,0), edgelabel=TRUE)

plot(xpl$space, horizontal=TRUE, arrowtail="open")
plot(xpl$space, arrowhead="empty")

#' Derive a basis from a surmise function
#'
#' \code{kmSF2basis} expects a surmise function data frame
#' and returns the corresponding basis.
#'
#' @param sf Surmise function
#'
#' @return Matrix representing the basis.
#'
#' @family Different representations for knowledge spaces
#'
#' @export
kmSF2basis <- function(sf) {
  noc <- dim(sf)[2]
  bas <- unique(apply(as.matrix(sf[, 2:noc]), 2, as.numeric))
  class(bas) <- unique(c("kmbasis", "kmfamset", class(bas)))
  bas
}

#' Compute the symmetric set difference between two sets
#'
#' @param x Binary vector representing a set
#' @param y Binary vector representing a set
#' @return kmsymmsetdiff: Symmetric set difference between 'x' and 'y'
#'
#' @examples
#' kmsymmsetdiff(c(1,0,0), c(1,1,0))
#'
#' @family Utilities
#'
#' @export
kmsymmsetdiff <- function(x, y) {
  (1 * (x | y) - 1 * (x & y))
}


#' @rdname kmsymmsetdiff
#' @return kmsetdistance: Distance between the sets 'x' and 'y', i.e. the cardinality
#' of the symmetric set difference
#' @examples
#' kmsetdistance(c(1,0,0), c(1,1,0))
#'
#' @family Utilities
#'
#' @export
kmsetdistance <- function(x, y) {
  sum(1 * (x | y) - 1 * (x & y))
}

#' Convert a binary matrix to a \code{kmstructure} object
#'
#' \code{kmstructure()} returns a \code{kmstructure} object after checking that
#' the passed object is a binary matrix without double rows. The empty set
#' and the full item set are added if missing.
#'
#' @param x Binary matrix representing a family of sets
#' @return \code{kmstructure} object
#'
#' @examples
#' m <- matrix(c(1,0,0,0,1,0,1,1,1), nrow=3, byrow=TRUE)
#' kmstructure(m)
#'
#' @family Constructors
#'
#' @export
kmstructure <- function(x) {
  if (!inherits(x, "matrix")) {
    stop(sprintf("%s must be of class %s.", dQuote("data"), dQuote("matrix")))
  }
  if (any(x != 1*as.logical(x))) {
    stop(sprintf("%s must be a binary matrix.", dQuote("data")))
  }

  t <- kmminimalspace(dim(x)[2])
  r <- rbind(x,t)
  res <- kmfamset(r)
  class(res) <- unique(c("kmstructure", class(res)))
  res
}

#' Compute the fringe of a state within a knowledge structure
#'
#' \code{kmfringe} computes the fringe of a state
#' within a knowledge structure, i.e. the set of items by which
#' the state differs from its neighbours.
#'
#' @param state Binary vector representing a knowledge state
#' @param struct Binary matrix representing a knowledge structure
#' @return Binary vector representing the fringe
#'
#' @examples
#' kmfringe(c(1,0,0,0), xpl$space)
#'
#' @family Functions for fringes & neighbourhoods
#'
#' @export
kmfringe <- function(state, struct) {
  if (!inherits(struct, "matrix")) {
    stop(sprintf("%s must be of class %s.", dQuote("struct"), dQuote("matrix")))
  }
  if (any(struct != 1*as.logical(struct))) {
    stop(sprintf("%s must be a binary matrix.", dQuote("struct")))
  }
  if (length(state) != dim(struct)[2]) {
    stop(sprintf("%s and %s don't match in size.", dQuote("state"), dQuote("struct")))
  }

  n <- kmneighbourhood(state, struct, include=FALSE)
  apply(t(apply(n, 1, function(x) {(state | x) - (state & x)})), 2, sum)
}


#' @rdname kmfringe
#' @export
kminnerfringe <- function(state, struct) {
  f <- kmfringe(state, struct)
  as.integer(f & state)
}


#' @rdname kmfringe
#' @export
kmouterfringe <- function(state, struct) {
  f <- kmfringe(state, struct)
  as.integer(f & !state)
}

#' Compute the "union" of two KST objects
#'
#' Both objects must be of the same kstMatrix class, the result will be
#' of the same class, i.e. a respective closure operator will be executed
#' automatically.
#'
#' For surmise functions, the algorithm by Dowling & Hockemeyer (1998) is used.
#'
#' Please note that, for families of sets, there are the functions
#' \code{kmunionclosure()} and \code{kmintersectionclosure()}.
#'
#' @references Dowling CE, Hockemeyer C (1998). “Computing the Intersection
#'    of Knowledge Spaces Using only their Basis.” In Dowling CE, Roberts FS,
#'    Theuns P (eds.), _Recent Progress in Mathematical Psychology,_ 133–141.
#'    Lawrence Erlbaum Associates Ltd., Mahwah, NJ.
#'
#' @param x First object
#' @param y Second object
#'
#' @return _Union_ of \code{x} and \code{y}
#'
#' @useDynLib kstMatrix
#'
#' @family Functions for working with knowledge structures
#'
#' @export
kmunion <- function(x, y) {
  UseMethod("kmunion")
}

#' Perform a simplified probabilistic knowledge assessment
#'
#' \code{kmsassess} performs a simplified probabilistic knowledge assessment
#' for a given response vector, knowledge structure, and BLIM parameters. It
#' assumes an equal probability distribution over the knowledge structure
#' as starting point and identical beta and eta values for all items.
#'
#' @param r Response pattern (binary vector)
#' @param ks Knowledge structure: a binary matrix
#' @param questioning Questioning rule ("halfsplit" o "informative")
#' @param update Update rule ("Bayesian" or "multiplicative")
#' @param beta Careless error probability
#' @param eta Lucky guess probability
#' @param zeta0 Update parameter for wrong responses
#' @param zeta1 Update parameter for correct responses
#' @param threshold Probability threshold for stopping criterion
#' @param probdev Provide information on the probability development
#'    including Hasse diagrams stored in \code{tempdir()}. Defaults to FALSE.
#' @param directory Where to store the Hasse diagrams.
#' @return A list with the following elements:
#' \describe{
#'   \item{state}{Diagnosed knowledge state (binary vector)}
#'   \item{probs}{Resulting probability distribution. If probdev is set to TRUE,
#'                a list of probability distributions for each step is given
#'                instead.}
#'   \item{queried}{Sequence of items used in the assessment (list)}
#'   \item{qtime}{Average time for finding a question}
#'   \item{utime}{Average time for updating the probabilities}
#' }
#'
#'
#' @examples
#' kmsassess(c(1,1,0,0), xpl$space, "halfsplit", "Bayesian", 0.1, 0.1, NULL, NULL, 0.55)
#'
#' @family Knowledge assessment functions
#'
#' @rdname kmassess
#' @export
kmsassess <- function(r, ks, questioning, update, beta, eta, zeta0, zeta1, threshold, probdev=FALSE, directory=NULL) {
  storage.mode(ks) <- "integer"
  if ((threshold < 0) | (threshold > 1))
    stop("Threshokd must be between 0 and 1.")
  if (threshold <= 0.5)
    warning("Threshold shoud be larger than 0.5!")
  if ((min(ks) < 0) | (max(ks) > 1))
    stop("Knowledge structure must be a binary matrix.")
  if (update == "Bayesian") {
    multiplicative <- FALSE
    if (is.null(beta) | is.null(eta))
      stop("beta and eta must be set!")
    if ((beta < 0) | (beta > 1))
      stop("Illegal beta value.")
    if ((eta < 0) | (eta > 1))
      stop("Illegal eta value.")
    if (!(is.null(zeta0)) | !(is.null(zeta1)))
      warning("Ignoring zeta values for Bayesian update rule")
  } else {
    multiplicative <- TRUE
    if (is.null(zeta0) | is.null((zeta1)))
      stop("zeeta0 and zeta1 must be set.")
    if ((min(zeta0) <= 1) | (min(zeta1) <= 1))
      stop("zeta0 and zeta1 must be greater than 1")
    if (!(is.null(beta)) | !(is.null(eta)))
      warning("Ignoring beta and eta values for multiplicative update rule")
  }

  noi <- dim(ks)[2]
  nos <- dim(ks)[1]
  probs <- as.matrix(rep(1/nos, nos), ncol = 1)
  pks <- cbind(as.data.frame(probs), as.data.frame(ks))
  bv <- rep(beta, noi)
  ev <- rep(eta, noi)
  z0 <- rep(zeta0, noi)
  z1 <- rep(zeta1, noi)
  if (update == "Bayesian")
    kmassess(r, pks, questioning, update, bv, ev, NULL, NULL, threshold, probdev, directory)
  else
    kmassess(r, pks, questioning, update, NULL, NULL, z0, z1, threshold, probdev, directory)
}

#' Compute the intersection of two knowledge spaces
#'
#' @family Functions for working with knowledge structures
#' @method kmintersection kmspace
#' @rdname kmintersection
#' @export
kmintersection.kmspace <- function(x, y) {
  if (!(inherits(y, "kmspace")))
    stop("'y' must also be a knowledge space.")
  print(x)
  print(y)
  m <- as.matrix(merge(x, y))
  print(m)
  m
}

#' Determine the quasi-ordinal knowledge space containing the knowledge space delineated by a surmise function
#'
#' \code{kmqspace.kmsurmisefunction()} takes a surmise function and
#' determines the quasi-ordinal knowledge space which is the closure
#' under intersection of the knowledge space delineated by this surmise
#' function.
#'
#' @family Constructors
#' @rdname kmqspace
#' @method kmqspace kmsurmisefunction
#' @export
kmqspace.kmsurmisefunction <- function(x) {
  b <- kmbasis(x)
  sr <- kmsurmiserelation(b)
  b <- kmbasis(sr)
  s <- kmunionclosure(b)
  class(s) <- unique(c("kmqspace", class(s)))
  s
}

#' Generate a knowledge structure from a set of response patterns
#'
#' \code{kmgenerate} returns a matrix representing a knowledge structure generated
#' from data. It uses a simplistic approach: patterns with a frequency above a
#' specified threshold are considered as knowledge states.
#' If the specified threshold is 0 (default) a real threshold is computed as
#' N (number of  response patterns) divided by 2^|Q|.
#' Please note that the number of response patterns should be much higher than the
#' size of the power set of the item set Q. A factor of art least 10 is recommended.
#' Currently, the number of items is limited to the number of bits in a C long
#' minus one (i.e. 31 under Windows and 63 otherwise). But we would probably run into
#' memory problems way earlier anyway.
#'
#' @param x Binary matrix representing a data set
#' @param threshold Threshold for taking response patterns as knowledge states (default 0)
#' @return Binary matrix representing the generated knowledge structure
#'
#'
#' @examples
#' kmgenerate(xpl$sim, 15)
#'
#' @useDynLib kstMatrix
#'
#'
#' @family Functions for working with data
#'
#' @export
kmgenerate <- function(x, threshold = 0) {

  gen_errmsg <- c(
    "Too many items! The number of items is generally limited to 64, on Windows systems it is probalby limited to 32.",
    "Not enough memory!"
  )
  if (!inherits(x, "matrix")) {
    stop(sprintf("%s must be of class %s.", dQuote("x"), dQuote("matrix")))
  }
  if (any(x != 1*as.logical(x))) {
    stop(sprintf("%s must be a binary matrix.", dQuote("x")))
  }

  noi <- as.integer(dim(x)[2])
  nod <- as.integer(dim(x)[1])
  if (threshold == 0) {
    threshold <- ceiling(nod / (2^noi));
  }
  storage.mode(x) <- "integer"
  nos <- as.integer(0)
  rc <- as.integer(0)
  result <- .C("generate", noi, nod, t(x), as.integer(threshold), rc, nos, package="kstMatrix")
  rc <- result[[5]][1]
  if (rc != 0) {
    stop(gen_errmsg[rc])
  }
  nos <- result[[6]][1]
  kstruct <- matrix(1:(noi*nos), ncol=noi, nrow=nos, byrow = TRUE)
  storage.mode(kstruct) <- "integer"
  result2 <- .C("generate_results", kstruct, package="kstMatrix")
  s <- result2[[1]]
  colnames(s) <- colnames(x)
  class(s) <- unique(c("kmstructure", "kmfamset", class(s)))
  s
}

#' Expand an attribution function to a larger set of items and close it to a surmise function
#'
#' \code{kmexpand()} expands an attribution function by additional items and
#' closes it to a surmise function.
#'
#' @param af Attribution function
#' @param ni New (complete) item list
#' @returns Expanded surmise function
#'
#' @examples
#' xpl$sf
#' kmexpand(xpl$sf, c("a", "b", "c", "d", "e", "f"))
#'
#'
#' @family Functions for working with knowledge structures
#'
#' @export
kmexpand <- function(af, ni) {
  if (!inherits(af, "kmattributionfunction"))
    stop("'af' must be of class 'kmattributionfunction'")
  af <- data.frame(af)
  oi <- colnames(af)[-1]
  if (!all(oi %in% ni))
    stop("'ni' must contain all itms of 'af'.")
  rows <- dim(af)[1]
  additionals <- ni[which(!(ni %in% oi))]
  hmat <- matrix(rep(0, rows*length(additionals)), nrow=rows)
  colnames(hmat) <- additionals
  af <- cbind(af, data.frame(hmat))
  af <- af[c(colnames(af)[1], ni)]
  class(af) <- unique(c("kmattributionfunction", class(af)))
  kmclosure(af)
}

#' Refine a famset by dissolving a notion through a basis on the notion
#'
#' @param x Family of sets
#' @param b Basis for refinement
#' @returns Refined family of sets
#'
#' @examples
#' x <- kmfamset(matrix(c(1,0,0,0,0,1,0,0,1,1,1,1), ncol=4, byrow=TRUE))
#' colnames(x) <- c("a", "b", "c", "d")
#' x
#' b <- kmbasis(kmfamset(matrix(c(1,0,1,1), ncol=2, byrow=TRUE)))
#' colnames(b) <- c("c", "d")
#' b
#' kmrefine(x,b)
#'
#' @export
kmrefine <- function(x, b) {
  if (!inherits(x, "kmfamset"))
    stop("'x' must be a 'kmfamset' object.")
  if (!inherits(b, "kmbasis"))
    stop("'b' must be a 'kmbasis' object.")
  n <- kmnotions(x)
  rownames(x) <- NULL
  bnames <- colnames(b)
  totalitems <- dim(x)[2]
  found <- FALSE
  apply(n, MARGIN=1, function(s) {
    items <- colnames(x)[which(s==1)]
    if (all(items %in% colnames(b))) found <<- TRUE
  })
  if (!found) stop("The items of 'b' must be a subset of one notion in 'x'.")
  item <- colnames(b)[1]
  states <- (which(x[, item] == 1))
  mat <- x[-(states),]
  sapply(list(states), function(s) {
    hmat <- matrix(rep(x[s,], dim(b)[1]), ncol=totalitems, byrow=TRUE)
    colnames(hmat) <- colnames(x)
    hmat[,bnames] <- b
    mat <<- rbind(mat, hmat)
  })
  kmfamset(mat)
}

#' @rdname kmunion
#'
#' @method kmunion kmsurmisefunction
#' @family Functions for working with knowledge structures
#' @export
kmunion.kmsurmisefunction <- function(x, y) {
  if (!(inherits(y, "kmsurmisefunction")))
    stop("'y' must be a kmbasis object.")
  noix <- dim(x)[2] - 1
  noiy <- dim(y)[2] - 1
  if (noix != noiy)
    stop("Objects do not fit in number of items.")
  if (is.null(colnames(x))) {
    if (!(is.null(colnames(y))))
      stop("Item names do not fit.")
  } else if (is.null(colnames(y)))
    stop("Item names do not fit.")
  else {
    cnx <- colnames(x)[-1]
    cny <- colnames(y)[-1]
    if (any(cnx != cny))
      stop("Item names do not fit")
  }

  res <- data.frame("ignore", t(rep(0,noix)))
  colnames(res) <- colnames(x)
  apply(x, MARGIN=1, function(c) {
    item <- c[1]
    clause <- as.numeric(c[-1])
    clist = which(y[,1] == item)
    apply(y[clist,], MARGIN=1, function(d) {
      u <- data.frame(item, t(as.numeric(clause | as.numeric(d[-1]))))
      colnames(u) <- colnames(res)
      rownames(u) <- NULL
      res <<- rbind(res, u)
    })
  })
  res <- res[-1,]
  class(res) <- unique(c("kmattributionfunction", class(res)))
  kmclosure(res)
}

#' Convert a binary matrix to a \code{kmfamset} object (family of sets)
#'
#' \code{kmfamset} returns a \code{kmfamset} object after checking that
#' the passed object is a binary matrix with all different rows. If the
#' passe object inherits the \code{kmfamset} property, nothing else is
#' changed.
#'
#' @param x Binary matrix representing a family of sets
#' @return \code{kmfamset} object
#'
#' @examples
#' m <- as.matrix(c(1,0,0,0,1,0,1,1,1), nrow=3, byrow=TRUE)
#' kmfamset(m)
#'
#' @family Constructors
#'
#' @export
kmfamset <- function(x) {
  if (!inherits(x, "matrix")) {
    stop(sprintf("%s must be of class %s.", dQuote("x"), dQuote("matrix")))
  }
  if (any(x != 1*as.logical(x))) {
    stop(sprintf("%s must be a binary matrix.", dQuote("x")))
  }
  res <- unique(x, MARGIN = 1)
  class(res) <- unique(c('kmfamset', class(x)))
  res
}

#' Compute the "intersection" of two KST objects
#'
#' Both objects must be of the same kstMatrix class, the result will be
#' of the same class, i.e. a respective closure operator will be executed
#' automatically.
#'
#' @param x First KST object
#' @param y Second KST object
#' @returns _Intersection_ of \code{x} and \code{y}
#'
#' @useDynLib kstMatrix
#'
#' @family Functions for working with knowledge structures
#'
#' @export
kmintersection <- function(x, y) {
  UseMethod("kmintersection")
}

#' @rdname kmunion
#'
#' @method kmunion kmfamset
#' @family Functions for working with knowledge structures
#' @export
kmunion.kmfamset <- function(x, y) {
  if (!(inherits(y, "kmfamset")))
    stop("'y' must be a kmfamset object.")
  noix <- dim(x)[2]
  noiy <- dim(y)[2]
  if (noix != noiy)
    stop("Objects do not fit in number of items.")
  if (is.null(colnames(x))) {
    if (!(is.null(colnames(y))))
      stop("Item names do not fit.")
  } else if (is.null(colnames(y)))
    stop("Item names do not fit.")
  else if (any(colnames(x) != colnames(y)))
    stop("Item names do not fit")
  res <- kmfamset(rbind(x,y))
  res
}

#' Validate a knowledge structure against a data set
#'
#' \code{kmvalidate} returns a list with three elements,
#' a named vector (dist) with the frequencies of distances
#' between a set of response patterns and a knowledge structure, the
#'  Discrepancy Index (DI), and the Distance Agreement Coefficient (DA).
#'
#' @param data Binary matrix representing a set of response patterns
#' @param struct Binary matrix representing a knowledge structure
#' @return A list with three elements:
#' \describe{
#'   \item{dist}{Distance distribution vector}
#'   \item{DI}{Discrepancy Index}
#'   \item{DA}{Distance Agreement Coefficient}
#' }
#'
#' @section Warning:
#' The DA computation can take quite some time for larger item sets as the
#' power set has to be computed. For item sets with around 30 items or more,
#' it may even crash the system due to huge memory requests.
#'
#' @examples
#' kmvalidate(xpl$data, xpl$space)
#'
#' @family Functions for working with data
#'
#' @export
kmvalidate <- function(data, struct) {
  if (dim(data)[2] != dim(struct)[2])
    stop("data and struct do not match in item number!")
  distvector <- kmdist(data, struct)
  ddat <- sum(as.integer(names(distvector)) * distvector) / sum(distvector)
  di <- kmdist(kmmaximalspace(dim(struct)[2]), struct)
  dpot <- sum(as.integer(names(di))*di)/sum(di)
  list("dist"=distvector, "DI"=ddat, "DA"=ddat/dpot)
}

#' Determine next question for probabilistic knowledge assessment
#'
#' \code{kmassessinformative} determines the next question in a probabiliststic
#' assessment according to the informative rule.
#'
#' @param probs Probability distribution over the knowledge structure (vector)
#' @param ks Binary matrix of the knowledge structure
#' @param update Update rule ("Bayesian" or "multiplicative")
#' @param beta Careless error probabilities (vector)
#' @param eta Lucky guess probabilities (vector)
#' @param zeta0 Vector of update parameters for wrong responses
#' @param zeta1 Vector of update parameters for correct responses

#' @return Number of the selected question
#'
#' @examples
#' kmassessinformative(c(0.02, 0.1, 0.07, 0.01, 0.4, 0.17, 0.07, 0.08, 0.08),
#'                   xpl$space,
#'                   "Bayesian",
#'                   rep(0.3,4),
#'                   rep(0.2,4),
#'                   NULL,
#'                   NULL
#'                  )
#'
#' @family Knowledge assessment functions
#'
#' @export
kmassessinformative <- function(probs, ks, update, beta, eta, zeta0, zeta1) {
  storage.mode(ks) <- "integer"
  noi <- dim(ks)[2]

  if ((min(ks) < 0) | (max(ks) > 1))
    stop("Knowledge structure must be a binary matrix.")
  if ((min(probs) < 0) | (max(probs) > 1))
    stop("State probabilities must be between 0 and 1.")
  if (!(kmdoubleequal(sum(probs), 1.0)))
    stop("State probabilities must sum up to 1.")
  if (update == "Bayesian") {
    multiplicative <- FALSE
    if (is.null(beta) | is.null(eta))
      stop("beta and eta must be set!")
    if (length(beta) != noi)
      stop("beta and pks do not fit in size.")
    if ((min(beta) < 0) | (max(beta) > 1))
      stop("Illegal beta values.")
    if (length(eta) != noi)
      stop("eta and pks do not fit in size.")
    if ((min(eta) < 0) | (max(eta) > 1))
      stop("Illegal eta values.")
    if (!(is.null(zeta0)) | !(is.null(zeta1)))
      warning("Ignoring zeta values for Bayesian update rule")
  } else {
    multiplicative <- TRUE
    if (is.null(zeta0) | is.null((zeta1)))
      stop("zeeta0 and zeta1 must be set.")
    if (length(zeta0) != noi)
      stop("zeta0 and pks do not fit in size")
    if (length(zeta1) != noi)
      stop("zeta1 and pks do not fit in size")
    if ((min(zeta0) <= 1) | (min(zeta1) <= 1))
      stop("zeta0 and zeta1 must be larger than 1")
    if (!(is.null(beta)) | !(is.null(eta)))
      warning("Ignoring beta and eta values for multiplicative update rule")
  }

  H1 <- sapply((1:noi), function(q) {
    if (update == "Bayesian")
      ph <- kmassessbayesian(probs, ks, beta, eta, q, 1)
    else
      ph <- kmassessmultiplicative(probs, ks, zeta0, zeta1, q, 1)
    (-sum(ph*log2(ph)))
  })
  p1 <- sapply((1:noi), function(q) {
    sum(probs[which(ks[,q]==1)])
  })
  H0 <- sapply((1:noi), function(q) {
    if (update == "Bayesian")
      ph <- kmassessbayesian(probs, ks, beta, eta, q, 0)
    else
      ph <- kmassessmultiplicative(probs, ks, zeta0, zeta1, q, 0)
    (-sum(ph*log2(ph)))
  })
  p0 <- sapply((1:noi), function(q) {
    sum(probs[which(ks[,q]==0)])
  })
  hd <- p1*H1 + p0*H0
  hs <- which(hd == min(hd))
  unlist(sample(list(hs), 1))
}

#' Test if a state is contained in a family of states
#'
#' @param s State
#' @param f Family of sets
#' @return Boolean is s is contained in f
#'
#' @examples
#' kmsetiselement(c(1,1,1,0), xpl$space)
#'
#' @family Utilities
#'
#'
#'@export
kmsetiselement <- function(s, f) {
  if (!inherits(f, "matrix")) stop("'f' must be a binary matrix.")
  l <- apply(f, 1, function(x) all(x==s))
  any(l==TRUE)
}

#' Simulate assessments for a set of response patterns
#'
#' \code{kmassessmentsimulation} does a probabilistic knowledge assessment
#' for each response pattern in a data matrix and stores information about
#' the assessment.
#'
#' @param respdata Data matrix
#' @param ks Knowledge structure
#' @param questioning Question rule
#' @param update Updating rule
#' @param beta Careless error probability
#' @param eta Lucky guess probability
#' @param zeta0 Update parameter for wrong responses
#' @param zeta1 Update parameter for correct responses
#' @param threshold Stopping criterion
#' @return Assessment data as data frame
#'
#' @details
#' \code{kmassessmentsimulation} applies the \code{kmsassess} function.
#'
#' @examples
#' kmassessmentsimulation(
#'   xpl$data,
#'   xpl$space,
#'   "halfsplit",
#'   "multiplicative",
#'   NULL,
#'   NULL,
#'   5,
#'   5,
#'   0.55
#' )
#'
#' @family Knowledge assessment functions
#'
#' @importFrom tidyr unnest
#'
#' @export
kmassessmentsimulation <- function(respdata,
                                   ks,
                                   questioning,
                                   update,
                                   beta,
                                   eta,
                                   zeta0,
                                   zeta1,
                                   threshold) {
  columnnames <- c(
    "No items",
    "No States",
    "Questioning",
    "Update",
    "Beta",
    "Eta",
    "Zeta_0",
    "Zeta_1",
    "Threshold",
    "Pattern no.",
    "Assessment error",
    "Distance",
    "Net assessm. error",
    "Qestions asked",
    "Computation time",
    "Avg. questioning time",
    "Avg. updating time",
    "State",
    "Pattern"
  )
  cnt <- 0
  ars <- apply(respdata, 1, function(rp) {
    cnt <<- cnt+1
    # print(cnt)
    # print(rp)
    dist <- min(unlist(apply(ks, 1, function(state) {kmsetdistance(rp, state)})))
    t <- system.time(ar <- kmsassess(rp, ks, questioning, update, beta, eta, zeta0, zeta1, threshold))
    if (!is.null(ar)) {
      list(dim(ks)[2], dim(ks)[1], questioning, update, beta, eta, zeta0, zeta1, threshold, cnt,
           kmsetdistance(rp, ar$state), dist, kmsetdistance(rp, ar$state) - dist, length(ar$queried), t[[3]],
           ar$qtime, ar$utime, paste(ar$state, collapse = ", "), paste(rp, collapse = ", ")
      )
    } else {
      warning(sprintf("NULL assessment result for pattern # %s.", cnt))
      NULL
    }
  })
  # print(length(ars))
  arsdf <- unnest(as.data.frame(do.call(rbind, ars)), 1:19)
  colnames(arsdf) <- columnnames
  arsdf
}

#' Determine the closure of KST objects
#'
#' \code{kmclosure()} is an S3 method closing \code{kstMatrix} objects.
#'
#' @param x KST object
#' @param ... Optional additional parameters
#' @return Corresponding closed KST object
#'
#' An attribution relation or
#' attribution function is closed to the corresponding surmise relation
#' or surmise function, respectively.
#'
#' A family of sets (including bases, knowledge structure, and knowledge space)
#' is closed to a knowledge space or quasi-ordinal knowledge space.
#'
#' @family Closure operators
#'
#' @export
kmclosure <- function(x, ...) {
  UseMethod("kmclosure")
}

#' Compute the surmise relation of a quasi-ordinal knowledge space
#'
#' \code{kmsurmiserelation} returns a matrix representing the
#' surmise relation of a quasi-ordinal knowledge space. If \code{x}
#' is a general knowledge space, a knowledge structure or an
#' arbitrary family of sets, \code{kmsurmiserelation} returns the
#' surmise relation of the smallest quasi-ordinal knowledge space
#' containing \code{x}.
#'
#' @param x Binary matrix representing a family of sets (class kmfamset)
#' @return Binary matrix representing the surmise relation
#'
#' Note: The columns of the surmise relation matrix describe the
#' minimal state for the respective item in the quasi-ordinal
#' knowledge space.
#'
#' @examples
#' kmsurmiserelation(xpl$space)
#'
#' @family Different representations for knowledge structures
#'
#' @export
kmsurmiserelation <- function(x) {
  if (!inherits(x, "kmfamset")) {
    stop(sprintf("%s must be of class %s.", dQuote("x"), dQuote("kmfamset")))
  }
  if (any(x != 1L*as.logical(x))) {
    stop(sprintf("%s must be a binary matrix.", dQuote("x")))
  }

  noi <- dim(x)[2]
  sr <- matrix(rep(0, noi*noi), nrow = noi, ncol = noi)
  lapply(as.list(1:dim(x)[2]), function(i) {
    lapply(as.list(1:dim(x)[2]), function(j) {
      if (all(x[,i] <= x[,j])) {
        sr[j,i] <<- 1
      }
    })
  })
  # for (i in 1:noi) {
  #   for (j in 1:noi) {
  #     if (all(x[,i] <= x[,j])) {
  #       sr[j,i] <- 1
  #     }
  #   }
  # }
  storage.mode(sr) <- "integer"
  rownames(sr) <- colnames(x)
  colnames(sr) <- colnames(x)
  class(sr) <- unique(c("kmsurmiserelation", "kmattributionrelation", class(sr)))
  sr
}

#' Determine the basis for a surmise function
#'
#' \code{kmbasis.kmsurmisefunction} takes a surmise function and returns
#' the corresponding basis, i.e. effectively the collection of all clauses
#' in the surmise function.
#'
#' @param x Surmise Function
#'
#' @examples
#' kmbasis(xpl$sf)
#'
#'
#' @method kmbasis kmsurmisefunction
#' @family Different representations for knowledge structures
#' @rdname kmbasis
#' @export
kmbasis.kmsurmisefunction <- function(x) {
  noi <- ncol(x) - 1
  mat <- as.matrix(x[,(2:(noi+1))])
  storage.mode(mat) <- "integer"
  mat <- unique(mat, MARGIN=1)
  class(mat) <- unique(c("kmbasis", "kmfamset", class(mat)))
  mat
}

#' Determine the basis of a knowledge space from a surmise relation
#'
#' \code{kmSR2basis} takes a surmise relation and returns the
#' corresponding basis.
#'
#' @param sr Surmise relation
#' @return Basis
#'
#' @family Different representations for knowledge spaces
#'
#' @export
kmSR2basis <- function(sr) {
  mat <- unique(t(sr))
  class(mat) <- unique(c("kmbasis", "kmfamset", class(mat)))
  mat
}

concordance <- function(data, sr) {
  colnames(data) <- colnames(sr)
  sum(sapply(1:dim(sr)[1], function(x) {
    sum(sapply(1:dim(sr)[1], function(y) {
      if (data[x,y] == 1) {
        ind <- which(data[,x]==1 & data[,y]==0)
        length(ind)
      } else {
        0
      }
    }))
  }))
}

discordance <- function(data, sr) {
  colnames(data) <- colnames(sr)
  sum(sapply(1:dim(sr)[1], function(x) {
    sum(sapply(1:dim(sr)[1], function(y) {
      if (data[x,y] == 1) {
        ind <- which(data[,x]==0 & data[,y]==1)
        length(ind)
      } else {
        0
      }
    }))
  }))
}


#' Validate a surmise relation against a data set
#'
#' \code{kmSRvalidate} returns a list with two elements,
#' Goodman & Kruskal's gamma value and the violational
#' coefficient (VC).
#'
#' @param data Binary matrix representing a set of response patterns
#' @param sr Binary matrix representing a surmise relation
#' @return A list with two elements:
#' \describe{
#'   \item{gamma}{Goodman & Kruskal's gamma index}
#'   \item{vc}{Violational Coefficient}
#' }
#'
#' @examples
#' kmSRvalidate(xpl$data, xpl$sr)
#'
#' @family Functions for working with data
#'
#' @export

kmSRvalidate <- function(data, sr) {
  nc <- concordance(data, sr)
  nd <- discordance(data, sr)
  gamma <- (nc - nd) / (nc + nd)

  vc = nd / (dim(data)[1] * (sum(sr) - dim(data)[2]))

  list(gamma=gamma, vc=vc)
}

#' Compute the intersection of two quasi--ordinal knowledge spaces
#'
#' @family Functions for working with knowledge structures
#' @method kmintersection kmqspace
#' @rdname kmintersection
#' @export
kmintersection.kmqspace <- function(x, y) {
  if (!(inherits(y, "kmspace")))
    stop("'y' must also be a knowledge space.")
  srx <- kmsurmiserelation(x)
  sry <- kmsurmiserelation(y)
  sri <- kmunion(sry, sry)
  kmspace(sri)
  class(sri) <- unique(c("kmqspace", class(sri)))
  sri
}

#' Determine the knowledge space delineated by a surmise function
#'
#' @family Constructors
#' @family Different representations for knowledge structures
#' @method kmspace kmsurmisefunction
#' @rdname kmspace
#' @export
kmspace.kmsurmisefunction <- function(x)  {
  b <- kmbasis(x)
  kmunionclosure(b)
}

#' Knowledge spaces on AutoCAD knowledge
#'
#' Bases of knowledge spaces on AutoCAD knowledge obtained from querying experts.
#'
#' Six experts were queried about prerequisite relationships between 28 AutoCAD
#' knowledge items (Dowling, 1991; 1993). A seventh basis represents those
#' prerequisite relationships on which the majority (4 out of 6) of the experts
#' agree (Dowling & Hockemeyer, 1998).
#'
#' @docType data
#'
#' @usage cad
#'
#' @format A list containing seven bases (cad1 to cad6, and cadmaj) in binary
#' matrix form. Each matrix has 28 columns representing the different knowledge
#' items and a varying number of rows containing the basis elements.
#'
#' @references Dowling, C. E. (1991). _Constructing Knowledge Structures from
#' the Judgements of Experts._ Habilitationsschrift, Technische Universität
#' Carolo-Wilhelmina, Braunschweig, Germany.
#'
#' Dowling, C. E. (1993). Applying the basis of a knowledge space for controlling
#' the questioning of an expert. _Journal of Mathematical Psychology, 37,_ 21–48.
#'
#' Dowling, C. E. & Hockemeyer, C. (1998). Computing the intersection of knowledge
#' spaces using only their basis. In Cornelia E. Dowling, Fred S. Roberts, & Peter
#' Theuns, editors, _Recent Progress in Mathematical Psychology,_ pp. 133–141.
#' Lawrence Erlbaum Associates Ltd., Mahwah, NJ.
#'
"cad"

#' Close a knowledge space under intersection
#'
#' @param x Knowledge space
#' @returns Quasi--ordinal knowledge space
#'
#' @examples
#' kmintersectionclosure(xpl$space)
#'
#' @family Closure operators
#'
#' @export
kmintersectionclosure <- function(x) {
  if (!(inherits(x, "kmspace")))
    stop("'x' must be a 'kmspace' object.")
  kmqspace(x)
}

#' \code{kmspace} constructor method
#'
#' \code{kmspace()} determines the knowledge space corresponding to a KST object
#'
#' @param x KST object
#' @return Corresponding knowledge space
#'
#' @examples
#' kmspace(xpl$basis)
#' kmspace(xpl$sf)
#'
#' @family Constructors
#' @export
kmspace <- function(x) {
  UseMethod("kmspace")
}

#' @rdname kmunionclosure
#' @export
kmunionclosure.kmfamset <- function(x) {
  if (any(x != 1*as.logical(x))) {
    stop(sprintf("%s must be a binary matrix.", dQuote("x")))
  }

  x <- kmbasis(x)

  noi <- as.integer(dim(x)[2])
  nob <- as.integer(dim(x)[1])
  storage.mode(x) <- "integer"
  nos <- as.integer(0)
  result <- .C("constr", noi, nob, t(x), nos, package="kstMatrix")
  nos <- result[[4]][1]
  space <- matrix(1:(noi*nos), ncol=noi, nrow=nos, byrow = TRUE)
  storage.mode(space) <- "integer"
  result2 <- .C("constr_results", space, package="kstMatrix")
  s <- result2[[1]]
  colnames(s) <- colnames(x)
  class(s) <- unique(c("kmspace", "kmstructure", "kmfamset", class(s)))
  s
}

#' Close a family of sets under union, i.e. determine the smallest knowledge space containing it
#'
#' @param x Family of set
#' @returns Knowledge space
#'
#' \code{kmunionclosure} implements the irredundant algorithm developed by Dowling (1993).
#'
#' @references Dowling, C. E. (1993). On the irredundant construction of knowledge spaces.
#' _Journal of Mathematical Psychology, 37,_ 49–62.
#'
#' @examples
#' kmunionclosure(xpl$basis)
#'
#'
#' @family Closure operators
#' @export
kmunionclosure <- function(x) {
  x <- kmbasis(x)

  noi <- as.integer(dim(x)[2])
  nob <- as.integer(dim(x)[1])
  storage.mode(x) <- "integer"
  nos <- as.integer(0)
  result <- .C("constr", noi, nob, t(x), nos, package="kstMatrix")
  nos <- result[[4]][1]
  space <- matrix(1:(noi*nos), ncol=noi, nrow=nos, byrow = TRUE)
  storage.mode(space) <- "integer"
  result2 <- .C("constr_results", space, package="kstMatrix")
  s <- result2[[1]]
  colnames(s) <- colnames(x)
  class(s) <- unique(c("kmspace", "kmstructure", "kmfamset", class(s)))
  s
}

#' Compute the neighbourhod of a state within a knowledge structure
#'
#' \code{kmneighbourhood} computes the neighbourhood of a state
#' within a knowledge structure, i.e. the family of all other
#' states with a symmetric set difference of 1.
#'
#' @param state Binary vector representing a knowledge state
#' @param struct Binary matrix representing a knowledge structure
#' @param include Boolean whether the original state should be included
#' in the result (default FALSE)
#' @return Matrix containing the neighbouring states, one per row
#'
#' @examples
#' kmneighbourhood(c(1,1,0,0), xpl$space)
#'
#' @family Functions for fringes & neighbourhoods
#'
#' @export
kmneighbourhood <- function(state, struct, include = FALSE) {
  if (!inherits(struct, "matrix")) {
    stop(sprintf("%s must be of class %s.", dQuote("struct"), dQuote("matrix")))
  }
  if (any(struct != 1*as.logical(struct))) {
    stop(sprintf("%s must be a binary matrix.", dQuote("struct")))
  }
  if (length(state) != dim(struct)[2]) {
    stop(sprintf("%s and %s don't match in size.", dQuote("state"), dQuote("struct")))
  }
  n <- matrix(
    unlist(apply(struct, 1, function(x) {
      if (kmsetdistance(x, state) == 1)
        return(x)
      else
        return(NULL)
    })),
    ncol = length(state),
    byrow = TRUE
  )
  if (include) n <- rbind(n, state)
  colnames(n) <- colnames(struct)
  rownames(n) <- NULL
  class(n) <- unique(c("kmneighbourhood", "kmfamset", class(n)))
  n
}



#' Compute the n-neighbourhod of a state within a knowledge structure
#'
#' \code{kmnneighbourhood} computes the n-neighbourhood of a state
#' within a knowledge structure, i.e. the family of all other
#' states with a symmetric set difference maximal n.
#'
#' @param state Binary vector representing a knowledge state
#' @param struct Binary matrix representing a knowledge structure
#' @param distance Size of the n-neighbourhood
#' @param include Boolean whether the original state should be included
#' (defaul FALSE)
#' @return Matrix containing the neighbouring states, one per row
#'
#' @examples
#' kmnneighbourhood(c(1,1,0,0), xpl$space, 2)
#'
#' @family Functions for fringes & neighbourhoods
#' @rdname kmneighbourhood
#'
#' @export
kmnneighbourhood <- function(state, struct, distance, include=FALSE) {
  if (!inherits(struct, "matrix")) {
    stop(sprintf("%s must be of class %s.", dQuote("struct"), dQuote("matrix")))
  }
  if (any(struct != 1*as.logical(struct))) {
    stop(sprintf("%s must be a binary matrix.", dQuote("struct")))
  }
  if (length(state) != dim(struct)[2]) {
    stop(sprintf("%s and %s don't match in size.", dQuote("state"), dQuote("struct")))
  }
  n <- matrix(
    unlist(apply(struct, 1, function(x) {
      d <- kmsetdistance(x, state)
      if ((d >= 1) && (d <= distance))
        return(x)
      else
        return(NULL)
    })),
    ncol = length(state),
    byrow = TRUE
  )
  if (include) n <- rbind(n, state)
  colnames(n) <- colnames(struct)
  rownames(n) <- NULL
  class(n) <- unique(c("kmneighbourhood", "kmfamset", class(n)))
  n
}

#' Test two double numbers on equity with a certain tolerance
#'
#' @param x First double to compare
#' @param y Second double to compare
#' @param tol Tolerance optional)
#' @return Boolean for (approximate) equity
#'
#' @examples
#' kmdoubleequal(0.5+0.5, 1)
#' 0.5 + 0.5 == 1
#'
#' @family Utilities
#'
#' @export
kmdoubleequal <- function(x, y, tol = sqrt(.Machine$double.eps)){
  abs(x - y) < tol
}

#' Knowledge space on linear functions
#'
#' Basis of a small knowledge space and list of items on linear functions
#' used in a manuscript by Steiner et al.
#'
#' @docType data
#'
#' @usage phsg
#'
#' @format A list containing the basis and the list of items
#'
"phsg"

#' Compute the distance between a data set and a knowledge structure
#'
#' \code{kmdist} returns a named vector with the frequencies of distances
#' between a set of response patterns and a knowledge structure. This vector
#' can be used to compute, e.g., the Discrepancy Index (DI) or the
#' Distance Agreement Coefficient (DA).
#'
#' @param data Binary matrix representing a set of response patterns
#' @param struct Binary matrix representing a knowledge structure
#' @return Distance distribution vector
#'
#' @examples
#' kmdist(xpl$data, xpl$space)
#'
#' @family Functions for working with data
#'
#' @export
kmdist <- function(data, struct) {
  if (!inherits(data, "matrix")) {
    stop(sprintf("%s must be of class %s.", dQuote("data"), dQuote("matrix")))
  }
  if (any(data != 1*as.logical(data))) {
    stop(sprintf("%s must be a binary matrix.", dQuote("data")))
  }
  if (!inherits(struct, "matrix")) {
    stop(sprintf("%s must be of class %s.", dQuote("struct"), dQuote("matrix")))
  }
  if (any(struct != 1*as.logical(struct))) {
    stop(sprintf("%s must be a binary matrix.", dQuote("struct")))
  }
  if (dim(data)[2] != dim(struct)[2]) {
    stop(sprintf("%s and %s hve different item numbers!.", dQuote("data"), dQuote("struct")))
  }

  distvec <- rep(as.integer(0), dim(struct)[2]+1)
  names(distvec) <- 0:dim(struct)[2]
  result <- .C("dist", dim(data)[2], t(data), dim(data)[1], t(struct), dim(struct)[1], distvec, package="kstMatrix")
  distvec <- result[[6]]
  # names(distvec) <- 0:dim(struct)[2]
  distvec
}

#' Determine the notions of a knowledge structure
#'
#' \code{kmnotions} returns a matrix representing the
#' notions of a knowledge structure.
#'
#' @param x Binary matrix representing a knowledge structure
#' @return Binary matrix representing notions in the knowledge structure
#'
#' The matrix has a '1' in row 'i' and column 'j' if 'i' and 'j' belong
#' to the same notion (i.e. are equivalent). It is a symmetric matrix
#' with '1's in the main diagonal.
#'
#' @examples
#' kmnotions(xpl$space)
#'
#' @family Properties of knowledge structures
#' @family Functions for working with knowledge structures
#'
#' @export
kmnotions <- function(x) {
  if (!inherits(x, "matrix")) {
    stop(sprintf("%s must be of class %s.", dQuote("x"), dQuote("matrix")))
  }
  if (any(x != 1*as.logical(x))) {
    stop(sprintf("%s must be a binary matrix.", dQuote("x")))
  }

  noi <- dim(x)[2]
  sr <- matrix(rep(0, noi*noi), nrow = noi, ncol = noi)
  for (i in 1:noi) {
    for (j in 1:i) {
      if (all(x[,i] == x[,j])) {
        sr[j,i] <- 1
        sr[i,j] <- 1
      }
    }
  }
  rownames(sr) <- NULL
  colnames(sr) <- colnames(x)
  red <-unique(sr)
  class(red) <- unique(c("kmfamset", class(red)))
  red
}

#' Determine subfamily of minimal states in a \code{kmfamset}
#'
#' @param x kmfamset
#' @return kmfamset of minimal states
#'
#' \code{kmminimalfamset()} determines the sub family of minimal sets
#' within a famset. If \code{x} is a knowledge structure, the result will
#' be a family containing (only) the empty set because the empty set is a
#' state in any knowledge structure and is its minimal state.
#'
#' The concept of the minimal state of a family of sets should not be confused
#' with the concept of knowledge states minimal for some item.
#'
#' @family Utilities
#'
#' @examples
#' m <- matrix(c(1,0,0,0,1,0,1,0,1), ncol=3, byrow=TRUE)
#' m
#' kmminimalfamset(kmfamset(m))
#'
#'
#' @export
kmminimalfamset <- function(x) {
  if (!(inherits(x, "kmfamset")))
    stop("x must be of class 'kmfamset'.")
  nos <- dim(x)[1]
  if (nos == 1) return(x)
  for (i in (1:(nos-1)))
    for (j in ((i+1):nos)) {
      if (all(x[i,] <= x[j,]))
        return(kmminimalfamset(kmfamset(matrix(x[-j,], ncol=dim(x)[2]))))
      else if (all(x[i,] >= x[j,]))
        return(kmminimalfamset(kmfamset(matrix(x[-i,], ncol=dim(x)[2]))))
    }
  x
}

#' DEPRECATED: \code{kmSR2basis()} has been replaced by \code{kmbasis()}.
#' @param x Surmise relation
#' @return Corresponding basis
#'
#' @family Deprecated functions and methods
#' @export
kmSR2basis <- function(x) {
  .Deprecated("kmbasis")
  kmbasis(x)
}

#' DEPRECATED: \code{kmSF2basis()} has been replaced by \code{kmbasis()}.
#' @param x Surmise function
#' @return Corresponding basis
#'
#' @family Deprecated functions and methods
#' @export
kmSF2basis <- function(x) {
  .Deprecated("kmbasis")
  kmbasis(x)
}

#' DEPRECATED: \code{kmhasse()} has been replaced by a \code{plot()} method.
#' @param x Knowledge structure
#' @param horizontal Boolean whether the diagram should be drawn horizontally or vertically (default TRUE)
#' @param colors Color vector (default NULL)
#'
#' @family Deprecated functions and methods
#' @export
kmhasse <- function(x, horizontal=TRUE, colors=NULL) {
  .Deprecated("plot")
  if (horizontal)
    plot(x, horizontal=horizontal, colors=colors, arrowtail="normal")
  else
    plot(x, horizontal=horizontal, colors=colors)
}

#' DEPRECATED: \code{kmbasisdiagram()} has been replaced by a \code{plot()} method.
#' @param x Basis
#' @param horizontal Boolean whether the diagram should be drawn horizontally or vertically (default TRUE)
#' @param colors Color vector (default NULL)
#'
#' @family Deprecated functions and methods
#' @export
kmbasisdiagram <- function(x, horizontal=TRUE, colors=NULL) {
  .Deprecated("plot")
  if (horizontal)
    plot(x, horizontal=horizontal, colors=colors, arrowtail="normal")
  else
    plot(x, horizontal=horizontal, colors=colors)
}

#' DEPRECATED: \code{kmSRdiagram()} has been replaced by a \code{plot()} method.
#' @param x Surmise relation
#' @param horizontal Boolean whether the diagram should be drawn horizontally or vertically (default FALSE)
#' @param colors Color vector (default NULL)
#'
#' @family Deprecated functions and methods
#' @export
kmSRdiagram <- function(x, horizontal=FALSE, colors=NULL) {
  .Deprecated("plot")
  if (horizontal)
    plot(x, horizontal=horizontal, colors=colors, arrowtail="normal")
  else
    plot(x, horizontal=horizontal, colors=colors)
}

#' Determine the knowledge space delineated by a surmise relation
#'
#' Please note that a knowledge space delineated by a surmise relation is
#' quasi-ordinal. Therefore, the result is automatically marked as
#' \code{kmqspace} object.
#'
#' @family Constructors
#' @family Different representations for knowledge structures
#' @method kmspace kmsurmiserelation
#' @rdname kmspace
#' @export
kmspace.kmsurmiserelation <- function(x) {
  b <- kmbasis(x)
  sp <- kmunionclosure(b)
  class(sp) <- unique(c("kmqspace", class(sp)))
  sp
}

#' Update probability distribution applying Bayesian update
#'
#' \code{kmassessbayesian} updates a probability distribution over a
#' knowledge structure according to the Bayesian update rule.
#'
#' @param probs Probability distribution over the knowledge structure (vector)
#' @param ks Binary matrix of the knowledge structure
#' @param beta Vector of careless error probabilities
#' @param eta Vector of lucky guess probabilities
#' @param question Item that has been posed
#' @param response Correctness of received response (0 or 1)
#' @return Updated probability vector
#'
#' @examples
#' kmassessbayesian(c(0.02, 0.1, 0.07, 0.01, 0.4, 0.17, 0.07, 0.08, 0.08),
#'                  xpl$space,
#'                  rep(0.2,4),
#'                  rep(0.1,4),
#'                  3,
#'                  1
#'                 )
#'
#' @family Knowledge assessment functions
#'
#' @export
kmassessbayesian <- function(probs, ks, beta, eta, question, response) {
  storage.mode(ks) <- "integer"
  nos <- dim(ks)[1]
  noi <- dim(ks)[2]

    if ((min(ks) < 0) | (max(ks) > 1))
    stop("Knowledge structure must be a binary matrix.")
  if ((min(probs) < 0) | (max(probs) > 1))
    stop("State probabilities must be between 0 and 1.")
  if (!(kmdoubleequal(sum(probs), 1.0)))
    stop ("Probabilities do not sum up to 1!")
  if (is.null(beta) | is.null(eta))
    stop("beta and eta must be set!")
  if (length(beta) != noi)
    stop("beta and pks do not fit in size.")
  if ((min(beta) < 0) | (max(beta) > 1))
    stop("Illegal beta values.")
  if (length(eta) != noi)
    stop("eta and pks do not fit in size.")
  if ((min(eta) < 0) | (max(eta) > 1))
    stop("Illegal eta values.")
  if (any(beta+eta > 1))
    warning("beta_q + eta_q should be less than 1 for all items q.")
  if (response == 1) {
    up <- 1 - beta[question]
    um <- eta[question]
  } else {
    up <- beta[question]
    um <- 1 - eta[question]
  }
  phelp <- sapply((1:nos), function(s) {
    if (ks[s, question] == 1) up * probs[s]
    else um * probs[s]
  })
  pnew <- phelp / sum(phelp)
  if (!(kmdoubleequal(sum(pnew), 1.0))) {
    print(pnew)
    print(sum(pnew))
    stop ("Internal error: Probabilities do not sum up to 1!")
  }
  pnew
}

#' Reduce a family of knowledge states with respect to item equivalence
#'
#' \code{kmeqreduction} takes a family of knowledge states and returns its
#' reduction to non-equivalent items.
#'
#' @param x Family of sets (\code{kmfamset} object)
#' @return Binary matrix reduced by equivalences
#'
#' @examples
#' kmeqreduction(xpl$space)
#'
#' @family Functions for working with knowledge structures
#'
#' @export
kmeqreduction <- function(x) {
  if (!inherits(x, "kmfamset")) {
    stop(sprintf("%s must be of class %s.", dQuote("x"), dQuote("kmfamset")))
  }
  if (any(x != 1*as.logical(x))) {
    stop(sprintf("%s must be a binary matrix.", dQuote("x")))
  }

  red <- unique(x, MARGIN = 2)
  class(red) <- unique(c("kmfamset", class(x)), fromLast = TRUE)
  n <- colnames(red)
  notions <- kmnotions(x)
  eq <- which(rowSums(notions) > 1)
  if (length(eq) > 0) {
    sapply(eq, function(cl) {
      first <- min(which(notions[cl,]==1))
      newitem <- kmprettyprint(notions[cl,], open='[', close=']')
      n[first] <<- newitem
    })
    colnames(red) <- n
  }
  red
}

#' Reduce a KST object to a subset of items
#'
#' @param x KST object
#' @param items Selection of items (vector) to which the KST object shall be reduced
#' @returns Sub structure KST object (the kstMatrix class and its properties
#'          are preserved)
#'
#' \code{kmsubstructure()} takes a KST object and determines a sub-structure
#' based on the list of items.
#'
#' The items can be specified through their item names or through the respective
#' column numbers. At least two items must be specified.
#'
#' @examples
#' phsg$basis
#' kmsubstructure(phsg$basis, c("I1", "I2", "I5"))
#'
#' @family Functions for working with knowledge structures
#'
#' @export
kmsubstructure <- function(x, items) {
  UseMethod("kmsubstructure")
}


#' @rdname kmsubstructure
#' @method kmsubstructure kmdata
#' @export
kmsubstructure.kmdata <- function(x, items) {
  if (length(items) < 2)
    stop("The vector of items must contain at least two elements.")
  m <- x[, items]
  class(m) <- unique(c("kmdata", class(m)))
  m
}

#' @rdname kmsubstructure
#' @method kmsubstructure kmbasis
#' @export
kmsubstructure.kmbasis <- function(x, items) {
  if (length(items) < 2)
    stop("The vector of items must contain at least two elements.")
  m <- x[, items]
  class(m) <- unique(c("kmfamset", class(m)))
  kmbasis(m)
}

#' @rdname kmsubstructure
#' @method kmsubstructure kmstructure
#' @export
kmsubstructure.kmstructure <- function(x, items) {
  if (length(items) < 2)
    stop("The vector of items must contain at least two elements.")
  m <- unique(x[, items], MARGIN=1)
  class(m) <- unique(c("kmstructure", "kmfamset", class(m)))
  m
}

#' @rdname kmsubstructure
#' @method kmsubstructure kmspace
#' @export
kmsubstructure.kmspace <- function(x, items) {
  if (length(items) < 2)
    stop("The vector of items must contain at least two elements.")
  m <- unique(x[, items], MARGIN=1)
  class(m) <- unique(c("kmspace", "kmstructure", "kmfamset", class(m)))
  m
}

#' @rdname kmsubstructure
#' @method kmsubstructure kmsurmiserelation
#' @export
kmsubstructure.kmsurmiserelation <- function(x, items) {
  if (length(items) < 2)
    stop("The vector of items must contain at least two elements.")
  m <- x[, items]
  m <- m[items,]
  class(m) <- unique(c("kmsurmiserelation", "kmattributionrelation", class(m)))
  m
}

#' @rdname kmsubstructure
#' @method kmsubstructure kmsurmisefunction
#' @export
kmsubstructure.kmsurmisefunction <- function(x, items) {
  if (length(items) < 2)
    stop("The vector of items must contain at least two elements.")
  if (is.numeric(items))
    items = colnames(x)[c(1, items+1)]
  else {
    items <- c(colnames(x)[1], items)
  }
  df <- unique(x[, items], MARGIN=1)
  l <- which(df[,1] %in% items)
  df <- df[l,]
  class(df) <- unique(c("kmattributionfunction", class(df)))
  # kmclosure(df)
  df
}

#' Compute the neighbourhod of a state within a knowledge structure using its basis
#'
#' \code{kmbasisneighbourhood} computes the neighbourhood of a state
#' within a knowledge structure, i.e. the family of all other
#' states with a symmetric set difference of 1.
#'
#' @param state Binary vector representing a knowledge state
#' @param basis \code{kmbasis} object
#' @param include Boolean whether the original state should be included
#' in the result (default FALSE)
#' @return Matrix containing the neighbouring states, one per row
#'
#' @examples
#' kmbasisneighbourhood(c(1,1,0,0), xpl$basis)
#'
#' @family Functions for fringes & neighbourhoods
#'
#' @export
kmbasisneighbourhood <- function(state, basis, include = FALSE) {
  noi <- length(state)
  nob <- dim(basis)[1]
  if (!inherits(basis, "kmbasis"))
    stop("Basis must be a kmbasis object")
  if (noi != dim(basis)[2])
    stop("State and basis do not match in size")

  inner <- kmbasisinnerfringe(state, basis)
  outer <- kmbasisouterfringe(state, basis)
  non <- sum(inner) + sum(outer)
  if (include)
    non <- non + 1
  names(state) <- colnames(basis)
  if (include)
    m <- matrix(state, nrow=1)
  else
    m <- NULL
  sapply(which(inner==1), function(item) {
    h <- state
    h[item] <- 0
    if (is.null(m))
      m <<- matrix(h, nrow=1)
    else
      m <<- rbind(m, h)
  })
  sapply(which(outer==1), function(item) {
    h <- state
    h[item] <- 1
    if (is.null(m))
      m <<- matrix(h, nrow=1)
    else
      m <<- rbind(m, h)
  })
  colnames(m) <- colnames(basis)
  rownames(m) <- NULL
  class(m) <- unique(c("kmneighbourhood", "kmfringe", class(m)))
  m
}

#' Compute the surmise function for a knowledge space or basis
#'
#' \code{kmsurmisefunction} returns a data frame representing the
#' surmise function for a knowledge space or basis. The rows of the
#' data frame are ordered by item name.
#'
#' @param x Binary matrix representing a knowledge space or basis
#' @param itemname Name for the first column of the resulting data frame
#' @return Data frame  representing the surmise unction of \code{x}.
#'
#' @examples
#' kmsurmisefunction(xpl$space)
#'
#' @family Different representations for knowledge structures
#'
#' @importFrom sets as.set set_is_subset set %e%
#' @importFrom pks as.pattern as.binmat
#'
#' @export
kmsurmisefunction <- function(x, itemname="Item") {
  if (!inherits(x, "kmfamset")) {
    stop(sprintf("%s must be of class %s.", dQuote("x"), dQuote("kmfamset")))
  }
  if (any(x != 1*as.logical(x))) {
    stop(sprintf("%s must be a binary matrix.", dQuote("x")))
  }

  if (dim(x)[1] == 1) {
    return(NULL)
  }
  if (is.null(colnames(x))) {
    warning("Empty column names - numbering the columns.")
    colnames(x) <- as.character(c(1:(dim(x)[2])))
  }

  noi <- as.integer(dim(x)[2])
  nos <- as.integer(dim(x)[1])
  storage.mode(x) <- "integer"
  nob <- as.integer(0)

  result <- .C("basis_reduction", noi, nos, t(x), nob, package="kstMatrix")
  nob <- result[[4]][1]

  basis <- matrix(1:(noi*nob), ncol=noi, nrow=nob, byrow = TRUE)
  storage.mode(basis) <- "integer"
  mins <- matrix(1:(noi*nob), ncol=noi, nrow=nob, byrow = TRUE)
  storage.mode(mins) <- "integer"
  result2 <- .C("sf_results", basis, mins, package="kstMatrix")
  b <- matrix(result2[[1]], ncol=noi, nrow=nob, byrow=TRUE)
  colnames(b) <- colnames(x)
  m <- matrix(result2[[2]], ncol=noi, nrow=nob, byrow=TRUE)
  colnames(m) <- colnames(x)

  df <- data.frame(NULL)
  sapply(colnames(x), function(item) {
    hm <- matrix(b[m[,item]==1,], ncol=noi, byrow=FALSE)
    hr <- dim(hm)[1]
    hdf <- data.frame(cbind(t(t(rep(item, hr))), hm))
    df <<- rbind(df, hdf)
  })
  colnames(df) <- c(itemname, colnames(x))
  class(df) <- unique(c("kmsurmisefunction", "kmattributionfunction", class(df)))
  df
}

#' Simulate a set of response patterns according to the BLIM
#'
#' \code{kmsimulate} returns a data set of \code{n} simulated response patterns based
#' on the knowledge structure \code{x} given as a binary matrix. The simulation follows
#' the BLIM (Basic Local Independence Model; see Doigon & Falmagne, 1999).
#'
#' The \code{beta} and \code{eta} parameters must be either single numericals
#' or vectors with a length identical to the number of rows in the \code{x} matrix.
#' A mixture is possible.
#'
#' The `sample` function used by `kmsimulate` might work inaccurately for knowledge
#' structures `x` with 2^31 or more states.
#'
#' @param x Binary matrix representing a knowledge space
#' @param n Number of simulated response patterns
#' @param beta Careless error probability value or vector
#' @param eta Lucky guess probability value or vector
#' @return Binary matrix representing the simulated data set
#'
#' @references Doignon, J.-P. & Falmagne, J.-C. (1999). _Knowledge Spaces._
#' Springer–Verlag, Berlin.
#'
#' @examples
#' kmsimulate(xpl$space, 50, 0.2, 0.1)
#' kmsimulate(xpl$space, 50, c(0.2, 0.25, 0.15, 0.2), c(0.1, 0.15, 0.05, 0.1))
#' kmsimulate(xpl$space, 50, c(0.2, 0.25, 0.15, 0.2), 0)
#'
#' @family Functions for working with data
#'
#' @importFrom stats runif
#' @export
kmsimulate <- function(x, n, beta, eta) {
  if (!inherits(x, "matrix")) {
    stop(sprintf("%s must be of class %s.", dQuote("x"), dQuote("matrix")))
  }
  if (any(x != 1*as.logical(x))) {
    stop(sprintf("%s must be a binary matrix.", dQuote("x")))
  }

  noi <- dim(x)[2]

  if ((length(beta) != 1) && (length(beta) != noi)) {
    stop(sprintf("%s and %s don't match in size.", dQuote("x"), dQuote("beta")))
  }
  if ((length(eta) != 1) && (length(eta) != noi)) {
    stop(sprintf("%s and %s don't match in size.", dQuote("x"), dQuote("eta")))
  }
  if ((min(beta) < 0) || (max(beta) > 1)) {
    stop(sprintf("%s must be between 0 and 1.", dQuote("beta")))
  }
  if ((min(eta) < 0) || (max(eta) > 1)) {
    stop(sprintf("%s must be between 0 and 1.", dQuote("eta")))
  }

  sam <- sample.int(dim(x)[1], n, replace = TRUE)
  px <- t(sapply(sam, function(y) {
    1*(stats::runif(noi) <= (x[y,]*(1-beta) + ((1-x[y,])*eta)))
  }))
  colnames(px) <- colnames(x)
  class(px) <- unique(c("kmdata", class(px)))
  px

}

#' Obtain a quasi-ordinal knowledge space
#'
#' \code{kmqspace()} computes the  quasi-ordinal space
#' corresponding to a given KST object.
#'
#' @param x KST object
#' @return Corresponding \code{kmqspace} object
#'
#' @examples
#' m <- kmfamset(matrix(c(1,0,0,0,1,0,1,1,1), nrow=3, byrow=TRUE))
#' kmqspace(m)
#'
#' @family Constructors
#'
#' @export
kmqspace <- function(x) {
  UseMethod("kmqspace")
}

#' Determine all gradations between two states
#'
#' @param structure Knowledge structure
#' @param from Starting state (if NULL (default), it is the empty set)
#' @param to Goal state (if NULL (default), it is the full item set)
#' @return A list of gradations where each gradation is a list of states
#'
#' @examples
#' kmgradations(xpl$space)
#'
#'
#' @family Functions for wellgradedness & learning paths
#'
#' @export
kmgradations <- function(structure,
                            from = NULL,
                            to = NULL
) {
  if (!inherits(structure, "kmstructure"))
    stop("'structure' must be a kmstructure.")
  noi <- dim(structure)[2]
  if (is.null(from))
    from <- rep(0, noi)
  if (is.null(to))
    to <- rep(1, noi)
  if (!kmsetiselement(from, structure)) stop("'from' state must be an element of the structure.")
  if (!kmsetiselement(to, structure)) stop("'to' state must be an element of the structure.")
  if (all(from == to)) return(list(list(from)))
  lp <- list()
  apply(kmneighbourhood(from, structure), MARGIN=1, FUN=function(on) {
    if ((all((from & on) == from)) && (all((on & to) == on))) {
      lp2 <- kmgradations(structure, from=on, to=to)
      if (length(lp2) > 0) {
        lp3 <- lapply(lp2, function(p) {
          append(p, list(from), after=0)
        })
        lp <<- append(lp, lp3)
      }
    }
  })
  lp
}

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(kstMatrix)
library(grDevices)
library(sets)
library(pks)
library(Rgraphviz)

## ----classes, fig.cap = 'Figure 1: kstMatrix object classes', echo = FALSE, fig.align = 'center', cache=FALSE, out.width='95%'----
knitr::include_graphics("kstMatrix-Classes.png")

## ----eval=FALSE---------------------------------------------------------------
# plot(xpl$space)
# plot(xpl$sr)

## ----fig.cap = 'Figure 2: Example space', echo = FALSE, fig.align = 'center', cache=FALSE, out.width='450'----
knitr::include_graphics("space.jpeg")

## ----fig.cap = 'Figure 3: Example surmise relation', echo = FALSE, fig.align = 'center', cache=FALSE, out.width='200'----
knitr::include_graphics("SR.jpeg")

## ----eval=FALSE---------------------------------------------------------------
# plot(xpl$sr, colors="orange", vertexshape="circle")

## ----eval=FALSE---------------------------------------------------------------
# plot(kmneighbourhood(c(1,1,0,0), xpl$space, include=TRUE),
#      state=c(1,1,0,0), edgelabel=TRUE)

## ----neighbourhood, fig.cap = 'Figure 4: Neighbourhood', echo = FALSE, fig.align = 'center', cache=FALSE, out.width='400'----
knitr::include_graphics("neighbourhood.jpeg")

## ----eval=FALSE---------------------------------------------------------------
# plot(xpl$space, state=c(1,1,0,0))

## ----neighbourhood_space, fig.cap = 'Figure 5: Neighbourhood within a knowledge space', echo = FALSE, fig.align = 'center', cache=FALSE, out.width='650'----
knitr::include_graphics("neighbourhood2.png")

## ----eval=FALSE---------------------------------------------------------------
# plot(xpl$space, horizontal=TRUE, arrowtail="open")
# plot(xpl$space, arrowhead="empty")

## ----horizontal, fig.cap = 'Figure 6: Horizontal Hasse diagram', echo = FALSE, fig.align = 'center', cache=FALSE, out.width='800'----
knitr::include_graphics("horizontal.jpeg")

## ----emptyhead, fig.cap = 'Figure 7: Using arrows instead of lines', echo = FALSE, fig.align = 'center', cache=FALSE, out.width='400'----
knitr::include_graphics("emptyhead.jpeg")

## ----prettyprinting-----------------------------------------------------------
kmprettyprint(xpl$basis)
kmprettyprint(xpl$basis, simplified=TRUE)
kmprettyprint(xpl$sr,tuple=FALSE)
cat(kmprettyprint(kmlearningpaths(xpl$space)))

#' Determine the smallest knowledge space containing a family of sets/states.
#'
#' @rdname kmspace
#' @family Constructors
#' @method kmspace kmfamset
#' @export
kmspace.kmfamset <- function(x) {
  kmunionclosure(x)
}

#' Determine the "intersection" of two surmise functions.
#'
#' @examples
#' sf <- kmsurmisefunction(kmbasis(xpl$sr))
#' kmintersection(xpl$sf, sf)
#'
#' @family Functions for working with knowledge structures
#' @method kmintersection kmsurmisefunction
#' @rdname kmintersection
#' @export
kmintersection.kmsurmisefunction <- function(x,y) {
  if (!(inherits(y, "kmsurmisefunction")))
    stop("'y' must also be a surmise function.")
  basx <- kmbasis(x)
  basy <- kmbasis(y)
  basu <- kmunion(basx, basy)
  kmsurmisefunction(basu)
}

#' Close a family of sets to a quasi-ordinal knowledge space
#'
#' \code{kmqspace()} computes the smallest quasi-ordinal space
#' containing a given family of sets.
#'
#' @examples
#' m <- kmfamset(matrix(c(1,0,0,0,1,0,1,1,1), nrow=3, byrow=TRUE))
#' kmqspace(m)
#'
#' @family Constructors
#' @rdname kmqspace
#' @method kmqspace kmfamset
#'
#' @export
kmqspace.kmfamset <- function(x) {
  f <- kmfamset(x)
  sr <- kmsurmiserelation(f)
  b <- kmbasis(sr)
  sp <- kmspace(b)
  class(sp) <- unique(c("kmqspace", class(sp)))
  sp
}

#' Compute the fringe of a state within a knowledge structure using its basis
#'
#' \code{kmbasisfringe} computes the fringe of a state
#' within a knowledge structure, i.e. the set of items by which
#' the state differs from its neighbours.
#'
#' @param state Binary vector representing a knowledge state
#' @param basis \code{kmbasis} object
#' @return Binary vector representing the fringe
#'
#' @references Hockemeyer C (1997). Using the Basis of a Knowledge Space for
#' Determining the Fringe of a Knowledge State.
#' _Journal of Mathematical Psychology, 41,_ 275–279.
#'
#' @examples
#' kmbasisfringe(c(1,0,0,0), xpl$basis)
#'
#' @family Functions for fringes & neighbourhoods
#'
#' @export
kmbasisfringe <- function(state, basis) {
  as.integer(kmbasisinnerfringe(state, basis) |
               kmbasisouterfringe(state, basis))
}


#' @rdname kmbasisfringe
#' @export
kmbasisinnerfringe <- function(state, basis) {
  noi <- length(state)
  nob <- dim(basis)[1]
  if (!inherits(basis, "kmbasis"))
    stop("Basis must be a kmbasis object")
  if (noi != dim(basis)[2])
    stop("State and basis do not match in size")

  cs <- rep(0, noi)
  inner <- cs

  apply(basis, MARGIN=1, function(x) {
    if (all(as.integer(x & state) == x))
      cs <<- as.integer(cs | x)
  })
  if (any(cs != state))
    stop("The state is no element of the space represented by the basis.")

  bkvec <- c()
  sapply(1:nob, function(b) {
    if (all(basis[b,] <= state))
      bkvec <<- c(bkvec, b)
  })
  bk <- matrix(basis[bkvec,], ncol=noi, byrow=TRUE)
  print(bk)

  inner <- rep(0, noi)
  sapply(1:noi, function(it) {
    if (state[it] == 1) {
      bknqlist <- which(bk[,it] == 0)
      bknq <- matrix(bk[bknqlist,], nrow=length(bknqlist), ncol=noi, byrow=FALSE)
      cand <- rep(0, noi)
      apply(bknq, MARGIN=1, function(b) {
        cand <<- as.integer(cand | b)
      })
      if (sum(state - cand) == 1)
        inner[it] <<- 1
    }
  })

  names(inner) <- colnames(basis)
  inner
}


#' @rdname kmbasisfringe
#' @export
kmbasisouterfringe <- function(state, basis) {
  noi <- length(state)
  if (!inherits(basis, "kmbasis"))
    stop("Basis must be a kmbasis object")
  if (noi != dim(basis)[2])
    stop("State and basis do not match in size")

  cs <- rep(0, noi)
  of <- cs
  apply(basis, MARGIN=1, function(x) {
    if (all(as.integer(x & state) == x))
      cs <<- as.integer(cs | x)
    else if (sum(as.integer(x & !state)) == 1)
      of <<- as.integer(of | x)
  })
  if (any(cs != state))
    stop("The state is no element of the space represented by the basis.")
  of <- as.integer(of & !state)
  names(of) <- colnames(basis)
  of
}

#' Compute the basis of a knowledge space
#'
#' @param x KST object
#' @return Binary matrix representing the basis of the corresponding knowledge space.
#'
#' @family Different representations for knowledge structures
#'
#' @export
kmbasis <- function(x) {
  UseMethod("kmbasis")
}

#' Determine all learning paths in a knowledge structure
#'
#' @param structure Knowledge structure
#' @param through Knowledge state through which the paths should go
#' @param from Knowledge state where paths should start
#' @param to Knowledge state where paths should end
#' @param simplified Simplified printing of states within the paths (default FALSE)
#' @return A list of learning paths where each learning path is a
#' list of states
#'
#' If \code{through} is specified, only those learning paths are listed
#' which go through that state (binary vector). If \code{from} and
#' \code{to} are specified, the respective sub paths are returned.
#'
#' @examples
#' kmlearningpaths(xpl$space)
#'
#'
#' @family Functions for wellgradedness & learning paths
#'
#'
#'@export
kmlearningpaths <- function(structure,
                            through=NULL,
                            from=NULL,
                            to=NULL,
                            simplified = FALSE) {
  if (!inherits(structure, "kmstructure"))
    stop("'structure' must be of class 'kmstructure'.")

  structure <- t(structure)
  n <- ncol(structure)
  b = diag(0,n)
  items <- nrow(structure)
  if (is.null(from)) from <- rep(0, items)
  if (is.null(to)) to <- rep(1,items)
  if (is.null(through)) through <- to
  if (!any(through %in% structure))
    stop("'through' is no state of 'structure'.")
  if (!any(to %in% structure))
    stop("'to' is no state of 'structure'.")
  if (!any(from %in% structure))
    stop("'from' is no state of 'structure'.")

  if (simplified) coll <- ''
  else coll <- ', '

  for(i in 1:n){
    for(j in 1:n){
      if(sum(structure[,i]*structure[,j])==sum(structure[,i])) b[i,j]=1
    }
  }
  diag(b)<-0
  d <- b
  for(i in 1:n){
    for(j in c(1:n)[-i]){
      if(b[j,i]==1) d[j,]=d[j,]*(1-b[i,])
    }
  }
  ed <- NULL
  for(i in 1:n) for(j in 1:n) if(d[i,j]==1) ed <- c(ed,i,j)
  if (!is.null(rownames(structure))) {
    l <- lapply(1:n, function(i) {
      paste(c(c(rownames(structure))[structure[,i]*c(1:nrow(structure))]),collapse = coll)
    })
  } else {
    l <- lapply(1:n, function(i) {
      paste(c(c(make.unique(letters[(1:nrow(structure))%%26]))[structure[,i]*c(1:nrow(structure))]),collapse = coll)
    })
  }
  if (!simplified)
    l <- as.list(paste0("{",l,"}"))
  l <- lapply(l, function(n) {
    if ((n == '{}') || (n == ''))
      '\u2205'
    else
      n
  })
  med <- matrix(ed, ncol=2, byrow=TRUE)
  index0 <- which(colSums(structure == from) == nrow(structure))
  index1 <- which(colSums(structure == to) == nrow(structure))
  indext <- which(colSums(structure == through) == nrow(structure))

  find_paths <- function(state, t) {
    if (state == index1) {
      if (state == t) {
        p <- list(list(l[[state]]))
        class(p[[1]]) <- unique(c("kmlearningpath", class(p[[1]])))
        class(p) <- unique(c("kmlearningpaths", class(p)))
        return(p)
      } else return(NULL)
    }
    ind <- which(med[,1] == state)
    lp <-list()
    if (length(ind) == 0) {
      return(NULL)
    } else {
      ups <- med[ind,2]
      lapply(ups, function(on) {
        if (on == t)
          lp2 <- find_paths(on, index1)
        else
          lp2 <- find_paths(on, t)
        if (length(lp2) > 0) {
          lp3 <- lapply(lp2, function(p) {
            x <- append(p, list(l[[state]]), after=0)
            class(x) <- unique(c("kmlearningpath", class(x)))
            x
          })
          lp <<- append(lp, lp3)
        }
        else return(NULL)
      })
    }
    class(lp) <- unique(c("kmlearningpathlist", class(lp)))
    lp
  }

  find_paths(index0, indext)

}

#' Compute a binary matrix product
#'
#' \code{binary_matrix_product} expects two binary matrices and computes their Boolean product.
#'
#' @param m Binary matrix
#' @param n Binary matrix
#'
#' @return Boolean matrix product of m and n
#'
#' @examples
#' binary_matrix_product(xpl$sr, kmsurmiserelation(kmmaximalspace(4)))
#' binary_matrix_product(xpl$sr, kmsurmiserelation(kmminimalspace(4)))
#'
#'
#' @family Utilities
#'
#' @export
binary_matrix_product <- function(m,n) {
  if (dim(m)[2] != dim(n)[1]) {
    stop(sprintf("%s and %s do not fit in size!", dQuote("m"), dQuote("n")))
  }
  res <- matrix(rep(0,dim(m)[2]*dim(n)[1]), nrow=dim(m)[2])
  lapply((1:dim(m)[2]), function(x) {
    lapply((1:dim(n)[1]), function(y) {
      res[x,y] <<- max(m[x,] & n[,y])
    })
  })
  res
}

#' @method kmclosure kmfamset
#' @param closure Type of closure
#'
#' For families of sets (\code{kmfamset}), kmclosure is basically a wrapper
#' for \code{kmunionclosure()} and \code{kmintersectionclosure}.
#' \code{closure()} may take the values "union" (default) or "intersection".
#' The former requests a closure under union, the latter a closure under
#' union **and** intersection.
#'
#' @rdname kmclosure
#'
#' @family Closure operators
#' @export
kmclosure.kmfamset <- function(x, ..., closure="union") {
  if (closure == "union")
    kmunionclosure(x)
  else if (closure == "intersection")
    kmqspace(x)
  else stop("Unknown type of closure.")
}

#' @method kmclosure kmattributionfunction
#' @family Closure operators
#' @rdname kmclosure
#' @export
kmclosure.kmattributionfunction <- function(x, ...) {
  af <- data.frame(x)
  # We start with reflexivity
  sapply((1:dim(af)[1]), function(i) {
    j <- af[i,1]
    af[i, j] <<- 1
  })
  # Next step: Transitivity
  if (dim(af)[2] > 3) {
    chgd <- TRUE
    while (chgd) {
      chgd <- FALSE
      noclauses <- dim(af)[1]
      sapply((1:noclauses), function(r) {
        if (!chgd) {
          curritem <- af[r,1]
          currclause = af[r,]
          currclause[curritem] <- 0
          prereqs <- colnames(af)[which(currclause==1)]
          clause <- as.numeric(af[r,-1])
          if (length(prereqs)>0) {
            sapply(prereqs, function(p) {
              if (!chgd) {
                cprimes <- which(af[,1] == p)
                if (length(cprimes) == 0)
                  stop(sprintf("I found no clause for item %s", p))
                foundtrans <- FALSE
                sapply(cprimes, function(cp)  {
                  clauseprime <- as.numeric(af[cp,-1])
                  if (length(clause) != length(clauseprime)) print(af)
                  if (all((clause & clauseprime) == clauseprime))
                    foundtrans <<- TRUE
                })
                if (!foundtrans) {
                  sapply(cprimes, function(cp) {
                    hf <- data.frame(curritem, t(as.numeric(af[cp,-1]|af[r,-1])))
                    names(hf) <- colnames(af)
                    af <<- rbind(af, hf)
                  })
                  af <<- af[-r,]
                  chgd <<- TRUE
                }
              }
            })
          }
        }
      })
    }
  }
  # Pre-Final step: Incomparability
  items <- dim(af)[2]-1
  chgd <- TRUE
  af[,2:(items+1)] <- sapply(af[,2:(items+1)], as.numeric)
  while (chgd) {
    chgd <- FALSE
    lapply(unique(af[,1]), function(s) {
      if (!chgd) {
        sel <- which(af[,1] == s)
        lapply(sel, function(x) {
          if (!chgd) {
            lapply(sel, function(y) {
              if (!chgd) {
                if (x != y) {
                  if (all((af[x,-1] & af[y,-1]) == af[x,-1])) {
                    af <<- af[-y,]
                    chgd <<- TRUE
                  }
                }
              }
            })
          }
        })
      }
    })
  }
  # Very final step: make sure, every item has a clause
  items <- colnames(af)[-1]
  clauseditems <- unique(af[,1])
  exists <- items %in% clauseditems
  if (!(all(exists))) {
    missing <- items[which(!(exists))]
    hmat <- matrix(rep(0, length(missing)*length(items)), nrow=length(missing))
    hdf <- cbind(data.frame((missing)), data.frame(hmat))
    colnames(hdf) <- colnames(af)
    sapply((1:length(missing)), function(x) {
      index <- which(hdf[,1] == missing[x])
      hdf[index,missing[x]] <<- 1
    })
    af <- rbind(af, hdf)
  }
  class(af) <- unique(c("kmsurmisefunction", class(af)))
  af
}

#' Knowledge spaces on fractions
#'
#' Bases of knowledge spaces on fractions obtained from querying experts.
#'
#' Three experts were queried about prerequisite relationships between 77 items
#' on fractions (Baumunk & Dowling, 1997). A forth basis represents those
#' prerequisite relationships on which the majority of the experts agree
#' (Dowling & Hockemeyer, 1998).
#'
#' @docType data
#'
#' @usage fractions
#'
#' @format A list containing four bases (frac1 to frac3, and fracmaj) in binary
#' matrix form. Each matrix has 77 columns representing the different knowledge
#' items and a varying number of rows containing the basis elements.
#'
#' @references Baumunk, K. & Dowling, C. E. (1997). Validity of spaces for assessing
#' knowledge about fractions. _Journal of Mathematical Psychology, 41,_ 99–105.
#'
#' Dowling, C. E. & Hockemeyer, C. (1998). Computing the intersection of knowledge
#' spaces using only their basis. In Cornelia E. Dowling, Fred S. Roberts, & Peter
#' Theuns, editors, _Recent Progress in Mathematical Psychology,_ pp. 133–141.
#' Lawrence Erlbaum Associates Ltd., Mahwah, NJ.
#'
"fractions"

#' Compute the heights of items in a knowledge space specified through its basis
#'
#' The height of an item in the knowledge space is defined as the minimal
#' cardinality of the states containing it minus 1, i.e. the minimal number
#' of prerequisites.
#'
#' @param b Basis
#' @returns Data.frame with a table of heights.
#'
#' @examples
#' kmheights(xpl$basis)
#'
#' @family Properties of knowledge structures
#'
#' @export
kmheights <- function(b) {
  if (!(inherits(b, "kmbasis")))
    stop("'b' must be of class 'kmbasis'.")
  apply(b, MARGIN=2, function(c) {
    l <- which(c==1)
    min(rowSums(matrix(b[l,], ncol=dim(b)[2]))) - 1
  })
}

#' Determine next question for probabilistic knowledge assessment
#'
#' \code{kmassesshalfsplit} determines the next question in a probabilistic
#' assessment according to the halfsplit rule.
#'
#' @param probs Probability distribution over the knowledge structure (vector)
#' @param ks Binary matrix of the knowledge structure
#' @return Number of the selected question
#'
#' @examples
#' kmassesshalfsplit(c(0.02, 0.1, 0.07, 0.01, 0.4, 0.17, 0.07, 0.08, 0.08),
#'                   xpl$space)
#'
#' @family Knowledge assessment functions
#'
#' @export
kmassesshalfsplit <- function(probs, ks) {
  debug <- FALSE
  set.seed(NULL)

  if (debug) print(sprintf("Prob dist: [%s]", paste(probs, collapse = ", ")))
  storage.mode(ks) <- "integer"
  noi <- dim(ks)[2]

  if ((min(ks) < 0) | (max(ks) > 1))
    stop("Knowledge structure must be a binary matrix.")
  if ((min(probs) < 0) | (max(probs) > 1))
    stop("State probabilities must be between 0 and 1.")
  if (!(kmdoubleequal(sum(probs), 1.0)))
    stop("State probabilities must sum up to 1.")

  ip <- sapply(1:noi, function(x) {
    sum(probs[which(ks[,x]==1)])
  })
  hd <- abs(ip - 0.5)
  if (debug) print(hd)
  hs <- which(hd == min(hd))
  if (debug) print(sprintf("Possible questions: %s", paste(hs, collapse=", ")))
  if (length(hs)> 1)
    hs <- sample(hs, 1)
  if (debug) print(sprintf("Question selected: %d", hs))
  hs
}

plot_neighbourhood_structure <- function(state,
                                         structure,
                                         horizontal = FALSE,
                                         colors=NULL,
                                         keepNames = TRUE,
                                         itemsep = ',',
                                         braces = TRUE,
                                         vertexshape = "oval",
                                         arrowhead = "none",
                                         arrowtail = "none",
                                         edgelabel = FALSE,
                                         verbose = 0
){
  if (is.null(colors)) colors <- c("#ff7733", "#aaccff", "#bbffbb", "#fffff0")

  structure <- t(structure)
  stateindex <- which(apply(structure, 2, function(s) {all(s==state)}))

  n <- ncol(structure)
  b = diag(0,n)
  items <- nrow(structure)
  ocolors <- colors

  if (length(colors) == 4) {
    colors <- rep(ocolors[4], n)
    colors[stateindex] <- ocolors[1]
  } else {
    stop("Illegal lentgh of 'colors' vector.")
  }

  for(i in 1:n){
    for(j in 1:n){
      if(sum(structure[,i]*structure[,j])==sum(structure[,i])) b[i,j]=1
    }
  }
  # Build (reduced) list of edges
  if (verbose > 0) {
    print("Start Building edges")
    print(Sys.time())
  }
  nos <- dim(structure)[2]
  ed <- NULL
  lapply(1:nos, function(i) {
    if (verbose > 1)
      print(sprintf("Searching neighbours for state %d", i))
    lapply(1:nos, function(j) {
      if (all((structure[,i] & structure[,j]) == structure[,i]) && (i != j)) {
        trans <- FALSE
        k <- 0
        while (k < nos && !trans) {
          k <- k + 1
          if ((i != k) && (j != k) &&
              (all((structure[,i] & structure[,k]) == structure[,i])) &&
              (all((structure[,k] & structure[,j]) == structure[,k]))) {
            trans <- TRUE
          }
        }
        if (k <= nos && !trans)
          ed <<- c(ed, i, j)      }
    })
  })
  edges <- unique(matrix(ed, ncol=2, byrow=TRUE), MARGIN=2)
  if (length(ocolors) == 4) {
    upper <- edges[which(edges[,1]==stateindex),2]
    colors[upper] <- ocolors[2]
    lower <- edges[which(edges[,2]==stateindex),1]
    colors[lower] <- ocolors[3]
  }
  if (verbose > 0) {
    print("Finished Building edges, start building vertices")
    print(Sys.time())
  }
  if (keepNames && !is.null(rownames(structure))) {
    l <- lapply(1:n, function(i) {
      paste(c(c(rownames(structure))[structure[,i]*c(1:nrow(structure))]),collapse = itemsep)
    })
  } else {
    l <- lapply(1:n, function(i) {
      paste(c(c(make.unique(letters[(1:nrow(structure))%%26]))[structure[,i]*c(1:nrow(structure))]),collapse = itemsep)
    })
  }
  if (braces)
    l <- as.list(paste0("{",l,"}"))
  l <- lapply(l, function(n) {
    if ((n == '') || (n == '{}'))
      '\u2205'
    else
      n
  })

  l <- unlist(l)
  names(colors) <- l
  nl <- sprintf('"%s" [color="%s"];',
                l,
                colors[l])
  el <- apply(edges, 1, function(e) {
    if (edgelabel) {
      delta <- kmsymmsetdiff(structure[,e[1]], structure[,e[2]])
      warning(delta)
      ditems = which((delta == 1))
      if (keepNames && !is.null(rownames(structure))) {
        edlabel <- paste(rownames(structure)[ditems], collapse=',')
      } else {
        edlabel <- paste((ditems), collapse=',')
      }
      sprintf('"%s" -> "%s" [label = "%s"];',
              l[e[2]],
              l[e[1]],
              edlabel
      )
    } else {
      sprintf('"%s" -> "%s";', l[e[2]], l[e[1]])
    }
  })
  if (horizontal) {
    direction <- "RL"
    if (arrowtail == "none") warning("For horizontal diagrams, arrowtail should be set.")
  } else
    direction <- "TB"
  dotGraph <- paste0(
    'digraph hasse {',
    sprintf('rankdir=%s;', direction),
    sprintf('node [fontname="Helvetica", shape=%s, style=filled];', vertexshape),
    paste(nl, collapse="\n"),
    sprintf('edge [dir="both", arrowtail="%s", arrowhead="%s"];', arrowtail, arrowhead),
    paste(el, collapse="\n"),
    # print(el, collapse="\n"),
    '}'
  )
  if (verbose > 0) {
    print("Ready for plotting")
    print(Sys.time())
  }
  grViz(dotGraph)
}



highlightprint <- function(x, ...,
                           open = '{', close = '}',
                           itemsep = ', ',
                           highlight = NULL,
                           highlightcolor = "red",
                           color = "beige") {
  if (is.null(names(x))) {
    stop("'x' must be named.")
  }
  if (!is.numeric(x)) stop("'x' must be a binary integer vector.")
  if ((min(x) < 0) || (max(x) > 1))
    stop("'x' must be a binary integer vector.")
  if (sum(x) == 0) return('empty [label = \u2205]')
  # if (sum(x) == 0) return("")
  l <- which(x>0)
  v <- names(x)[l]
  if (!is.null(highlight)) {
    if (length(x) != length(highlight)) stop("x and highlight do not fit in size.")
    h <- which(highlight[l] > 0)
    v[h] <- sprintf('<font color="%s">%s</font>', highlightcolor, v[h])
  }
  h <- paste(c("label = <", open, paste(v, collapse=itemsep), close, ">"), collapse='')
  pp <- kmprettyprint(x, simplified=TRUE)
  paste(c(pp, ' [', h, ']'), collapse='')
}



#' Plot a Hasse diagram
#'
#' \code{plot} draws a Hasse diagram for a family of sets or a surmise relation.
#'
#' \code{plot} takes a matrix representing a family of sets (knowledge states)
#' or a surmise relation and a color vector, and draws a Hasse diagram.
#' If the color vector is NULL the states are drawn in green, the items in
#' the relation are drawn in orange.
#'
#' For a surmise relation, if there are equivalent items, they are contracted
#' into one vertex labelled 'a ~ b ~ ...' for equivalent items a, b, ...
#'
#' If the plot is to be used within a Shiny app, it must be included with
#' \code{grVizOutput()} and \code{renderGrViz()} from the \code{DiagrammeR}
#' package (\code{plotOutput()} and \code{renderPlot()} do not work).
#'
#' Please note that, for equivalent items in a relation plot, the prerequisites
#' are lost.
#'
#' If the KST object to be plotted is a family of sets (`kmfamset`) **and**
#' a `state` is specified, the `colors` vector must have four elements, (i)
#' the color for the state, (ii) the color for its upper neighbours, (iii)
#' the color for its lower neighbours, and (iv) the color for all other states.
#'
#' @param x Binary matrix representing a family of sets
#' @param ... Optional inherited parameters
#' @param state Knowledge state whose neighbourhood shall be highlighted
#' @param horizontal Boolean defining orientation of the graph, default FALSE
#' @param colors Color value or vector (default NULL).
#' @param keepNames Keep item names (default TRUE)
#' @param itemsep Item separator in sets (default ','; only for families
#' of states)
#' @param braces Put braces around vertices (default TRUE; only for
#' families of states)
#' @param vertexshape Shape of the vertex objects, e.g. circle, oval, box, or none. See
#' \href{https://graphviz.org/doc/info/shapes.html}{Graphviz Node Shapes}
#' for a complete list of possible values.
#' @param arrowhead Form of the arrow head, e.g. vee or none (default). See
#' \href{https://graphviz.org/docs/attr-types/arrowType/}{Graphviz Arrow
#' Types} for a complete list of possible values. This may be used for vertical
#' graphs although none is the standard there.
#' @param arrowtail Form of the arrow tail, e.g. vee or none (default). See
#' \href{https://graphviz.org/docs/attr-types/arrowType/}{Graphviz Arrow
#' Types} for a complete list of possible values. This should be used for
#' horizontal graphs.
#' @param edgelabel Boolean whether to label the edges of the diagram
#' (default FALSE)
#' @param verbose Verbosity level (0 (default), 1, or 2)
#'
#' @examples
#' \dontrun{
#' plot(phsg$basis)
#'
#' sp <- kmunionclosure(phsg$basis)
#' n <- kmneighbourhood(phsg$basis[3,], sp, include=TRUE)
#' plot(n, state=phsg$basis[3,])
#'
#' m <- matrix(c(1,0,0,1,1,1,1,1,1), ncol=3, byrow=FALSE)
#' class(m) <- unique(c("kmsurmiserelation", class(m)))
#' plot(m, vertexshape="oval")
#' }
#'
#' @family Functions for plotting and printing knowledge structures
#'
#' @importFrom DiagrammeR grViz
#'
#' @rdname plot
#' @name plot
#'
#' @method plot kmfamset
#' @export
plot.kmfamset <- function(x,
                          ...,
                          state = NULL,
                          horizontal = FALSE,
                          colors=NULL,
                          keepNames = TRUE,
                          itemsep = ',',
                          braces = TRUE,
                          vertexshape = "oval",
                          arrowhead = "none",
                          arrowtail = "none",
                          edgelabel = FALSE,
                          verbose = 0
){
  if (!is.null(state))
    plot_neighbourhood_structure(state, x, horizontal, colors, keepNames, itemsep, braces,
                                 vertexshape, arrowhead, arrowtail, edgelabel, verbose)
  else {
    structure <- t(x)

    n <- ncol(structure)
    b = diag(0,n)
    items <- nrow(structure)

    # if (is.null(rownames(structure))) {
    #   rownames(structure) <- paste(1:items)
    # }

    if (!(is.null(colors))) {
      if (length(colors) == 1) {
        colors <- rep(colors, n)
      } else if (n != length(colors)) {
        stop("Incompatible parameters (length of 'colors')!")
      }
    } else { # is.null(colors)
      colors <- rep("#aaffbb", n)
    }

    for(i in 1:n){
      for(j in 1:n){
        if(sum(structure[,i]*structure[,j])==sum(structure[,i])) b[i,j]=1
      }
    }
    # Build (reduced) list of edges
    if (verbose > 0) {
      print("Start Building edges")
      print(Sys.time())
    }
    nos <- dim(x)[1]
    ed <- NULL
    lapply(1:nos, function(i) {
      if (verbose > 1)
        print(sprintf("Searching neighbours for state %d", i))
      lapply(1:nos, function(j) {
        if (all((x[i,] & x[j,]) == x[i,]) && (i != j)) {
          trans <- FALSE
          k <- 0
          while (k < nos && !trans) {
            k <- k + 1
            if ((i != k) && (j != k) &&
                (all((x[i,] & x[k,]) == x[i,])) &&
                (all((x[k,] & x[j,]) == x[k,]))) {
              trans <- TRUE
            }
          }
          if (k <= nos && !trans)
            ed <<- c(ed, i, j)      }
      })
    })
    if (verbose > 0) {
      print("Finished Building edges, start building vertices")
      print(Sys.time())
    }
    if (keepNames && !is.null(rownames(structure))) {
      l <- lapply(1:n, function(i) {
        paste(c(c(rownames(structure))[structure[,i]*c(1:nrow(structure))]),collapse = itemsep)
      })
    } else {
      l <- lapply(1:n, function(i) {
        paste(c(c(make.unique(letters[(1:nrow(structure))%%26]))[structure[,i]*c(1:nrow(structure))]),collapse = itemsep)
      })
    }
    if (braces)
      l <- as.list(paste0("{",l,"}"))
    l <- lapply(l, function(n) {
      if ((n == '') || (n == '{}'))
        '\u2205'
      else
        n
    })

    l <- unlist(l)
    names(colors) <- l
    nl <- sprintf('"%s" [color="%s"];',
                  l,
                  colors[l])
    edges <- unique(matrix(ed, ncol=2, byrow=TRUE), MARGIN=2)
    el <- apply(edges, 1, function(e) {
      if (edgelabel) {
        delta <- kmsymmsetdiff(structure[,e[1]], structure[,e[2]])
        warning(delta)
        ditems = which((delta == 1))
        if (keepNames && !is.null(rownames(structure))) {
          edlabel <- paste(rownames(structure)[ditems], collapse=',')
        } else {
          edlabel <- paste((ditems), collapse=',')
        }
        sprintf('"%s" -> "%s" [label = "%s"];',
                l[e[2]],
                l[e[1]],
                edlabel
        )
      } else {
        sprintf('"%s" -> "%s";', l[e[2]], l[e[1]])
      }
    })
    if (horizontal) {
      direction <- "RL"
      if (arrowtail == "none") warning("For horizontal diagrams, arrowtail should be set.")
    } else
      direction <- "TB"
    dotGraph <- paste0(
      'digraph hasse {',
      sprintf('rankdir=%s;', direction),
      sprintf('node [fontname="Helvetica", shape=%s, style=filled];', vertexshape),
      paste(nl, collapse="\n"),
      sprintf('edge [dir="both", arrowtail="%s", arrowhead="%s"];', arrowtail, arrowhead),
      paste(el, collapse="\n"),
      # print(el, collapse="\n"),
      '}'
    )
    if (verbose > 0) {
      print("Ready for plotting")
      print(Sys.time())
    }
    grViz(dotGraph)
  }
}


#' @param state Knowledge state whose neighbourhood is to be pictured
#'              (Default `NULL``)
#'
#' If `state` is `NULL`, the neighbourhood is depicted as a 'normal'
#' family of sets (`kmfamset`) and a respective warning is issued.
#'
#' @rdname plot
#' @name plot
#'
#' @method plot kmneighbourhood
#' @export
plot.kmneighbourhood <- function(x,
                                 ...,
                                 horizontal = FALSE,
                                 colors=c("#eeee00", "#aaccff", "#bbffbb"),
                                 keepNames = TRUE,
                                 itemsep = ',',
                                 braces = TRUE,
                                 vertexshape = "oval",
                                 arrowhead = "none",
                                 arrowtail = "none",
                                 edgelabel = FALSE,
                                 state = NULL,
                                 verbose = 0
){
  if (is.null(state)) {
    warning("NULL 'state' parameter, plotting 'x' as normal 'kmfamset', i.e. in identical colours.")
    plot.kmfamset(x,
                  horizontal = horizontal,
                  colors = colors[1],
                  keepNames = keepNames,
                  itemsep = itemsep,
                  braces = braces,
                  vertexshape = vertexshape,
                  arrowhead = arrowhead,
                  arrowtail = arrowtail,
                  edgelabel = edgelabel,
                  verbose = verbose
    )
  } else {
    structure <- t(x)

    n <- ncol(structure)
    items <- nrow(structure)

    if (!(is.null(colors))) {
      if (length(colors) == 1) {
        colors <- rep(colors, n)
      } else if (length(colors) == 3) {
        if (!is.null(state)) {
          if (length(state) != items)
            stop("Incompatible parameter (state size).")
          threecolors <- colors
          colors <- rep(threecolors[1], n)
          colors <- apply(structure, MARGIN=2, function(s) {
            if (all(s == state)) threecolors[1]
            else if (all((s & state) == state))
              threecolors[2]
            else threecolors[3]
          })
        } else {
          colors <- rep("#eeee00", n)
        }
      } else if (n != length(colors)) {
        stop("Incompatible parameters (length of 'colors')!")
      }
    } else { # is.null(colors)
      colors <- rep("#aaffbb", n)
    }

    if (n != length(colors)) {
      warning(colors)
      stop("Incompatible parameters (length of 'colors')!")
    }

    nos <- dim(x)[1]
    ed <- NULL
    lapply(1:nos, function(i) {
      if (verbose > 1)
        print(sprintf("Searching neighbours for state %d", i))
      lapply(1:nos, function(j) {
        if (all((x[i,] & x[j,]) == x[i,]) && (i != j)) {
          trans <- FALSE
          k <- 0
          while (k < nos && !trans) {
            k <- k + 1
            if ((i != k) && (j != k) &&
                (all((x[i,] & x[k,]) == x[i,])) &&
                (all((x[k,] & x[j,]) == x[k,]))) {
              trans <- TRUE
            }
          }
          if (k <= nos && !trans)
            ed <<- c(ed, i, j)      }
      })
    })

    if (keepNames && !is.null(rownames(structure))) {
      l <- lapply(1:n, function(i) {
        paste(c(c(rownames(structure))[structure[,i]*c(1:nrow(structure))]),collapse = itemsep)
      })
    } else {
      l <- lapply(1:n, function(i) {
        paste(c(c(make.unique(letters[(1:nrow(structure))%%26]))[structure[,i]*c(1:nrow(structure))]),collapse = itemsep)
      })
    }
    if (braces)
      l <- as.list(paste0("{",l,"}"))
    l <- lapply(l, function(n) {
      if ((n == '') || (n == '{}'))
        '\u2205'
      else
        n
    })

    l <- unlist(l)
    names(colors) <- l
    nl <- sprintf('"%s" [color="%s"];',
                  l,
                  colors[l])
    edges <- unique(matrix(ed, ncol=2, byrow=TRUE), MARGIN=2)
    el <- apply(edges, 1, function(e) {
      if (edgelabel) {
        delta <- kmsymmsetdiff(structure[,e[1]], structure[,e[2]])
        ditems = which(delta == 1)
        if (keepNames && !is.null(rownames(structure))) {
          edlabel <- paste(rownames(structure)[ditems], collapse=',')
        } else {
          edlabel <- paste((ditems), collapse=',')
        }
        sprintf('"%s" -> "%s" [label = "%s"];',
                l[e[2]],
                l[e[1]],
                edlabel
        )
      } else {
        sprintf('"%s" -> "%s";', l[e[2]], l[e[1]])
      }
    })
    if (horizontal) {
      direction <- "RL"
      if (arrowtail == "none") warning("For horizontal diagrams, arrowtail should be set.")
    } else
      direction <- "TB"
    dot <- paste0(
      'digraph hasse {',
      sprintf('rankdir=%s;', direction),
      sprintf('node [fontname="Helvetica", shape=%s, style=filled];', vertexshape),
      paste(nl, collapse="\n"),
      sprintf('edge [dir="both", arrowtail="%s", arrowhead="%s"];', arrowtail, arrowhead),
      paste(el, collapse="\n"),
      # print(el, collapse="\n"),
      '}'
    )
    grViz(dot)
  }
}






#' @rdname plot
#' @name plot
#'
#' @method plot kmsurmiserelation
#' @export
plot.kmsurmiserelation <- function(x,
                                   ...,
                                   horizontal = FALSE,
                                   colors = NULL,
                                   keepNames = TRUE,
                                   vertexshape = "circle",
                                   arrowhead = "none",
                                   arrowtail = "none",
                                   verbose = 0
){
  if (is.null(colnames(x))) {
    colnames(x) <- make.unique(letters[(0L:(ncol(x)-1)) %% 26 + 1])
    rownames(x) <- colnames(x)
  }
  bas <- kmbasis(x)
  notions <- kmnotions(bas)
  w <- which(rowSums(notions)>1)
  if (length(w) > 0) {  # There exist equivalent items
    redsr <- kmsurmiserelation(kmeqreduction(bas))
    label <- sapply(as.list(1:(dim(notions)[1])), function(item) {
      paste(colnames(notions)[which(notions[item,]>0)], collapse='~')
    })
    x <- redsr
    rownames(x) <- label
    colnames(x) <- label
  }

  n <- ncol(x)

  if (is.null(colors)) {
    colors <- rep("orange", n)
  }
  if (length(colors)==1) colors <- rep(colors, n)
  if (length(colors) != n)
    stop("Incompatible parameters (length of 'colors').")

  nos <- dim(x)[1]
  ed <- NULL
  lapply(1:nos, function(i) {
    if (verbose > 1)
      print(sprintf("Searching neighbours for item %d", i))
    lapply(1:nos, function(j) {
      if (all((x[i,] & x[j,]) == x[i,]) && (i != j)) {
        trans <- FALSE
        k <- 0
        while (k < nos && !trans) {
          k <- k + 1
          if ((i != k) && (j != k) &&
              (all((x[i,] & x[k,]) == x[i,])) &&
              (all((x[k,] & x[j,]) == x[k,]))) {
            trans <- TRUE
          }
        }
        if (k <= nos && !trans)
          ed <<- c(ed, j, i)      }
    })
  })
  if (keepNames && !is.null(colnames(x))) {
    l <- colnames(x)
  } else {
    l <- make.unique(letters[(1:ncol(x))%%26])
  }

  names(colors) <- l
  nl <- sprintf('"%s" [color="%s"];',
                l,
                colors[l])
  edges <- matrix(ed, ncol=2, byrow=TRUE)
  el <- apply(edges, 1, function(e) {
    sprintf('"%s" -> "%s";', l[e[2]], l[e[1]])
  })
  if (horizontal) {
    direction <- "RL"
    if (arrowtail == "none") warning("For horizontal diagrams, arrowtail should be set.")
  } else
    direction <- "TB"
  dot <- paste0(
    'digraph hasse {',
    sprintf('rankdir=%s;', direction),
    'TBbalance="max";
       fontsize=18.0;',
    sprintf('node [fontname="Helvetica", shape=%s, style=filled];', vertexshape),
    paste(nl, collapse="\n"),
    sprintf('edge [dir="both", arrowtail="%s", arrowhead="%s"];', arrowtail, arrowhead),
    paste(el, collapse="\n"),
    '}'
  )
  grViz(dot)
}



#' @param structure Knowledge structure within which the paths shall be depicted
#' @param pathcolors Colors for the individual learning paths
#' @rdname plot
#' @method plot kmlearningpathmatrices
#' @export
plot.kmlearningpathmatrices <- function(x,
                                        ...,
                                        horizontal = FALSE,
                                        colors=NULL,
                                        keepNames = TRUE,
                                        itemsep = ',',
                                        braces = TRUE,
                                        vertexshape = "oval",
                                        arrowhead = "none",
                                        arrowtail = "none",
                                        edgelabel = FALSE,
                                        verbose = 0,
                                        structure = NULL,
                                        pathcolors = c("red", "blue", "gold", "deeppink", "brown", "violet", "green")
){
  if (is.null(structure)) stop("Unspecified 'structure'")

  structure <- t(structure)

  n <- ncol(structure)
  b = diag(0,n)
  items <- nrow(structure)

  if (!(is.null(colors))) {
    if (length(colors) == 1) {
      colors <- rep(colors, n)
    } else if (n != length(colors)) {
      stop("Incompatible parameters (length of 'colors')!")
    }
  } else { # is.null(colors)
    colors <- rep("#bbffcc", n)
  }

  for(i in 1:n){
    for(j in 1:n){
      if(sum(structure[,i]*structure[,j])==sum(structure[,i])) b[i,j]=1
    }
  }
  # Build (reduced) list of edges
  if (verbose > 0) {
    print("Start Building edges")
    print(Sys.time())
  }
  nos <- dim(structure)[2]
  ed <- NULL
  lapply(1:nos, function(i) {
    if (verbose > 1)
      print(sprintf("Searching neighbours for state %d", i))
    lapply(1:nos, function(j) {
      if (all((structure[,i] & structure[,j]) == structure[,i]) && (i != j)) {
        trans <- FALSE
        k <- 0
        while (k < nos && !trans) {
          k <- k + 1
          if ((i != k) && (j != k) &&
              (all((structure[,i] & structure[,k]) == structure[,i])) &&
              (all((structure[,k] & structure[,j]) == structure[,k]))) {
            trans <- TRUE
          }
        }
        if (k <= nos && !trans)
          ed <<- c(ed, i, j)      }
    })
  })
  if (verbose > 0) {
    print("Finished Building edges, start building vertices")
    print(Sys.time())
  }
  if (keepNames && !is.null(rownames(structure))) {
    l <- lapply(1:n, function(i) {
      paste(c(c(rownames(structure))[structure[,i]*c(1:nrow(structure))]),collapse = itemsep)
    })
  } else {
    l <- lapply(1:n, function(i) {
      paste(c(c(make.unique(letters[(1:nrow(structure))%%26]))[structure[,i]*c(1:nrow(structure))]),collapse = itemsep)
    })
  }
  if (braces)
    l <- as.list(paste0("{",l,"}"))
  l <- lapply(l, function(n) {
    if ((n == '') || (n == '{}'))
      '\u2205'
    else
      n
  })

  l <- unlist(l)
  names(colors) <- l
  nl <- sprintf('"%s" [color="%s"];',
                l,
                colors[l])
  edges <- unique(matrix(ed, ncol=2, byrow=TRUE), MARGIN=2)
  el <- apply(edges, 1, function(e) {
    if (edgelabel) {
      delta <- kmsymmsetdiff(structure[,e[1]], structure[,e[2]])
      ditems = which((delta == 1))
      if (keepNames && !is.null(rownames(structure))) {
        edlabel <- paste(rownames(structure)[ditems], collapse=',')
      } else {
        edlabel <- paste((ditems), collapse=',')
      }
      sprintf('"%s" -> "%s" [label = "%s"];',
              l[e[2]],
              l[e[1]],
              edlabel
      )
    } else {
      sprintf('"%s" -> "%s";', l[e[2]], l[e[1]])
    }
  })
  lpind <- 0
  ellp <- sapply(x, function(lp) {
    lpind <<- lpind + 1
    len <- dim(lp)[1] - 1
    sapply(1:len, function(s) {
      col_match <- apply(structure, 2, identical, lp[s,])
      ind1 <- which(col_match)
      col_match <- apply(structure, 2, identical, lp[s+1,])
      ind2 <- which(col_match)
      sprintf('"%s" -> "%s" [color="%s", penwidth="2.5"];', l[ind2], l[ind1], pathcolors[((lpind-1) %% length(pathcolors))+1])
    })
  })
  if (horizontal) {
    direction <- "RL"
    if (arrowtail == "none") warning("For horizontal diagrams, arrowtail should be set.")
  } else
    direction <- "TB"
  dotGraph <- paste0(
    'digraph hasse {',
    sprintf('rankdir=%s;', direction),
    sprintf('node [fontname="Helvetica", shape=%s, style=filled];', vertexshape),
    paste(nl, collapse="\n"),
    sprintf('edge [dir="both", arrowtail="%s", arrowhead="%s"];', arrowtail, arrowhead),
    paste(el, collapse="\n"),
    paste(ellp, collapse="\n"),
    '}'
  )
  if (verbose > 0) {
    print("Ready for plotting")
    print(Sys.time())
  }
  grViz(dotGraph)
}





#' @param structure Knowledge structure within which the path shall be depicted
#' @param pathcolor Colors for the individual learning paths (default: blue)
#' @rdname plot
#' @method plot kmlearningpathmatrix
#' @export
plot.kmlearningpathmatrix <- function(x,
                                      ...,
                                      horizontal = FALSE,
                                      colors=NULL,
                                      keepNames = TRUE,
                                      itemsep = ',',
                                      braces = TRUE,
                                      vertexshape = "oval",
                                      arrowhead = "none",
                                      arrowtail = "none",
                                      edgelabel = FALSE,
                                      verbose = 0,
                                      structure = NULL,
                                      pathcolor = "blue"
){
  if (is.null(structure)) {
    warning("No 'structure' specified, plotting 'x' as 'kmfamset'.")
    plot.kmfamset(x, horizontal, colors, keepNames,
                  itemsep, braces, vertexshape,
                  arrowhead, arrowtail, edgelabel,
                  verbose)
  } else {
    l <- list(x)
    class(l) <- unique(c("kmlearningpathmatrices", class(l)))
    plot(l,
         horizontal = horizontal,
         colors = colors,
         keepNames = keepNames,
         itemsep = itemsep,
         braces = braces,
         vertexshape = vertexshape,
         arrowhead = arrowhead,
         arrowtail = arrowtail,
         edgelabel = edgelabel,
         verbose = verbose,
         structure = structure,
         pathcolors = c(pathcolor)
    )
  }
}



#' @param space Should basis or space de depicted? (default FALSE)
#' @param highlightcolor Color for highlighting
#' @rdname plot
#' @export
plot.kmsurmisefunction <- function(x,
                                   ...,
                                   space = FALSE,
                                   horizontal = FALSE,
                                   colors="Beige",
                                   keepNames = TRUE,
                                   itemsep = ',',
                                   braces = TRUE,
                                   vertexshape = "oval",
                                   arrowhead = "none",
                                   arrowtail = "none",
                                   edgelabel = FALSE,
                                   verbose = 0,
                                   highlightcolor = "red"
){
  if (braces) {
    open <- '{'
    close <- '}'
  } else {
    open <- ''
    close <- ''
  }
  atoms <- kmbasis(x)
  if (space)
    structure <- t(kmspace(x))
  else
    structure <- t(atoms)
  noa <- dim(atoms)[1]
  items <- dim(atoms)[2]
  atomic <- matrix(rep(0, noa*items), ncol=items)
  colnames(atomic) <- colnames(atoms)
  sapply((1:noa), function(s) {
    minimals <- x[which(apply(x[,(2:(items+1))], 1, function(v) {all(v == atoms[s,])})),1]
    atomic[s, minimals] <<- 1
  })

  n <- ncol(structure)
  b = diag(0,n)

  if (!(is.null(colors))) {
    if (length(colors) != 1) {
      stop("For plotting surmise functions, only one node color may be specified")
    }
  } else { # is.null(colors)
    colors <- "Beige"
  }

  for(i in 1:n){
    for(j in 1:n){
      if(sum(structure[,i]*structure[,j])==sum(structure[,i])) b[i,j]=1
    }
  }
  # Build (reduced) list of edges
  if (verbose > 0) {
    print("Start Building edges")
    print(Sys.time())
  }
  nos <- dim(structure)[2]
  x <- t(structure)
  ed <- NULL
  lapply(1:nos, function(i) {
    if (verbose > 1)
      print(sprintf("Searching neighbours for state %d", i))
    lapply(1:nos, function(j) {
      if (all((x[i,] & x[j,]) == x[i,]) && (i != j)) {
        trans <- FALSE
        k <- 0
        while (k < nos && !trans) {
          k <- k + 1
          if ((i != k) && (j != k) &&
              (all((x[i,] & x[k,]) == x[i,])) &&
              (all((x[k,] & x[j,]) == x[k,]))) {
            trans <- TRUE
          }
        }
        if (k <= nos && !trans)
          ed <<- c(ed, i, j)      }
    })
  })
  if (verbose > 0) {
    print("Finished Building edges, start building vertices")
    print(Sys.time())
  }
  if (is.null(rownames(structure)) || !keepNames) {
    rownames(structure) <- c(make.unique(letters[(1:nrow(structure))%%26]))
  }
  l <- lapply(1:n, function(i) {
    w <- which(apply(atoms, 1, function(v) {all(v == structure[,i])}))
    if (length(w) == 0)
      highlightprint(structure[,i], open=open, close=close, itemsep=itemsep)
    else if (length(w) == 1)
      highlightprint(structure[,i], highlight=atomic[w,], highlightcolor=highlightcolor,
                     open=open, close=close, itemsep=itemsep)
    else stop("Internal error: Found too many fitting atoms")
  })

  l <- unlist(l)
  nl <- l
  l <- sapply(nl, function(node) {
    spcpos <- unlist(gregexpr(' ', node))[1]
    substr(node, 1, spcpos-1)
  })
  edges <- unique(matrix(ed, ncol=2, byrow=TRUE), MARGIN=2)
  el <- apply(edges, 1, function(e) {
    if (edgelabel) {
      delta <- kmsymmsetdiff(structure[,e[1]], structure[,e[2]])
      warning(delta)
      ditems = which((delta == 1))
      if (keepNames && !is.null(rownames(structure))) {
        edlabel <- paste(rownames(structure)[ditems], collapse=',')
      } else {
        edlabel <- paste((ditems), collapse=',')
      }
      sprintf('"%s" -> "%s" [label = "%s"];',
              l[e[2]],
              l[e[1]],
              edlabel
      )
    } else {
      sprintf('"%s" -> "%s";', l[e[2]], l[e[1]])
    }
  })
  if (horizontal) {
    direction <- "RL"
    if (arrowtail == "none") warning("For horizontal diagrams, arrowtail should be set.")
  } else
    direction <- "TB"
  dotGraph <- paste0(
    'digraph hasse {',
    sprintf('rankdir=%s;', direction),
    sprintf('node [fontname="Helvetica", shape=%s, style=filled, color=%s];\n', vertexshape, colors),
    paste(nl, collapse="\n"),
    sprintf('edge [dir="both", arrowtail="%s", arrowhead="%s"];\n', arrowtail, arrowhead),
    paste(el, collapse="\n"),
    # print(el, collapse="\n"),
    '}'
  )
  if (verbose > 0) {
    print("Ready for plotting")
    print(Sys.time())
  }
  grViz(dotGraph)
}

#' Check for wellgradedness of a knowledge structure
#'
#' \code{kmiswellgraded()} returns whether a knowledge space (or its basis)
#' is well-graded.
#'
#' @param x \code{kmspace} or \code{kmbasis}
#' @return Logical value specifying whether `x` is well-graded
#'
#' @references Doignon, J.-P. & Falmagne, J.-C. (1999). _Knowledge Spaces._
#' Springer–Verlag, Berlin.
#'
#' @examples
#' kmiswellgraded(xpl$space)
#'
#' @family Properties of knowledge structures
#' @family Functions for wellgradedness & learning paths
#'
#' @export
kmiswellgraded <- function(x) {
  if (!inherits(x, "kmspace") && !inherits(x, "kmbasis"))
    stop("'x' must be a 'kmspace' or 'kmbasis' object.")
  sf <- kmsurmisefunction(x)[,-1]
  usf <- unique(sf, MARGIN=1)
  if (dim(sf)[1] != dim(usf)[1]) return(FALSE)
  else return(TRUE)
}

#' Update probability distribution applying multiplicative rule
#'
#' \code{kmassessmultiplicative} updates a probability distribution on a
#' knowledge structure according to the multiplicative rule.
#'
#' @param probs Probability distribution over the knowledge structure (vector)
#' @param ks Binary matrix of the knowledge structure
#' @param zeta0 Vector of update parameters for wrong responses
#' @param zeta1 Vector of update parameters for correct responses
#' @param question Item that has been posed
#' @param response Correctness of received response (0 or 1)
#' @return Updated probability vector
#'
#' @examples
#' kmassessmultiplicative(c(0.02, 0.1, 0.07, 0.01, 0.4, 0.17, 0.07, 0.08, 0.08),
#'                  xpl$space,
#'                  rep(1.2,4),
#'                  rep(2.1,4),
#'                  3,
#'                  1
#'                 )
#'
#' @family Knowledge assessment functions
#'
#' @export
kmassessmultiplicative <- function(probs, ks, zeta0, zeta1, question, response) {
  storage.mode(ks) <- "integer"
  nos <- dim(ks)[1]
  noi <- dim(ks)[2]

    if ((min(ks) < 0) | (max(ks) > 1))
    stop("Knowledge structure must be a binary matrix.")
  if ((min(probs) < 0) | (max(probs) > 1))
    stop("State probabilities must be between 0 and 1.")
  if (!(kmdoubleequal(sum(probs), 1.0)))
    stop ("Probabilities do not sum up to 1!")
  if (is.null(zeta0) | is.null((zeta1)))
    stop("zeeta0 and zeta1 must be set.")
  if (length(zeta0) != noi)
    stop("zeta0 and pks do not fit in size")
  if (length(zeta1) != noi)
    stop("zeta1 and pks do not fit in size")
  if ((min(zeta0) <= 1) | (min(zeta1) <= 1))
    stop("zeta0 and zeta1 must be larger than 1")

  if (response == 1) {
    up <- zeta1[question]
    um <- 1
  } else {
    up <- 1
    um <- zeta0[question]
  }
  phelp <- sapply((1:nos), function(s) {
    if (ks[s, question] == 1) up * probs[s]
    else um * probs[s]
  })
  pnew <- phelp / sum(phelp)
  if (!(kmdoubleequal(sum(pnew), 1.0))) {
    print(pnew)
    print(sum(pnew))
    stop ("Internal error: Probabilities do not sum up to 1!")
  }
  pnew
}

#' Convert an IITA result into a surmise relation matrix
#'
#' \code{kmiita2SR} takes the result of a \code{DAKS::iita()} call and
#' delivers the matrix of the computed surmise relation.
#'
#' @param ii \code{iita()} result
#' @param names Vector of item names (default NULL)
#' @param items Minimal number of items (default 0)
#' @return Surmise relation matrix
#'
#' The \code{iita()} function looses information on the item names and
#' uses consecutive numbers instead. The \code{implications} part of its
#' result does not give any hint on isolated items, i.e. items which
#' neither have a prerequisite nor are prerequisite of any other item.
#' Therefore, a minimal number of items can be passed to
#' \code{kmiita2SR()}. If the highest item number within
#' \code{implications} is higher, this \code{items} parameter is ignored.
#'
#' @family Functions for working with data
#'
#' @export
kmiita2SR <- function(ii, names = NULL, items = 0) {
  hm <- matrix(unlist(ii$implications), nrow=2)
  noi <- max(max(unlist(ii$implications)), items)
  sr <- diag(noi)
  apply(hm, 2, function(x) {sr[x[1],x[2]] <<- 1})
  if (!is.null(names)) {
    rownames(sr) <- names
    colnames(sr) <- names
  }
  class(sr) <- unique(c("kmsurmiserelation", class(sr)))
  sr
}

#' Create trivial knowledge spaces
#'
#' These functions create trivial knowledge spaces of a given item number. The minimal
#' space contains just the empty set and the full item set while the maximal space is
#' equal to the power set.
#'
#' Please note that the computation time for creating large power sets can grow quite
#' large easily.
#'
#' @param noi Number of items
#' @return A binary matrix representing the respective knowledge space
#'
#' @examples
#' kmminimalspace(3)
#' kmmaximalspace(3)
#'
#' @family Utilities
#' @family Functions for working with knowledge structures
#'
#' @name kmtrivial
NULL

#' @rdname kmtrivial
#' @export
kmminimalspace <- function(noi) {
  m <- matrix(c(rep(0,noi), rep(1,noi)), nrow = 2, ncol = noi, byrow = TRUE)
  class(m) <- unique(c("kmspace", "kmstructure", "kmfamset", class(m)))
  m
}

#' @rdname kmtrivial
#' @export
kmmaximalspace <- function(noi) {
  m <- kmunionclosure(kmfamset(diag(noi)))
  class(m) <- unique(c("kmspace", "kmstructure", "kmfamset", class(m)))
  m
}

#' @method kmclosure kmattributionrelation
#' @rdname kmclosure
#'
#' @family Closure operators
#' @export
kmclosure.kmattributionrelation <- function(x, ...) {
  if ((max(x) > 1) || (min(x) < 0)) {
    stop(sprintf("%s is not a binary matrix!", dQuote("x")))
  }
  size <- dim(x)[1]
  if (size != dim(x)[2]) {
    stop(sprintf("%s is not a quadratic matrix!", dQuote("x")))
  }

  # Close ar under reflexivity
  d <- diag(1,size,size)
  ar <- 1 * (x | d)

  # Close ar under transitivity
  sr <- binary_matrix_product(ar, ar)
  sr.old <- ar
  while(any(sr != sr.old)) {
    sr.old <- sr
    sr <- binary_matrix_product(sr, sr)
  }
  colnames(sr) <- colnames(ar)
  rownames(sr) <- rownames(ar)
  class(sr) <- unique(c("kmsurmiserelation", class(sr)))
  sr
}

#' Compute the basis of a knowledge space
#'
#' \code{kmbasis.matrix} returns a matrix representing the basis of a
#' knowledge space. If \code{x} is a knowledge structure or an
#' arbitrary family of sets \code{kmbasis} returns the basis of
#' the smallest knowledge space containing \code{x}.
#'
#' @param x Binary matrix representing a knowledge space
#'
#' @examples
#' kmbasis(xpl$space)
#'
#' @method kmbasis matrix
#' @rdname kmbasis
#' @family Different representations for knowledge structures
#' @export
kmbasis.matrix <- function(x) {
  if (!inherits(x, "matrix")) {
    stop(sprintf("%s must be of class %s.", dQuote("x"), dQuote("matrix")))
  }
  if (any(x != 1*as.logical(x))) {
    stop(sprintf("%s must be a binary matrix.", dQuote("x")))
  }

  if (dim(x)[1] == 1) {
    if (sum(x) > 0)
      return(x)
    else
      return(NULL)
  }

  noi <- as.integer(dim(x)[2])
  nos <- as.integer(dim(x)[1])
  storage.mode(x) <- "integer"
  nob <- as.integer(0)

  result <- .C("basis_reduction", noi, nos, t(x), nob, package="kstMatrix")
  nob <- result[[4]][1]

  basis <- matrix(1:(noi*nob), ncol=noi, nrow=nob, byrow = TRUE)
  storage.mode(basis) <- "integer"
  result2 <- .C("basis_results", basis, package="kstMatrix")
  b <- matrix(result2[[1]], ncol=noi, nrow=nob, byrow=TRUE)
  colnames(b) <- colnames(x)
  class(b) <- c("kmbasis", "kmfamset", class(basis))
  b
}

#' Determine the quasi-ordinal knowledge space for a surmise relation
#'
#' \code{kmqspace.kmsurmiserelation()} computes the quasi-ordinal
#' knowledge space delineated by a surmise relation
#'
#' @examples
#' sr <- kmsurmiserelation(xpl$space)
#' kmqspace(sr)
#'
#' @family Constructors
#' @rdname kmqspace
#' @method kmqspace kmsurmiserelation
#'
#' @export
kmqspace.kmsurmiserelation <- function(x) {
  b <- kmbasis(x)
  s <- kmunionclosure(b)
  class(s) <- unique(c("kmqspace", class(s)))
  s
}

#' Print KST objects in set notation
#'
#' \code{kmprettyprint()} formats KST objects in set notation for printing.
#' For further details, see below.
#'
#' @param x Object to be printed
#' @param simplified Omit inner braces for families of sets (default FALSE)
#' @param ... Optional further arguments
#' @param open (Outer) opening bracket
#' @param close (Outer) closing bracket
#' @param inneropen Inner opening bracket
#' @param innerclose Inner closing bracket
#'
#' @details
#' If \code{simplified} is TRUE, a state '\{a, b, c\}' is printed as 'abc'.
#' This should only be used if the item names are single letters.
#'
#' For Families of sets (\code{kmfamset} and sub classes), the states are
#' ordered by cardinality.
#'
#' \code{inneropen} and \code{innerclose} are ignored for numeric vectors
#' (i.e. states/plain sets).
#'
#' \code{mapsto} defines the 'mapsto' symbol for pretty-printing a surmise or
#' attribution function. It may contain more than one character, as in the
#' default.
#'
#' For surmise and attribution relations, there are two special parameters.
#' \code{tuple} sets whether the relation shall be printed as set of ordered
#' pairs or whether it shall be printed as tuple of pairs with relation
#' symbol which can  be set with the \code{relation} parameter.
#'
#' For \code{kmlearningpaths} lists, thre result should be printed through
#' \code{cat()} instead of \code{print()} because the latter does not execute
#' the line breaks.
#'
#' Please note that the methods can also be used for sub classes, e.g. the
#' \code{kmprettyprinting.kmfamset()} method is also applicable for bases,
#' knowledge structures, and knowledge spaces.
#'
#' \code{kmprettyprint()} was decidedly not implemented as \code{print()}
#' method. This way, one can _print_ KST objects in matrix or data frame
#' notation or _prettyprint_ them in set notation etc.
#'
#' @examples
#' kmprettyprint(xpl$basis)
#' kmprettyprint(xpl$basis, simplified=TRUE)
#' kmprettyprint(xpl$sr)
#' kmprettyprint(xpl$sf)
#'
#' @family Functions for plotting and printing knowledge structures
#'
#' @export
kmprettyprint <- function(x, simplified=FALSE, ...,
                          open = '{', close = '}',
                          inneropen = '{', innerclose = '}') {
  UseMethod("kmprettyprint")
}


#' @param itemsep Item separating character string
#' @method kmprettyprint numeric
#' @rdname kmprettyprint
#' @export
kmprettyprint.numeric <- function(x, simplified=FALSE, ...,
                                  open = '{', close = '}',
                                  itemsep = ', ') {
  if (is.null(names(x))) {
    stop("'x' must be named.")
  }
  if ((min(x) < 0) || (max(x) > 1))
    stop("'x' must be a binary integer vector.")
  l <- which(x>0)
  v <- names(x)[l]
  if (simplified) {
    open <- ''
    close <- ''
    itemsep <- ''
  }
  paste(c(open, paste(v, collapse=itemsep), close), collapse='')
}

#' @method kmprettyprint kmfamset
#' @rdname kmprettyprint
#' @export
kmprettyprint.kmfamset <- function(x, simplified=FALSE, ...,
                                   open = '{', close = '}',
                                   inneropen = '{', innerclose = '}') {
  famstr <- apply(x, MARGIN=1, function(v) {
    kmprettyprint(v, simplified, open = inneropen, close = innerclose)
  })
  ofamstr <- famstr[order(nchar(famstr))]
  paste(c(open,
          paste(ofamstr,
                collapse=', '),
          close),
        collapse = '')
}


#' @encoding UTF-8
#' @method kmprettyprint kmattributionfunction
#' @param mapsto Mapsto symbol for pretty-printing surmise
#'               and attribution functions
#'               (Default: '\ifelse{html}{\out{&mapsto;}}{\out{$\mapsto$}}')
#' @rdname kmprettyprint
#'
#' @export
kmprettyprint.kmattributionfunction <- function(x,
                                                simplified=FALSE,
                                                ...,
                                                open = '[', close = ']',
                                                inneropen = '{', innerclose = '}',
                                                mapsto = NULL) {
  if (is.null(mapsto))
    mapsto <- ' \u21A6'
  basestrings <- apply(x, MARGIN=1, function(rule) {
    clause <- as.integer(rule[-1])
    names(clause) <- colnames(x)[-1]
    paste(c(rule[1], kmprettyprint(clause, simplified,
                                   open = inneropen,
                                   close = innerclose)), collapse = mapsto)
  })
  paste(c(open,
          paste(basestrings, collapse=', '),
          close), collapse='')
}


#' @encoding UTF-8
#' @method kmprettyprint kmattributionrelation
#' @param tuple Should we use tuple notation (default TRUE)?
#' @param relation Relation symbol for pretty-printing surmise
#'                 and attribution relations
#' @rdname kmprettyprint
#'
#' @export
kmprettyprint.kmattributionrelation <- function(x,
                                                simplified = FALSE,
                                                ...,
                                                open = '{', close = '}',
                                                inneropen = '(', innerclose = ')',
                                                tuple = TRUE,
                                                relation = NULL) {
  if (is.null(relation))
    relation <- ' \u2AAF'
  size <- dim(x)[1]
  major <- lapply(1:size, function(row) {
    lapply(1:size, function(column) {
      if (x[row,column] == 1) {
        if (tuple)
          paste(c(inneropen, colnames(x)[row], ', ', colnames(x)[column], innerclose), collapse='')
        else
          paste(c(colnames(x)[row], ' ', relation, ' ', colnames(x)[column]), collapse='')
      }
    })
  })
  if (tuple)
    paste(c(open,
            paste(unlist(major), collapse=', '),
            close),
          collapse='')
  else
    paste(c(open,
            paste(unlist(major), collapse=', '),
            close),
          collapse='')
}


#' @method kmprettyprint kmdata
#' @rdname kmprettyprint
#' @export
kmprettyprint.kmdata <- function(x, simplified=FALSE, ...,
                                 open = '(', close = ')',
                                 inneropen = '{', innerclose = '}') {
  famstr <- apply(x, MARGIN=1, function(v) {
    kmprettyprint(v, simplified, open = inneropen, close = innerclose)
  })
  paste(c("(",
          paste(famstr,
                collapse=', '),
          ")"),
        collapse = '')
}




#' @method kmprettyprint kmsurmiserelation
#' @export
kmprettyprint.kmsurmiserelation <- kmprettyprint.kmattributionrelation



#' @method kmprettyprint kmsurmisefunction
#' @export
kmprettyprint.kmsurmisefunction <- kmprettyprint.kmattributionfunction


#' @method kmprettyprint kmlearningpath
#' @export
kmprettyprint.kmlearningpath <- function(x, simplified=FALSE, ...,
                                         open = '{', close = '}',
                                         inneropen = '{', innerclose = '}',
                                         path = NULL) {
  if (is.null(path))
    path <- ' \u2192'
  paste(x, collapse = path)
}



#' @param path Rightarrow for pretty-printing path
#'             (Default: '\ifelse{html}{\out{&rarr;}}{\out{$\rightarrow$}}')
#' @method kmprettyprint kmlearningpathlist
#' @rdname kmprettyprint
#' @export
kmprettyprint.kmlearningpathlist <- function(x, simplified=FALSE, ...,
                                             open = '', close = '',
                                             inneropen = '{', innerclose = '}',
                                             path = NULL) {
  if (is.null(path))
    path <- ' \u2192'
  s <- paste(sapply(x, kmprettyprint, path=path, simplified=simplified), collapse=', \n')
  s <- paste0(open, s, close)
  s
}



printlpm <- function(x, simplified=FALSE, path=NULL) {
  if (is.null(path))
    path <- ' \u2192'
  paste(apply(x, MARGIN=1, kmprettyprint, simplified=simplified), collapse=path)
}

#' @param path Rightarrow for pretty-printing path
#'             (Default: '\ifelse{html}{\out{&rarr;}}{\out{$\rightarrow$}}')
#' @method kmprettyprint kmlearningpathmatrices
#' @rdname kmprettyprint
#' @export
kmprettyprint.kmlearningpathmatrices <- function(x, simplified=FALSE, ...,
                                                 open = '', close = '',
                                                 inneropen = '{', innerclose = '}',
                                                 path = NULL) {
  if (is.null(path))
    path <- ' \u2192'
  s <- paste(sapply(x, printlpm, path=path, simplified=simplified), collapse=',\n')
  s <- paste0(open, s, close)
  s
}



#' @param path Rightarrow for pretty-printing path
#'             (Default: '\ifelse{html}{\out{&rarr;}}{\out{$\rightarrow$}}')
#' @method kmprettyprint kmlearningpathmatrix
#' @rdname kmprettyprint
#' @export
kmprettyprint.kmlearningpathmatrix <- function(x, simplified=FALSE, ...,
                                               open = '', close = '',
                                               inneropen = '{', innerclose = '}',
                                               path = NULL) {
  if (is.null(path))
    path <- ' \u2192'
  paste(apply(x, MARGIN=1, kmprettyprint, simplified=simplified), collapse=path)
}

#' Perform a probabilistic knowledge assessment
#'
#' \code{kmassess} performs a probabilistic knowledge assessment for a given
#' response vector, knowledge structure, and BLIM parameters.
#'
#' @param r Response pattern (binary vector)
#' @param pks Probabilistic knowledge structure: a data frame with a
#'    probability distribution in the first columns and the structure
#'    matrix in the subsequent columns.
#' @param questioning Questioning rule ("halfsplit" or "informative")
#' @param update Update rule ("Bayesian" or "multiplicative")
#' @param beta Careless error probabilities (vector)
#' @param eta Lucky guess probabilities (vector)
#' @param zeta0 Vector of update parameters for wrong responses
#' @param zeta1 Vector of update parameters for correct responses
#' @param threshold Probability threshold for stopping criterion. SHould be
#'    larger than 0.5.
#' @param probdev Create colored Hasse diagrams in each step (default FALSE)
#'    and save them to \code{tempdir()}. This requires the installation of
#'    the \code{DiagrammeRsvg} package. Additionally, the development of the
#'    probabilities is stored in the result.
#' @param directory Directory to store diagrams requested through \code{probdev}.
#'    Will be created if non-existent. Default is \code{tempdir()}. If NULL,
#'    no plots will be generated.
#' @return A list with the following elements:
#' \describe{
#'   \item{state}{Diagnosed knowledge state (binary vector)}
#'   \item{probs}{Resulting probability distribution. If probdev is set to TRUE,
#'                a list of probability distributions for each step is given
#'                instead.}
#'   \item{queried}{Sequence of items used in the assessment (list)}
#'   \item{qtime}{Average time for finding a question}
#'   \item{utime}{Average time for updating the probabilities}
#' }
#'
#' @details
#' \code{kmassess} implements the stochastic assessment procedures according
#' to Doignon & Falmagne, 1999, chapter 10.
#'
#' \code{kmassess} stops if the number of questions has reached twice the
#' number of items.
#'
#' @section Background:
#' Doignon & Falmagne (1985, 1999) proposed knowledge space theory originally
#' with adaptive knowledge assessment in mind. The basic idea is to apply
#' prerequisite relationships between items for reducing the number of problems
#' to be posed to a learner in knowledge assessment.
#'
#' Falmagne & Doignon (1988; Doignon & Falmange, 1999, chapte 10) proposed
#' a class of stochastic procdures for such adaptive assessment which take
#' into account that careless errors and lucky guesses may happen during the
#' assessment by estimating a probability distribution over the knowledge
#' structure. Such an assessment consists of three important parts
#' \itemize{
#' \item{Question rule}
#' \item{Update rule}
#' \item{Stopping criterion}
#' }
#'
#' For the **question rule,** they propose the _halfsplit_ and the _infomrative_
#' rules, implemented in \code{kmassesshalfslit} and \code{kmassessinfomrative}.
#'
#' For the **update rule,** they again propose two possibilities there the
#' _multiplicative rule_ is a generalisation of the (classical) _Bayesian
#' update rule_ implemented here in \code{kmassessmultiplicative} and
#' \code{kmassessbayesian}, respectively.
#'
#' As **stopping criterion,** usually a threshold for the maximal probability for
#' one knowledge state is used. It is strongly recommended to keep this larger
#' than 0.5 in order to have one unequivocal resulting state (see also
#' Hockemeyer, 2002).
#'
#' ## Framework of assessment functions within the \pkg{kstMatrix} package:
#' The founding stones are the four aforementioned functions for finding
#' suitable questions and for updating the probability estimates, respectively.
#' They could also be used in an interactive system, e.g. a Shiny app, for
#' "real" adaptive assessment.
#'
#' The remaining three assessment functions serve for mere simulation of
#' adaptive assessment. \code{kmassess} takes, among others, a full response
#' pattern as parameter and takes the responses for the selected questions
#' from this vector. \code{kmsassess} is a simplified version where the
#' update parameters (beta and eta for Bayesian or zeta0 and zeta1 for
#' multiplicative update, respectively) are identical for all items whereas
#' they are item-specific in \code{kmassess}. Finally,
#' \code{kmassesssimulation} takes a whole data set, i.e. a collection of
#' response patterns, and does an assessment for each of these patterns. Its
#' result is a data frame which should be suitable for further statistical
#' evaluation, especially if it is called several times with variant
#' parameters (e.g., structures, update parameters, update and question rules).
#'
#' Both, \code{kmsassess} and \code{kmassesssimulation} call \code{kmassess}.
#'
#' ## Problems
#' In rare cases \code{kmassess} may flip forth and back between probability
#' distributions resulting in an endless loop. Therefore, it stops after
#' twice the number of items delivering a \code{NULL} result.
#'
#' @references
#' Doignon, J.-P. & Falmagne, J.-C. (1985). Spaces for the assessment of
#' knowledge. _International Journal of Man-Machne-Studies, 23,_ 175-196.
#' \doi{10.1016/S0020-7373(85)80031-6}.
#'
#' Doignon, J.-P. & Falmagne, J.-C. (1999). _Knowledge Spaces._ Springer Verlag,
#' Berlin. \doi{10.1007/978-3-642-58625-5}.
#'
#' Falmagne, J.-C. & Doignon, J.-P. (1988). A class of stochastic procedures
#' for the assessment of knowledge. _British Journal of Mathematical and
#' Statistical Psychology, 41,_ 1-23. \doi{10.1111/j.2044-8317.1988.tb00884.x}.
#'
#' Hoxkemeyer, C. (2002). A comparison of non-deterministic procedures for the
#' adaptive assessment of knowledge. _Psychlogische Beiträge, 44(4),_ 495-503.
#'
#' @examples
#' kmassess(c(1, 1, 0, 0),
#'          cbind(as.data.frame(as.matrix(rep(1/9.0, 9), ncol=1)), xpl$space),
#'          "halfsplit",
#'          "Bayesian",
#'          rep(0.12, 4),
#'          rep(0.1, 4),
#'          NULL,
#'          NULL,
#'          0.55
#'         )
#'
#' @importFrom grDevices dev.off jpeg terrain.colors
#' @importFrom DiagrammeR grViz
#' @importFrom rsvg rsvg_png
#'
#' @family Knowledge assessment functions
#' @aliases Assessment
#'
#' @export
kmassess <- function(r,
                     pks,
                     questioning,
                     update,
                     beta,
                     eta,
                     zeta0,
                     zeta1,
                     threshold,
                     probdev = FALSE,
                     directory = tempdir()) {
  debug <- FALSE
  probdevplots <- FALSE
  if (probdev) {
    if (!requireNamespace("DiagrammeRsvg", quietly = TRUE)) {
      warning(sprintf("Plotting stepwise Hasse diagrams requires package 'DiagrammeRsvg'. Setting 'probdev' to FALSE."))
      probdevplots <- FALSE
    } else if (is.null(directory)) probdevplots <- FALSE
    else {
      if (!dir.exists(directory)) {
        if (!dir.create(directory)) probdevplots <- FALSE
      }
      else probdevplots <- TRUE
    }
  }

  problist <- list(pks[,1])
  noi <- dim(pks)[2] - 1
  nos = dim(pks)[1]

  if ((threshold < 0) | (threshold > 1))
    stop("Threshokd must be between 0 and 1.")
  if (threshold <= 0.5)
    warning("Threshold shoud be larger than 0.5!")
  ks <- as.matrix(pks[,(2:(noi+1))])
  class(ks) <- unique(c("kmstructure", "kmfamset", class(ks)))
  storage.mode(ks) <- "integer"
  if ((min(ks) < 0) | (max(ks) > 1))
    stop("Knowledge structure must be a binary matrix.")
  probs <- pks[,1]
  if ((min(probs) < 0) | (max(probs) > 1))
    stop("State probabilities must be between 0 and 1.")
  if (!kmdoubleequal(sum(probs), 1))
    stop("State probabilities must sum up to 1.")
  if (update == "Bayesian") {
    multiplicative <- FALSE
    if (is.null(beta) | is.null(eta))
      stop("beta and eta must be set!")
    if (length(beta) != noi)
      stop("beta and pks do not fit in size.")
    if ((min(beta) < 0) | (max(beta) > 1))
      stop("Illegal beta values.")
    if (length(eta) != noi)
      stop("eta and pks do not fit in size.")
    if ((min(eta) < 0) | (max(eta) > 1))
      stop("Illegal eta values.")
    if (!(is.null(zeta0)) | !(is.null(zeta1)))
      warning("Ignoring zeta values for Bayesian update rule")
  } else if (update == "multiplicative") {
    multiplicative <- TRUE
    if (is.null(zeta0) | is.null((zeta1)))
      stop("zeeta0 and zeta1 must be set.")
    if (length(zeta0) != noi)
      stop("zeta0 and pks do not fit in size")
    if (length(zeta1) != noi)
      stop("zeta1 and pks do not fit in size")
    if ((min(zeta0) <= 1) | (min(zeta1) <= 1))
      stop("zeta0 and zeta1 must be larger than 1")
    if (!(is.null(beta)) | !(is.null(eta)))
      warning("Ignoring beta and eta values for multiplicative update rule")
  } else
    stop("Undefined update rule selection.")

  queried <- c()
  qtime <- c()
  utime <- c()
  if (probdevplots) {
    # plog <- log(probs + 1e-8)
    # pnorm <- (plog - min(plog)) / (max(plog) - min(plog))
    pl <- plot(ks, colors=kmcolors(probs, terrain.colors), method="DiagrammeR")
    rsvg_png(charToRaw(DiagrammeRsvg::export_svg(pl)), paste0(directory, "/assess-fig0.png"))
  }
  while (max(probs) <= threshold) {
    if (questioning == "halfsplit") {
      qtime <- c(qtime, system.time(q <- kmassesshalfsplit(probs, ks))[3])
    } else if (questioning == "informative") {
      qtime <- c(qtime, system.time(q <- kmassessinformative(probs, ks, update, beta, eta, zeta0, zeta1))[3])
    } else
      stop("Undefined questioning rule.")
    queried <- c(queried, q)
    if (length(queried) > 2*noi) {
      warning("Reached twice of number of items as number of questions!")
      qs <- paste(queried, collapse = ", ")
      warning(sprintf("Question sequence: %s", qs))
      return(NULL)
    }

    resp <- r[q]

    if (multiplicative)
      utime <- c(utime, system.time(probs <- kmassessmultiplicative(probs, ks, zeta0, zeta1, q, resp))[3])
    else
      utime <- c(utime, system.time(probs <- kmassessbayesian(probs, ks, beta, eta, q, resp))[3])

    problist <- append(problist, list(probs))
    if (probdevplots) {
      # plog <- log(probs + 1e-10)
      # pnorm <- (plog - min(plog)) / (max(plog) - min(plog))
      fn <- paste0(directory, "/assess-fig", length(queried), ".png")
      pl <- plot(ks, colors=kmcolors(probs, terrain.colors), method="DiagrammeR")
      rsvg_png(charToRaw(DiagrammeRsvg::export_svg(pl)), fn)
    }
    if (debug) {
      print(problist)
      print(sum(probs))
    }
  }
  result <- which(probs == max(probs))
  if (probdev) probs <- problist
  list(state = as.integer(ks[result,]),
       probs = probs,
       queried = queried,
       qtime = sum(qtime)/length(qtime),
       utime = sum(utime)/length(utime)
  )
}

#' Determine the basis of a knowledge space from a surmise relation
#'
#' \code{kmbasis.kmsurmiserelation} takes a surmise relation and returns the
#' corresponding basis.
#'
#' @param x Surmise relation
#'
#' @examples
#' kmbasis(xpl$sr)
#'
#' @method kmbasis kmsurmiserelation
#' @family Different representations for knowledge structures
#' @rdname kmbasis
#' @export
kmbasis.kmsurmiserelation <- function(x) {
  mat <- unique(t(x))
  rownames(mat) <- NULL
  class(mat) <- unique(c("kmbasis", "kmfamset", class(mat)))
  mat
}

kmcompatibility <- function(x,y) {
  if (!inherits(x, "kmstructure"))
    stop("'x' must be a 'kmstructure'.")
  if (!inherits(y, "kmstructure"))
    stop("'y' must be a 'kmstructure'.")
  xitems <- colnames(x)
  yitems <- colnames(y)
  intersection <-  intersect(xitems, yitems)
  if (is.null(intersection) || (length(intersection) <= 1)) return(TRUE)
  xred <- kmsubstructure(x, intersection)
  yred <- kmsubstructure(y, intersection)
  if (any(dim(xred) != dim(yred))) return(FALSE)
  apply(xred, MARGIN=1, function(x) {
    if (!(any(x %in% yred))) return(FALSE)
  })
  return(TRUE)
}


#' Determine the maximal mesh of two knowledge structures
#'
#' If the two structures are not compatible, NULL is returned.
#'
#' @references
#' Falmagne J, Doignon J (1998). “Meshing Knowledge Structures.” In Dowling CE,
#' Roberts FS, Theuns P (eds.), _Recent Progress in Mathematical Psychology,_
#' 143–153. Lawrence Erlbaum Associates Ltd., Mahwah, NJ.
#'
#' @param x First structure
#' @param y Second structure
#' @returns Maximal mesh
#'
#' @family Functions for working with knowledge structures
#'
#' @export
kmmesh <-  function (x, y) {
  if (!inherits(x, "kmstructure"))
    stop("'x' must be a 'kmstructure'.")
  if (!inherits(y, "kmstructure"))
    stop("'y' must be a 'kmstructure'.")
  if (!(kmcompatibility(x,y))) return(NULL)

  fullitems <- unique(c(colnames(x), colnames(y)))
  expx <- matrix(rep(0, nrow(x)*length(fullitems)), nrow=nrow(x))
  colnames(expx) <- fullitems
  sapply(colnames(x), function(i) {
    expx[,i] <<- x[,i]
  })
  expy <- matrix(rep(0, nrow(y)*length(fullitems)), nrow=nrow(y))
  colnames(expy) <- fullitems
  sapply(colnames(y), function(i) {
    expy[,i] <<- y[,i]
  })
  result <- matrix(rep(0, length(fullitems)),ncol=length(fullitems))
  colnames(result) <- fullitems
  apply(expx, MARGIN=1, function(i) {
    hmat <- expy
    sapply((1:dim(hmat)[1]), function(j) {
      hmat[j,] <<- hmat[j,] | i
    })
    result <<- unique(rbind(result, hmat), MARGIN=1)
  })
  kmstructure(kmfamset(result))
}

#' Small example knowledge space
#'
#' Basis and space matrix, surmise relation and surmise function of a small
#' fictional knowledge space, and two data sets (data (7 patterns) and
#' sim (500 patterns)to be used in examples. The latter was produced from
#' the space with \code{kmsimulate()} with beta and eta values of 0.1.
#'
#' @docType data
#'
#' @usage xpl
#'
#' @format A list containing the basis, the space, the surmise relation,
#'         the surmise function, and the two data matrices data and sim.
#'
"xpl"

#' Determine the intersection of two surmise relations
#'
#' Please note that the intersection of surmise relations is automatically
#' a surmise relation without any closure needs.
#'
#' @family Functions for working with knowledge structures
#' @method kmintersection kmsurmiserelation
#' @rdname kmintersection
#' @export
kmintersection.kmsurmiserelation <- function(x, y) {
  if (!(inherits(y, "kmsurmiserelation")))
    stop("'y' must also be a kmsurmiserelation object.")
  noix <- dim(x)[2]
  noiy <- dim(y)[2]
  if (noix != noiy)
    stop("Objects do not fit in number of items.")
  if (is.null(colnames(x))) {
    if (!(is.null(colnames(y))))
      stop("Item names do not fit.")
  } else if (is.null(colnames(y)))
    stop("Item names do not fit.")
  else if (any(colnames(x) != colnames(y)))
    stop("Item names do not fit")
  res <-matrix(as.numeric(x&y), ncol=noix)
  class(res) <- unique(c("kmsurmiserelation", class(res)))
  colnames(res) <- colnames(x)
  rownames(res) <- rownames(x)
  res
}

#' Determine a color vector based on probabilities
#'
#' \code{kmcolors} takes a probabilty vector and a color palette and
#' creates a color vector to be used with \code{kstMatrix::plot}.
#'
#' @param prob Probability vector
#' @param palette Color palette (default = cm.colors)
#'
#' @family Functions for plotting and printing knowledge structures
#'
#' @importFrom grDevices cm.colors
#'
#' @export
kmcolors <- function(prob, palette=cm.colors) {
  palette(1001)[1001-1000*prob]
}

---
title: "kstMatrix"
author: "Cord Hockemeyer"
date: "`r Sys.Date()`"
output: html
vignette: >
  %\VignetteIndexEntry{kstMatrix}
  %\VignetteEngine{litedown::vignette}
  %\VignetteEncoding{UTF-8}

---

```{r setup, include = FALSE}
# knitr::opts_chunk$set(
#   collapse = TRUE,
#   comment = "#>"
# )
library(kstMatrix)
library(grDevices)
library(sets)
library(pks)
```

# Content

- [Introduction](#introduction)
- [S3 classes in `kstMatrix`](#s3-classes-in-kstmatrix)
- [Different representations for knowledge spaces](#different-representations-for-knowledge-spaces)
- [Propertiesd of knowledge spaces](#propertiesd-of-knowledge-spaces)
- [Creating trivial knowledge spaces](#creating-trivial-knowledge-spaces)
- [Simulating response patterns](#simulating-response-patterns)
- [Validating knowledge structures](#validating-knowledge-structures)
- [Assessment](#assessment)
- [Fringes and learning paths](#fringes-and-learning-paths)
- [Plotting](#plotting)
- [Utilities](#utilities)
- [Datasets provided by `kstMatrix`](#datasets-provided-by-kstmatrix)
- [References](#references)

# Introduction
Knowledge space theory applies prerequisite relationships between items of knowledge
within a given domain for efficient adaptive assessment and training (Doignon & Falmagne, 
1999). The `kstMatrix` package implements some basic functions for working with
knowledge space. Furthermore, it provides several empirically obtained knowledge spaces
in form of their bases.

There is a certain overlap in functionality between the `kst`and `kstMatrix` packages, 
however the former uses a set representation and the latter a matrix representation. The
packages are to be seen as complementary, not as a replacement for each other.

This document gives a short overview on the `kstMatrix` package. For more details on the specific functions, please have a look at the manual.

Subsequently, we start with a introduction into the S3 classes used in `kstMatrix`. Afterwards,
the various functions are presented groupd by topic.

# S3 classes in `kstMatrix`
With version 2, S3 classes were introduced in `kstMatrix`. The following classes are used:

* `kmfamset`
* `kmbasis`
* `kmstructure`
* `kmspace`
* `kmneighbourhood`
* `kmsurmiserelation`
* `kmsurmisefunction`

The following figure shows the dependencies between these classes, e.g., `kmspace` being a sub-class of `kmstructure`.

<style>
  img {
    border: 0;
  }
</style>
![](kstMatrixClasses.png)
```{r kstMatrixClasses, echo=FALSE, fig.cap="Structure of <tt>kstMatrix</tt> classes", out.width="700"}
# litedown::include_graphics("kstMatrixClasses.png")
```

## Constructors
There exist four constructors

- `kmfamset()`
- `kmstructure()`
- `kmbasis()`
- `kmspace()`

which create the respective objects from a given matrix.

# Different representations for knowledge spaces
There exist various ways to represent knowledge spaces: the whole space itself, its basis, and the corresponding
surmise function; in case of quasi-ordinal spaces also the surmise relation. The subsequent functions allow to 
map between these representations.

## `kmbasis()`
The `kmbasis()` S3 method allows to ap different representations to bases. There are implementations for the classes
`kmsurmiserelation`, `kmsurmisefunction`, and `matrix`, the latter aiming at `kmfamset`s including `kmstructure`s 
and `kmspace`s.

## `kmsf2basis()`
This function determines a basis from a surmise function.

## `kmSR2basis()`
This function creates a basis from a surmise relation.

## `kmsurmisefunction()`
This function takes an arbitrary `kmfamset` (family of sets) and determines the surmise function of the smallest
knowledge space containing this family of sets, i.e. its closure under union.

## `kmsurmiserelation()`
This function determines the surmise relation for a quasi-ordinal knowledge space. If the parameter is an arbitrary
`kmfamset`, the result is the surmise relation for the smalest quasi-ordinal knowledge space containing the family 
of sets, i.e. its closure under union *and* intersection.

## `kmunionclosure()`
This function returns the closure under union (i.e. a knowledge space) of a family of sets.

# Properties of knowledge spaces
There are `kmeqreduction()` and `kmnotions()` for dealing with equivalent items. The former reduces a famset
to the structure on its notions, and the latter gives the notions itself.

Another point is the weelgradedness - `kmiswellgraded()`tests if a structure fulfils this condition.

# Creating trivial knowledge spaces
For a given item number, there are two trivial knowledge spaces, the maximal knowledge
space representing absolutely no prerequisite relationships (the knowledge space is the 
power set of the item set and the basis matrix is the diagonal matrix), and the minimal
knowledge space representing equivalence of all items (the knowledge space contains just 
the empty set and the full item set, and the basis matrix contains one line full of '1's).

# Simulating response patterns
Response patterns are simulated by `kmsimulate()` applying the BLIM (Basic Local
Independence Model). It assumes identical $\beta$ and $\eta$ values for all items. 

More elaborated functions may follow.

# Validating knowledge structures
There exist two core functions for validation. `kmvalidate()` determines validity indices based on the states in the
knowledge structure, and `kmSRvalidate()`determines indices based on the surmise relation of a quasi-ordinal knowledge space.
Furthermore, there is a helpüer function `kmdist()` ehich returns a distance distribution.

# Assessment
There exist several functions in the assessment context. They are all based on a paper by Falmagne & Doignon (1988,
see also Doignon & Falmagne, 1999, Chapter 10).

The core functiuon is `kmassess()`. It does a complete assessmednt for a given response vector and a given 
knowledge structure. A simplified version is `kmsassess()` where the originally item-specific parameters are identical
for all items.

Both these functions rely on alternative question and update rules realised by `kmassesshalfsplit()` and
`kmassessinfomrative()` and by `kmassessbayesian()` and `kmassessmultipöicatiuve()`, respectively. The separation of 
these functions allows also for a use in an interactive system, e.g., a Shiny app.

Finally, there is `kmassessmentsimulation()` which does assessments for a whole dataset of response patterns and 
produces a table which could be further evaluated statistically.

# Fringes and Learning Paths
Well-graded learning paths are an important concept in knowledge space theory. With `kmlearningpaths()`, we obtain a 
collection of all learning paths in a knowledge structure, with `kmgradations()` a collection of all gradations
between two states in a structure. In this context also `kmiswellgraded()` should be mentioned which tests if a
structure is well-graded.

In this context, we may also need fringes and neighbourhoods. The former can be determined with `kmfringe()`, `kminnerfringe()`,
and `kmouterfringe()`. For the latter, there are the functions `kmneighbourhood()` (determining the 1-neighbourhood of a state)
and `kmnneighbourhood()` (determining an arbitrary n-neighbourhood).

There exists also an alternative implementation with the functions `kmbasisfringe()`, `kmbasisinnerfringe()`, and
`kmbasisouterfringe()`, as well as `kmbasisneighbourhood()`. These functions work on the basis of a knowledge space and
may, thus, be faster, especially for large knowledge spaces (see Hockemeyer, 1997). Please note that, as a consequence 
of using the basis, these functions do not apply for general knowledge structures but only for knowledge spaces. 
Furthermore, due to complexity, there is no `kmbasisnneighbourhood()`(yet).

# Plotting 
With version 2.0  of `kstMatrix`, plotting functionality is provided through the S3 `plot` 
method. The method is available for the `kmfamset`,  `kmsurmiserelation`, and `kmneighbourhood` classes.
```{r}
plot(xpl$space)
plot(xpl$sr)
```
`plot` uses the `DiagrammeR` package which in turn uses the `Graphviz` software. The default parameters are slightly different for `kmfamset`s and `kmsurmiserelation`s. 
```{r}
plot(xpl$sr, colors="#aaffaa", vertexshape="box")
```
For plotting neighbourhoods, by default three different colors are used.
```{r}
plot(kmneighbourhood(c(1,1,0,0), xpl$space, include=TRUE), state=c(1,1,0,0), edgelabel=TRUE)
```
By default, Hasse diagrams are drawn vertically with just lines as edges. However, on finds also horizontal diagrams
in which case the arrows are set as arrows from left to right.
```{r}
plot(xpl$space, horizontal=TRUE, arrowtail="open")
plot(xpl$space, arrowhead="empty")
```

# Utilities
This group contains several smaller helper functions.

- `kmsymmsetdiff()`
- `kmsetdistance()`
- `kmsetiselement()`
- `kmdoubleequal()`
- `kmcolors()`

# Datasets provided by `kstMatrix`
## Empirical structures
The provided empirical datasets were obtained by the research group around Cornelia Dowling through querying experts
in the respective fields. 

### cad
Six experts were queried about prerequisite relationships between 28 AutoCAD
knowledge items (Dowling, 1991; 1993a). A seventh basis represents those
prerequisite relationships on which the majority (4 out of 6) of the experts
agree (Dowling & Hockemeyer, 1998).

### fractions
Three experts were queried about prerequisite relationships between 77 items
on fractions (Baumunk & Dowling, 1997). A fourth basis represents those
prerequisite relationships on which the majority of the experts agree
(Dowling & Hockemeyer, 1998).

### readwrite
Three experts were queried about prerequisite relationships between 48 items
on reading and writing abilities (Dowling, 1991; 1993a). A fourth basis represents
those prerequisite relationships on which the majority of the experts agree
(Dowling & Hockemeyer, 1998).

## Example structures
### phsg
A small knowledge space on a set of seven items on linear functions. This example
is used in a manuscript by Steiner et al.


### xpl
This is just a small fictitious 4-item-example used for the examples in the
documentation. Please note that this knowledge space is not quasi-ordinal.

# References
- Baumunk, K. & Dowling, C. E. (1997). Validity of spaces for assessing knowledge 
  about fractions. _Journal of Mathematical Psychology, 41,_ 99–105.
- Doignon, J.-P. & Falmagne, J.-C. (1999). _Knowledge Spaces._ Springer–Verlag, Berlin.
- Dowling, C. E. (1991). _Constructing Knowledge Structures from the Judgements of Experts._ 
  Habilitationsschrift, Technische Universität Carolo-Wilhelmina, Braunschweig, Germany.
- Dowling, C. E. (1993a). Applying the basis of a knowledge space for controlling the
  questioning of an expert. _Journal of Mathematical Psychology, 37,_ 21–48.
- Dowling, C. E. (1993b). On the irredundant construction of knowledge spaces.
  Journal of Mathematical Psychology, 37, 49–62.
- Dowling, C. E. & Hockemeyer, C. (1998). Computing the intersection of knowledge spaces using only 
  their basis. In Cornelia E. Dowling, Fred S. Roberts, & Peter Theuns, editors,
  _Recent Progress in Mathematical Psychology,_ pp. 133–141. Lawrence Erlbaum
  Associates Ltd., Mahwah, NJ.
- Falmagne J.-C. & Doignon J.-P. (1988). A class of stochastic procedures for the assessment of knowledge. 
  _British Journal of Mathematical and Statistical Psychology, 41,_ 1–23. 
- Hockemeyer C (1997). Using the Basis of a Knowledge Space for Determining the Fringe of a Knowledge State. 
  _Journal of Mathematical Psychology, 41,_ 275–279. 

---
title: "kstMatrix"
author: "Cord Hockemeyer"
date: "Version 3.0-0"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{kstMatrix}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

<style type="text/css">
  body {
    font-family: Helvetica;
    font-size: 15pt;
  }
  h1,h2,h3,h4,h5,h6 {
    font-family: Arial;
  }
</style>

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
library(kstMatrix)
library(grDevices)
library(sets)
library(pks)
library(Rgraphviz)
```

# Content

- [Introduction](#introduction)
- [S3 classes in `kstMatrix`](#s3-classes-in-kstmatrix)
    - [Constructors](#constructors)
- [Different representations for knowledge spaces](#different-representations-for-knowledge-spaces)
- [Working with knowledge structures](#working-with-knowledge-structures)
    - [Creating trivial knowledge spaces](#creating-trivial-knowledge-spaces)
    - [Determining variants](#determining-variants)
        - [Closure operatorsa](#closure-operators)
        - [Dealing with equivalent items](#dealing-with-equivalent-items)
        - [Combining knowledge structures](#combining-knowledge-structures)
        - [Reducing and expanding structures](#reducing-and-expanding-structures)
- [Working with data](#working-with-data)
    - [Simulating response patterns](#simulating-response-patterns)
    - [Generating knowledge structures from data](#generating-knowledge-structures-from-data)
    - [Validating knowledge structures](#validating-knowledge-structures)
- [Assessment](#assessment)
- [Fringes and learning paths](#fringes-and-learning-paths)
- [Plotting and pretty-printing](#plotting-and-pretty-printing)
    - [Creating color vectors](#creating-color-vectors)
    - [Pretty-printing](#pretty-printing)
- [Utilities](#utilities)
- [Datasets provided by `kstMatrix`](#datasets-provided-by-kstmatrix)
    - [Empirical structures](#empirical-structures)
    - [Example structures](#example-structures)
- [Removed and obsoleted functions and methods](#removed-and-obsoleted-functions-and-methods)
- [References](#references)

# Introduction
Knowledge space theory applies prerequisite relationships between items of knowledge
within a given domain for efficient adaptive assessment and training (Doignon & Falmagne, 
1999). The `kstMatrix` package implements some basic functions for working with
knowledge space. Furthermore, it provides several empirically obtained knowledge spaces
in form of their bases.

There is a certain overlap in functionality between the `kst`and `kstMatrix` packages, 
however the former uses a set representation and the latter a matrix representation. The
packages are to be seen as complementary, not as a replacement for each other.

This document gives a short overview on the `kstMatrix` package. For more details on the specific functions, please have a look at the manual.

Subsequently, we start with a introduction into the S3 classes used in `kstMatrix`. Afterwards,
the various functions are presented groupd by topic.

# S3 classes in `kstMatrix`
With version 2, S3 classes were introduced in `kstMatrix` most of which are in some hierarchy. 
A short explanation of the various classes:

- `kmdata`: A matrix of response patterns. These may be either empirical or simulated.
- `kmfamset`: A family of sets, i.e. of knowledge states. Due to the set character, any row may occur only once.
- `kmbasis`: Basis of a knowledge space; sub-class of `kmfamset`.
- `kmstructure`: Knowledge structure; a `kmfamset` which contains teh empty set and the full item set as states.
- `kmspace`: Knowledge space; a `kmstructure` which is closed under union.
- `kmqspace`: Quasi-ordinal knowledge space; a `kmspace` which is additionally closed under intersection.
- `kmneidghbourhood`: The neighbourhood of an individual knowledge state; sub-class of `kmfamset`.
- `kmattributionrelation`: Incidence matrix of an attribution relation; please note that here the rows contain the minimal states.
- `kmsurmiserelation`: Closure of a `kmattributionrelation` under reflexivity and transitivity.
- `kmattributionfunction`: Data frame describing an attribution function.
- `kmsurmisefunction`: Closure of a `kmattributionfunction` under extended reflexivity, transitivity, and incomparability.

The following classes are isolated from the general class hierarchy.

- `kmlearningpath`: A list of printed knowledge states building a gradation.
- `kmlearningpaths`: A list of `kmlearningpath` objects.
- `kmlearningpathmatrix`: A matrix whose rows describe the states within a gradation. Technically also a family of sets.
- `kmlearningpathmatrices`: A list of `kmlearningpathmatrix` objects.


Figure 1 shows the dependencies between the former group of classes, e.g., `kmspace` being a sub-class of `kmstructure`. 
Besides the `kstMatrix` classes in blue, also the underlying standard `R` classes are shown in green. 

```{r classes, fig.cap = 'Figure 1: kstMatrix object classes', echo = FALSE, fig.align = 'center', cache=FALSE, out.width='95%'}
knitr::include_graphics("kstMatrix-Classes.png")
```

## Constructors
There exist several constructors

- `kmfamset()`
- `kmstructure()`
- `kmbasis()`
- `kmspace()`
- `kmqspace()`

which create the respective objects from various KST objects.

# Different representations for knowledge spaces
There exist various ways to represent knowledge spaces: the whole space itself, its basis, and the corresponding
surmise function; in case of quasi--ordinal spaces also the surmise relation. The subsequent functions allow to 
map between these representations.

## `kmbasis()`
The `kmbasis()` S3 method allows to map different representations to bases. There are implementations for the classes
`kmsurmiserelation`, `kmsurmisefunction`, and `matrix`, the latter aiming at `kmfamset`s including `kmstructure`s 
and `kmspace`s.

## `kmspace()`
Beyond being regarded as constructors, certain `kmspace()` S3 methods include mappings betwen different representations
of knowledge structures.

## `kmsurmisefunction()`
This function takes an arbitrary `kmfamset` (family of sets) and determines the surmise function of the smallest
knowledge space containing this family of sets, i.e. its closure under union.

## `kmsurmiserelation()`
This function determines the surmise relation for a quasi--ordinal knowledge space. If the parameter is an arbitrary
`kmfamset`, the result is the surmise relation for the smallest quasi--ordinal knowledge space containing the family 
of sets, i.e. its closure under union *and* intersection.

# Working with knowledge structures
## Creating trivial knowledge spaces
For a given item number, there are two trivial knowledge spaces, the maximal knowledge
space representing absolutely no prerequisite relationships (the knowledge space is the 
power set of the item set and the basis matrix is the diagonal matrix), and the minimal
knowledge space representing equivalence of all items (the knowledge space contains just 
the empty set and the full item set, and the basis matrix contains one line full of '1's).
These spaces can be generated with `kmmaximalspace()` and `kmminimalspace()`, respectively. 
Both functions generate the whole space, not the basis.

## Determining variants
### Closure Operators
Several functions provide closures for respective data.

- `kmclosure()` is the closure method providing closures
    - from an attribution relation (i.e. an arbitrary binary relation on the
      set of items) to a surmise relation, and
    - from an attribution function to a surmise function.
- `kmunionclosure()` closes a family of sets under union, i.e. it determines
  the corresponding knowledge space. `kmspace()` does the same.
- `kmqspace()` close an arbitrary family of sets to a quasi--ordinal knowledge
  space. `kmintersectionclosure()` does the same restricted to `kmspace` objects.

### Dealing with equivalent items
`kmnotions()` determines classes of equivalent items, `kmeqreduction()` reduces a family of states to a notion-free
subset of items.

`kmrefinenotion()` dissolves notions by specifying a structure within the notion. Each basis element containing 
the notion is replaced by a family of states structured by a second basis on the equivalent items.

### Combining knowledge structures
`kmunion()` computes the "union" of knowledge structures, surmise relations, or surmise functions. `kmintersection()` 
analogously computes their "intersection". Both functions return an object of the same type. Please keep in mind that 
a union of knowledge spaces corresponds to an intersection of the respective surmise relations or surmise functions 
and vice versa.

`kmmesh()` determines the maximal mesh of two knowledge structures.

### Reducing and expanding structures
`kmsubstructure()` reduces knowledge structure representations to a subset of items. It is an S3 method applicable for
data, bases, structures, spaces, surmise relations, and surmise functions. The result is of the same object class as the
original, i.e. the respective properties are checked/enforced.

`kmexpand()`does the opposite: it expands an attribution function to a superset of items and closes it to a surmise function.
The additional items are independent, i.e. they neither have nor are prerequisites.

# Working with data
## Simulating response patterns
Response patterns are simulated by `kmsimulate()` applying the BLIM (Basic Local
Independence Model). It assumes identical $\beta$ and $\eta$ values for all items. 

More elaborated functions may follow.

## Generating knowledge structures from data
`kmgenerate()` offers a trivial, straightforward approach to generating knowledge structures from data:
any response pattern with an observed frequency beyond a given threshold is taken as a knowledge state.

Furthermore, there exists the `kmiita2SR()` function linking to the <a href="https://cran.r-project.org/package=DAKS" target="_blank">`DAKS`</a> package. It determines the surmise relation 
(in `kstMatrix` data representation form) from an `iita` object generated with the `DAKS` functions.

## Validating knowledge structures
There exist two core functions for validation. `kmvalidate()` determines validity indices based on the states in the
knowledge structure, and `kmSRvalidate()`determines indices based on the surmise relation of a quasi--ordinal knowledge space.
Furthermore, there is a helper function `kmdist()` which returns a distance distribution.

# Assessment
There exist several functions in the assessment context. They are all based on a paper by Falmagne & Doignon (1988,
see also Doignon & Falmagne, 1999, Chapter 10).

The core functiuon is `kmassess()`. It does a complete assessmednt for a given response vector and a given 
knowledge structure. A simplified version is `kmsassess()` where the originally item-specific parameters are identical
for all items.

Both these functions rely on alternative question and update rules realised by `kmassesshalfsplit()` and
`kmassessinfomrative()` and by `kmassessbayesian()` and `kmassessmultipöicatiuve()`, respectively. The separation of 
these functions allows also for a use in an interactive system, e.g., a Shiny app.

Finally, there is `kmassessmentsimulation()` which does assessments for a whole dataset of response patterns and 
produces a table which can then be further evaluated statistically.

# Fringes and Learning Paths
Well-graded learning paths are an important concept in knowledge space theory. With `kmlearningpaths()`, we obtain a 
collection of all learning paths in a knowledge structure, with `kmgradations()` a collection of all gradations
between two states in a structure. In this context also `kmiswellgraded()` should be mentioned which tests if a
structure is well-graded.

In this context, we may also need fringes and neighbourhoods. The former can be determined with `kmfringe()`, `kminnerfringe()`,
and `kmouterfringe()`. For the latter, there are the functions `kmneighbourhood()` (determining the 1-neighbourhood of a state)
and `kmnneighbourhood()` (determining an arbitrary n-neighbourhood).

There exists also an alternative implementation with the functions `kmbasisfringe()`, `kmbasisinnerfringe()`, and
`kmbasisouterfringe()`, as well as `kmbasisneighbourhood()`. These functions work on the basis of a knowledge space and
may, thus, be faster, especially for large knowledge spaces (see Hockemeyer, 1997). Please note that, as a consequence 
of using the basis, these functions do not apply for general knowledge structures but only for knowledge spaces. 
Furthermore, due to complexity, there is no `kmbasisnneighbourhood()`(yet).

# Plotting and pretty-printing
With version 2.0  of `kstMatrix`, plotting functionality is provided through the S3 `plot` 
method. The method is available for the `kmfamset`,  `kmsurmiserelation`, and `kmneighbourhood` classes.
```{r, eval=FALSE}
plot(xpl$space)
plot(xpl$sr)
```
```{r, fig.cap = 'Figure 2: Example space', echo = FALSE, fig.align = 'center', cache=FALSE, out.width='450'}
knitr::include_graphics("space.jpeg")
```
```{r, fig.cap = 'Figure 3: Example surmise relation', echo = FALSE, fig.align = 'center', cache=FALSE, out.width='200'}
knitr::include_graphics("SR.jpeg")
```

`plot` uses the `DiagrammeR` package which in turn uses the `Graphviz` software. The default parameters are slightly different for `kmfamset`s and `kmsurmiserelation`s. 
```{r, eval=FALSE}
plot(xpl$sr, colors="orange", vertexshape="circle")
```

For plotting neighbourhoods, by default three different colors are used.
```{r, eval=FALSE}
plot(kmneighbourhood(c(1,1,0,0), xpl$space, include=TRUE), 
     state=c(1,1,0,0), edgelabel=TRUE)
```
```{r neighbourhood, fig.cap = 'Figure 4: Neighbourhood', echo = FALSE, fig.align = 'center', cache=FALSE, out.width='400'}
knitr::include_graphics("neighbourhood.jpeg")
```

Alternatively, one can also plot the whole structure and highlight a certain state and its neighbourhood within the structure.
Here, four colors are used.
```{r eval=FALSE}
plot(xpl$space, state=c(1,1,0,0))
```
```{r neighbourhood_space, fig.cap = 'Figure 5: Neighbourhood within a knowledge space', echo = FALSE, fig.align = 'center', cache=FALSE, out.width='650'}
knitr::include_graphics("neighbourhood2.png")
```

By default, Hasse diagrams are drawn vertically with just lines as edges. However, on finds also horizontal diagrams
in which case the arrows are set as arrows from left to right. Please note that, for technical reasons, you should specify the `arrowhead` for horizontal but the `arrowtail` for vertical Hasse diagrams if you want arrows instead of plain lines.
```{r, eval=FALSE}
plot(xpl$space, horizontal=TRUE, arrowtail="open")
plot(xpl$space, arrowhead="empty")
```
```{r horizontal, fig.cap = 'Figure 6: Horizontal Hasse diagram', echo = FALSE, fig.align = 'center', cache=FALSE, out.width='800'}
knitr::include_graphics("horizontal.jpeg")
```
```{r emptyhead, fig.cap = 'Figure 7: Using arrows instead of lines', echo = FALSE, fig.align = 'center', cache=FALSE, out.width='400'}
knitr::include_graphics("emptyhead.jpeg")
```

## Creating color vectors
`kmcolors()` creates a colors vector for plot() based on an existing palette.

## Pretty-printing
The `kmprettyprinting()` method prints various KST object types in set notation (or similar).
```{r prettyprinting}
kmprettyprint(xpl$basis)
kmprettyprint(xpl$basis, simplified=TRUE)
kmprettyprint(xpl$sr,tuple=FALSE)
cat(kmprettyprint(kmlearningpaths(xpl$space)))
```

# Utilities
This group contains several smaller helper functions.

- `binarymatrixproduct()` computes the Boolean multiplaction of two binary
  matrices.
- `kmsymmsetdiff()`: Determine the symmetric set difference between two sets.
- `kmsetdistance()`: Determine the cardinality of the symmetric set difference
- `kmsetiselement()`: Test if an item is element of a state/set
- `kmminimalfamset()`: Determine the minimal states within a `kmfamset`
- `kmdoubleequal()`: Test if two doubles are (almost) equal thus avoiding 
  rounding problems with the '==' operator
- [`kmcolors()`](#sec:creating-color-vectors): Produce a color vector for 
  `plot()` based on existing palettes
- [`kmtrivial()`](#sec:creating-trivial-knowledge-spaces): Create a minimal
  or a maximal knowledge space for a given number of items
- `kmheights()`: Determine a table of items and their _heights_ in the 
  knowledge structure/space

# Datasets provided by `kstMatrix`
## Empirical structures
The provided empirical datasets were obtained by the research group around Cornelia Dowling through querying experts
in the respective fields. 

### cad
Six experts were queried about prerequisite relationships between 28 AutoCAD
knowledge items (Dowling, 1991; 1993a). A seventh basis represents those
prerequisite relationships on which the majority (4 out of 6) of the experts
agree (Dowling & Hockemeyer, 1998).

### fractions
Three experts were queried about prerequisite relationships between 77 items
on fractions (Baumunk & Dowling, 1997). A fourth basis represents those
prerequisite relationships on which the majority of the experts agree
(Dowling & Hockemeyer, 1998).

### readwrite
Three experts were queried about prerequisite relationships between 48 items
on reading and writing abilities (Dowling, 1991; 1993a). A fourth basis represents
those prerequisite relationships on which the majority of the experts agree
(Dowling & Hockemeyer, 1998).

## Example structures
### phsg
A small knowledge space on a set of seven items on linear functions. This example
is used in a manuscript by Steiner et al.


### xpl
This is just a small fictitious 4-item-example used for the examples in the
documentation. Please note that this knowledge space is not quasi--ordinal.

# Removed and obsoleted functions and methods
With the never-ending addition of new functions and methods, is sometimes becomes necessary 
to re-structure the interface.

## Removed/renamed functions with `kstMatrix` version 2.0-0
The functions `kmhasse()`, `kmbasisdiagram()`, and `kmSRdiagram()` were 
moved/renamed into S3 `plot()` methods for the respective object classes.

## Removed functions and methods with `kstMatrix` version 3.0-0
The following functions and methods were removed:
- The `kmunionclosure()` methods has been merged into a new `kmclosure()` 
  S3 method which also offers closure methods for attribution relations and 
  attribution functions.
- The `kmSF2basis()` and `kmSR2basis()` functions have been removed after their 
  integration into the `kmbasis()` S3 method with the earlier version 2.0-0.

# References
- Baumunk, K. & Dowling, C. E. (1997). Validity of spaces for assessing knowledge 
  about fractions. _Journal of Mathematical Psychology, 41,_ 99–105.
- Doignon, J.-P. & Falmagne, J.-C. (1999). _Knowledge Spaces._ Springer–Verlag, Berlin.
- Dowling, C. E. (1991). _Constructing Knowledge Structures from the Judgements of Experts._ 
  Habilitationsschrift, Technische Universität Carolo-Wilhelmina, Braunschweig, Germany.
- Dowling, C. E. (1993a). Applying the basis of a knowledge space for controlling the
  questioning of an expert. _Journal of Mathematical Psychology, 37,_ 21–48.
- Dowling, C. E. (1993b). On the irredundant construction of knowledge spaces.
  Journal of Mathematical Psychology, 37, 49–62.
  their basis. In Cornelia E. Dowling, Fred S. Roberts, & Peter Theuns, editors,
  _Recent Progress in Mathematical Psychology,_ pp. 133–141. Lawrence Erlbaum
  Associates Ltd., Mahwah, NJ.
- Falmagne J.-C. & Doignon J.-P. (1988). A class of stochastic procedures for the assessment of knowledge. 
  _British Journal of Mathematical and Statistical Psychology, 41,_ 1–23. 
- Hockemeyer C (1997). Using the Basis of a Knowledge Space for Determining the Fringe of a Knowledge State. 
  _Journal of Mathematical Psychology, 41,_ 275–279. 

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmbasisneighbourhood.R
\name{kmbasisneighbourhood}
\alias{kmbasisneighbourhood}
\title{Compute the neighbourhod of a state within a knowledge structure using its basis}
\usage{
kmbasisneighbourhood(state, basis, include = FALSE)
}
\arguments{
\item{state}{Binary vector representing a knowledge state}

\item{basis}{\code{kmbasis} object}

\item{include}{Boolean whether the original state should be included
in the result (default FALSE)}
}
\value{
Matrix containing the neighbouring states, one per row
}
\description{
\code{kmbasisneighbourhood} computes the neighbourhood of a state
within a knowledge structure, i.e. the family of all other
states with a symmetric set difference of 1.
}
\examples{
kmbasisneighbourhood(c(1,1,0,0), xpl$basis)

}
\seealso{
Other Functions for fringes & neighbourhoods:
\code{\link[=kmbasisfringe]{kmbasisfringe()}},
\code{\link[=kmfringe]{kmfringe()}},
\code{\link[=kmneighbourhood]{kmneighbourhood()}}
}
\concept{Functions for fringes & neighbourhoods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmsubstructure.R
\name{kmsubstructure}
\alias{kmsubstructure}
\alias{kmsubstructure.kmdata}
\alias{kmsubstructure.kmbasis}
\alias{kmsubstructure.kmstructure}
\alias{kmsubstructure.kmspace}
\alias{kmsubstructure.kmsurmiserelation}
\alias{kmsubstructure.kmsurmisefunction}
\title{Reduce a KST object to a subset of items}
\usage{
kmsubstructure(x, items)

\method{kmsubstructure}{kmdata}(x, items)

\method{kmsubstructure}{kmbasis}(x, items)

\method{kmsubstructure}{kmstructure}(x, items)

\method{kmsubstructure}{kmspace}(x, items)

\method{kmsubstructure}{kmsurmiserelation}(x, items)

\method{kmsubstructure}{kmsurmisefunction}(x, items)
}
\arguments{
\item{x}{KST object}

\item{items}{Selection of items (vector) to which the KST object shall be reduced}
}
\value{
Sub structure KST object (the kstMatrix class and its properties
are preserved)

\code{kmsubstructure()} takes a KST object and determines a sub-structure
based on the list of items.

The items can be specified through their item names or through the respective
column numbers. At least two items must be specified.
}
\description{
Reduce a KST object to a subset of items
}
\examples{
phsg$basis
kmsubstructure(phsg$basis, c("I1", "I2", "I5"))

}
\seealso{
Other Functions for working with knowledge structures:
\code{\link[=kmeqreduction]{kmeqreduction()}},
\code{\link[=kmexpand]{kmexpand()}},
\code{\link[=kmintersection]{kmintersection()}},
\code{\link[=kmmesh]{kmmesh()}},
\code{\link[=kmnotions]{kmnotions()}},
\code{\link{kmtrivial}},
\code{\link[=kmunion]{kmunion()}}
}
\concept{Functions for working with knowledge structures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readwrite.R
\docType{data}
\name{readwrite}
\alias{readwrite}
\title{Knowledge spaces on reading and writing abilities}
\format{
A list containing four bases (rw1 to rw3, and rwmaj) in binary
matrix form. Each matrix has 48 columns representing the different knowledge
items and a varying number of rows containing the basis elements.
}
\usage{
readwrite
}
\description{
Bases of knowledge spaces on reading/writing abilities obtained from querying
experts.
}
\details{
Three experts were queried about prerequisite relationships between 48 items
on reading and writing abilities (Dowling, 1991; 1993). A forth basis represents
those prerequisite relationships on which the majority of the experts agree
(Dowling & Hockemeyer, 1998).
}
\references{
Dowling, C. E. (1991). \emph{Constructing Knowledge Structures from
the Judgements of Experts.} Habilitationsschrift, Technische Universität
Carolo-Wilhelmina, Braunschweig, Germany.

Dowling, C. E. (1993). Applying the basis of a knowledge space for controlling
the questioning of an expert. \emph{Journal of Mathematical Psychology, 37,} 21–48.

Dowling, C. E. & Hockemeyer, C. (1998). Computing the intersection of knowledge
spaces using only their basis. In Cornelia E. Dowling, Fred S. Roberts, & Peter
Theuns, editors, \emph{Recent Progress in Mathematical Psychology,} pp. 133–141.
Lawrence Erlbaum Associates Ltd., Mahwah, NJ.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmdoubleequal.R
\name{kmdoubleequal}
\alias{kmdoubleequal}
\title{Test two double numbers on equity with a certain tolerance}
\usage{
kmdoubleequal(x, y, tol = sqrt(.Machine$double.eps))
}
\arguments{
\item{x}{First double to compare}

\item{y}{Second double to compare}

\item{tol}{Tolerance optional)}
}
\value{
Boolean for (approximate) equity
}
\description{
Test two double numbers on equity with a certain tolerance
}
\examples{
kmdoubleequal(0.5+0.5, 1)
0.5 + 0.5 == 1

}
\seealso{
Other Utilities:
\code{\link[=binary_matrix_product]{binary_matrix_product()}},
\code{\link[=kmminimalfamset]{kmminimalfamset()}},
\code{\link[=kmsetiselement]{kmsetiselement()}},
\code{\link[=kmsymmsetdiff]{kmsymmsetdiff()}},
\code{\link{kmtrivial}}
}
\concept{Utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Deprecated.R
\name{kmSR2basis}
\alias{kmSR2basis}
\title{DEPRECATED: \code{kmSR2basis()} has been replaced by \code{kmbasis()}.}
\usage{
kmSR2basis(x)
}
\arguments{
\item{x}{Surmise relation}
}
\value{
Corresponding basis
}
\description{
DEPRECATED: \code{kmSR2basis()} has been replaced by \code{kmbasis()}.
}
\seealso{
Other Deprecated functions and methods:
\code{\link[=kmSF2basis]{kmSF2basis()}},
\code{\link[=kmSRdiagram]{kmSRdiagram()}},
\code{\link[=kmbasisdiagram]{kmbasisdiagram()}},
\code{\link[=kmhasse]{kmhasse()}}
}
\concept{Deprecated functions and methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fractions.R
\docType{data}
\name{fractions}
\alias{fractions}
\title{Knowledge spaces on fractions}
\format{
A list containing four bases (frac1 to frac3, and fracmaj) in binary
matrix form. Each matrix has 77 columns representing the different knowledge
items and a varying number of rows containing the basis elements.
}
\usage{
fractions
}
\description{
Bases of knowledge spaces on fractions obtained from querying experts.
}
\details{
Three experts were queried about prerequisite relationships between 77 items
on fractions (Baumunk & Dowling, 1997). A forth basis represents those
prerequisite relationships on which the majority of the experts agree
(Dowling & Hockemeyer, 1998).
}
\references{
Baumunk, K. & Dowling, C. E. (1997). Validity of spaces for assessing
knowledge about fractions. \emph{Journal of Mathematical Psychology, 41,} 99–105.

Dowling, C. E. & Hockemeyer, C. (1998). Computing the intersection of knowledge
spaces using only their basis. In Cornelia E. Dowling, Fred S. Roberts, & Peter
Theuns, editors, \emph{Recent Progress in Mathematical Psychology,} pp. 133–141.
Lawrence Erlbaum Associates Ltd., Mahwah, NJ.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmgradations.R
\name{kmgradations}
\alias{kmgradations}
\title{Determine all gradations between two states}
\usage{
kmgradations(structure, from = NULL, to = NULL)
}
\arguments{
\item{structure}{Knowledge structure}

\item{from}{Starting state (if NULL (default), it is the empty set)}

\item{to}{Goal state (if NULL (default), it is the full item set)}
}
\value{
A list of gradations where each gradation is a list of states
}
\description{
Determine all gradations between two states
}
\examples{
kmgradations(xpl$space)


}
\seealso{
Other Functions for wellgradedness & learning paths:
\code{\link[=kmiswellgraded]{kmiswellgraded()}},
\code{\link[=kmlearningpathmatrices]{kmlearningpathmatrices()}},
\code{\link[=kmlearningpaths]{kmlearningpaths()}}
}
\concept{Functions for wellgradedness & learning paths}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmiswellgraded.R
\name{kmiswellgraded}
\alias{kmiswellgraded}
\title{Check for wellgradedness of a knowledge structure}
\usage{
kmiswellgraded(x)
}
\arguments{
\item{x}{\code{kmspace} or \code{kmbasis}}
}
\value{
Logical value specifying whether \code{x} is well-graded
}
\description{
\code{kmiswellgraded()} returns whether a knowledge space (or its basis)
is well-graded.
}
\examples{
kmiswellgraded(xpl$space)

}
\references{
Doignon, J.-P. & Falmagne, J.-C. (1999). \emph{Knowledge Spaces.}
Springer–Verlag, Berlin.
}
\seealso{
Other Properties of knowledge structures:
\code{\link[=kmheights]{kmheights()}},
\code{\link[=kmnotions]{kmnotions()}}

Other Functions for wellgradedness & learning paths:
\code{\link[=kmgradations]{kmgradations()}},
\code{\link[=kmlearningpathmatrices]{kmlearningpathmatrices()}},
\code{\link[=kmlearningpaths]{kmlearningpaths()}}
}
\concept{Functions for wellgradedness & learning paths}
\concept{Properties of knowledge structures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmqspace.R, R/kmqspace.kmfamset.R,
%   R/kmqspace.kmsurmisefunction.R, R/kmqspace.kmsurmiserelation.R
\name{kmqspace}
\alias{kmqspace}
\alias{kmqspace.kmfamset}
\alias{kmqspace.kmsurmisefunction}
\alias{kmqspace.kmsurmiserelation}
\title{Obtain a quasi-ordinal knowledge space}
\usage{
kmqspace(x)

\method{kmqspace}{kmfamset}(x)

\method{kmqspace}{kmsurmisefunction}(x)

\method{kmqspace}{kmsurmiserelation}(x)
}
\arguments{
\item{x}{KST object}
}
\value{
Corresponding \code{kmqspace} object
}
\description{
\code{kmqspace()} computes the  quasi-ordinal space
corresponding to a given KST object.

\code{kmqspace()} computes the smallest quasi-ordinal space
containing a given family of sets.

\code{kmqspace.kmsurmisefunction()} takes a surmise function and
determines the quasi-ordinal knowledge space which is the closure
under intersection of the knowledge space delineated by this surmise
function.

\code{kmqspace.kmsurmiserelation()} computes the quasi-ordinal
knowledge space delineated by a surmise relation
}
\examples{
m <- kmfamset(matrix(c(1,0,0,0,1,0,1,1,1), nrow=3, byrow=TRUE))
kmqspace(m)

m <- kmfamset(matrix(c(1,0,0,0,1,0,1,1,1), nrow=3, byrow=TRUE))
kmqspace(m)

sr <- kmsurmiserelation(xpl$space)
kmqspace(sr)

}
\seealso{
Other Constructors:
\code{\link[=kmfamset]{kmfamset()}},
\code{\link[=kmspace]{kmspace()}},
\code{\link[=kmstructure]{kmstructure()}}
}
\concept{Constructors}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmsetiselement.R
\name{kmsetiselement}
\alias{kmsetiselement}
\title{Test if a state is contained in a family of states}
\usage{
kmsetiselement(s, f)
}
\arguments{
\item{s}{State}

\item{f}{Family of sets}
}
\value{
Boolean is s is contained in f
}
\description{
Test if a state is contained in a family of states
}
\examples{
kmsetiselement(c(1,1,1,0), xpl$space)

}
\seealso{
Other Utilities:
\code{\link[=binary_matrix_product]{binary_matrix_product()}},
\code{\link[=kmdoubleequal]{kmdoubleequal()}},
\code{\link[=kmminimalfamset]{kmminimalfamset()}},
\code{\link[=kmsymmsetdiff]{kmsymmsetdiff()}},
\code{\link{kmtrivial}}
}
\concept{Utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmneighbourhood.R
\name{kmnneighbourhood}
\alias{kmnneighbourhood}
\title{Compute the n-neighbourhod of a state within a knowledge structure}
\usage{
kmnneighbourhood(state, struct, distance, include = FALSE)
}
\arguments{
\item{state}{Binary vector representing a knowledge state}

\item{struct}{Binary matrix representing a knowledge structure}

\item{distance}{Size of the n-neighbourhood}

\item{include}{Boolean whether the original state should be included
(defaul FALSE)}
}
\value{
Matrix containing the neighbouring states, one per row
}
\description{
\code{kmnneighbourhood} computes the n-neighbourhood of a state
within a knowledge structure, i.e. the family of all other
states with a symmetric set difference maximal n.
}
\examples{
kmnneighbourhood(c(1,1,0,0), xpl$space, 2)

}
\seealso{
Other Fringes & learning paths: 
\code{\link{kmbasisfringe}()},
\code{\link{kmbasisneighbourhood}()},
\code{\link{kmfringe}()},
\code{\link{kmgradations}()},
\code{\link{kmlearningpaths}()},
\code{\link{kmneighbourhood}()}
}
\concept{Fringes & learning paths}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot}
\alias{plot}
\alias{plot.kmfamset}
\alias{plot.kmneighbourhood}
\alias{plot.kmsurmiserelation}
\alias{plot.kmlearningpathmatrices}
\alias{plot.kmlearningpathmatrix}
\alias{plot.kmsurmisefunction}
\title{Plot a Hasse diagram}
\usage{
\method{plot}{kmfamset}(
  x,
  ...,
  state = NULL,
  horizontal = FALSE,
  colors = NULL,
  keepNames = TRUE,
  itemsep = ",",
  braces = TRUE,
  vertexshape = "oval",
  arrowhead = "none",
  arrowtail = "none",
  edgelabel = FALSE,
  verbose = 0
)

\method{plot}{kmneighbourhood}(
  x,
  ...,
  horizontal = FALSE,
  colors = c("#eeee00", "#aaccff", "#bbffbb"),
  keepNames = TRUE,
  itemsep = ",",
  braces = TRUE,
  vertexshape = "oval",
  arrowhead = "none",
  arrowtail = "none",
  edgelabel = FALSE,
  state = NULL,
  verbose = 0
)

\method{plot}{kmsurmiserelation}(
  x,
  ...,
  horizontal = FALSE,
  colors = NULL,
  keepNames = TRUE,
  vertexshape = "circle",
  arrowhead = "none",
  arrowtail = "none",
  verbose = 0
)

\method{plot}{kmlearningpathmatrices}(
  x,
  ...,
  horizontal = FALSE,
  colors = NULL,
  keepNames = TRUE,
  itemsep = ",",
  braces = TRUE,
  vertexshape = "oval",
  arrowhead = "none",
  arrowtail = "none",
  edgelabel = FALSE,
  verbose = 0,
  structure = NULL,
  pathcolors = c("red", "blue", "gold", "deeppink", "brown", "violet", "green")
)

\method{plot}{kmlearningpathmatrix}(
  x,
  ...,
  horizontal = FALSE,
  colors = NULL,
  keepNames = TRUE,
  itemsep = ",",
  braces = TRUE,
  vertexshape = "oval",
  arrowhead = "none",
  arrowtail = "none",
  edgelabel = FALSE,
  verbose = 0,
  structure = NULL,
  pathcolor = "blue"
)

\method{plot}{kmsurmisefunction}(
  x,
  ...,
  space = FALSE,
  horizontal = FALSE,
  colors = "Beige",
  keepNames = TRUE,
  itemsep = ",",
  braces = TRUE,
  vertexshape = "oval",
  arrowhead = "none",
  arrowtail = "none",
  edgelabel = FALSE,
  verbose = 0,
  highlightcolor = "red"
)
}
\arguments{
\item{x}{Binary matrix representing a family of sets}

\item{...}{Optional inherited parameters}

\item{state}{Knowledge state whose neighbourhood is to be pictured
(Default `NULL``)

If \code{state} is \code{NULL}, the neighbourhood is depicted as a 'normal'
family of sets (\code{kmfamset}) and a respective warning is issued.}

\item{horizontal}{Boolean defining orientation of the graph, default FALSE}

\item{colors}{Color value or vector (default NULL).}

\item{keepNames}{Keep item names (default TRUE)}

\item{itemsep}{Item separator in sets (default ','; only for families
of states)}

\item{braces}{Put braces around vertices (default TRUE; only for
families of states)}

\item{vertexshape}{Shape of the vertex objects, e.g. circle, oval, box, or none. See
\href{https://graphviz.org/doc/info/shapes.html}{Graphviz Node Shapes}
for a complete list of possible values.}

\item{arrowhead}{Form of the arrow head, e.g. vee or none (default). See
\href{https://graphviz.org/docs/attr-types/arrowType/}{Graphviz Arrow
Types} for a complete list of possible values. This may be used for vertical
graphs although none is the standard there.}

\item{arrowtail}{Form of the arrow tail, e.g. vee or none (default). See
\href{https://graphviz.org/docs/attr-types/arrowType/}{Graphviz Arrow
Types} for a complete list of possible values. This should be used for
horizontal graphs.}

\item{edgelabel}{Boolean whether to label the edges of the diagram
(default FALSE)}

\item{verbose}{Verbosity level (0 (default), 1, or 2)}

\item{structure}{Knowledge structure within which the path shall be depicted}

\item{pathcolors}{Colors for the individual learning paths}

\item{pathcolor}{Colors for the individual learning paths (default: blue)}

\item{space}{Should basis or space de depicted? (default FALSE)}

\item{highlightcolor}{Color for highlighting}
}
\description{
\code{plot} draws a Hasse diagram for a family of sets or a surmise relation.
}
\details{
\code{plot} takes a matrix representing a family of sets (knowledge states)
or a surmise relation and a color vector, and draws a Hasse diagram.
If the color vector is NULL the states are drawn in green, the items in
the relation are drawn in orange.

For a surmise relation, if there are equivalent items, they are contracted
into one vertex labelled 'a ~ b ~ ...' for equivalent items a, b, ...

If the plot is to be used within a Shiny app, it must be included with
\code{grVizOutput()} and \code{renderGrViz()} from the \code{DiagrammeR}
package (\code{plotOutput()} and \code{renderPlot()} do not work).

Please note that, for equivalent items in a relation plot, the prerequisites
are lost.

If the KST object to be plotted is a family of sets (\code{kmfamset}) \strong{and}
a \code{state} is specified, the \code{colors} vector must have four elements, (i)
the color for the state, (ii) the color for its upper neighbours, (iii)
the color for its lower neighbours, and (iv) the color for all other states.
}
\examples{
\dontrun{
plot(phsg$basis)

sp <- kmunionclosure(phsg$basis)
n <- kmneighbourhood(phsg$basis[3,], sp, include=TRUE)
plot(n, state=phsg$basis[3,])

m <- matrix(c(1,0,0,1,1,1,1,1,1), ncol=3, byrow=FALSE)
class(m) <- unique(c("kmsurmiserelation", class(m)))
plot(m, vertexshape="oval")
}

}
\seealso{
Other Functions for plotting and printing knowledge structures:
\code{\link[=kmcolors]{kmcolors()}},
\code{\link[=kmprettyprint]{kmprettyprint()}}
}
\concept{Functions for plotting and printing knowledge structures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xpl.R
\docType{data}
\name{xpl}
\alias{xpl}
\title{Small example knowledge space}
\format{
A list containing the basis, the space, the surmise relation,
the surmise function, and the two data matrices data and sim.
}
\usage{
xpl
}
\description{
Basis and space matrix, surmise relation and surmise function of a small
fictional knowledge space, and two data sets (data (7 patterns) and
sim (500 patterns)to be used in examples. The latter was produced from
the space with \code{kmsimulate()} with beta and eta values of 0.1.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmprettyprint.R
\encoding{UTF-8}
\name{kmprettyprint}
\alias{kmprettyprint}
\alias{kmprettyprint.numeric}
\alias{kmprettyprint.kmfamset}
\alias{kmprettyprint.kmattributionfunction}
\alias{kmprettyprint.kmattributionrelation}
\alias{kmprettyprint.kmdata}
\alias{kmprettyprint.kmlearningpathlist}
\alias{kmprettyprint.kmlearningpathmatrices}
\alias{kmprettyprint.kmlearningpathmatrix}
\title{Print KST objects in set notation}
\usage{
kmprettyprint(
  x,
  simplified = FALSE,
  ...,
  open = "{",
  close = "}",
  inneropen = "{",
  innerclose = "}"
)

\method{kmprettyprint}{numeric}(
  x,
  simplified = FALSE,
  ...,
  open = "{",
  close = "}",
  itemsep = ", "
)

\method{kmprettyprint}{kmfamset}(
  x,
  simplified = FALSE,
  ...,
  open = "{",
  close = "}",
  inneropen = "{",
  innerclose = "}"
)

\method{kmprettyprint}{kmattributionfunction}(
  x,
  simplified = FALSE,
  ...,
  open = "[",
  close = "]",
  inneropen = "{",
  innerclose = "}",
  mapsto = NULL
)

\method{kmprettyprint}{kmattributionrelation}(
  x,
  simplified = FALSE,
  ...,
  open = "{",
  close = "}",
  inneropen = "(",
  innerclose = ")",
  tuple = TRUE,
  relation = NULL
)

\method{kmprettyprint}{kmdata}(
  x,
  simplified = FALSE,
  ...,
  open = "(",
  close = ")",
  inneropen = "{",
  innerclose = "}"
)

\method{kmprettyprint}{kmlearningpathlist}(
  x,
  simplified = FALSE,
  ...,
  open = "",
  close = "",
  inneropen = "{",
  innerclose = "}",
  path = NULL
)

\method{kmprettyprint}{kmlearningpathmatrices}(
  x,
  simplified = FALSE,
  ...,
  open = "",
  close = "",
  inneropen = "{",
  innerclose = "}",
  path = NULL
)

\method{kmprettyprint}{kmlearningpathmatrix}(
  x,
  simplified = FALSE,
  ...,
  open = "",
  close = "",
  inneropen = "{",
  innerclose = "}",
  path = NULL
)
}
\arguments{
\item{x}{Object to be printed}

\item{simplified}{Omit inner braces for families of sets (default FALSE)}

\item{...}{Optional further arguments}

\item{open}{(Outer) opening bracket}

\item{close}{(Outer) closing bracket}

\item{inneropen}{Inner opening bracket}

\item{innerclose}{Inner closing bracket}

\item{itemsep}{Item separating character string}

\item{mapsto}{Mapsto symbol for pretty-printing surmise
and attribution functions
(Default: '\ifelse{html}{\out{&mapsto;}}{\out{$\mapsto$}}')}

\item{tuple}{Should we use tuple notation (default TRUE)?}

\item{relation}{Relation symbol for pretty-printing surmise
and attribution relations}

\item{path}{Rightarrow for pretty-printing path
(Default: '\ifelse{html}{\out{&rarr;}}{\out{$\rightarrow$}}')}
}
\description{
\code{kmprettyprint()} formats KST objects in set notation for printing.
For further details, see below.
}
\details{
If \code{simplified} is TRUE, a state '\{a, b, c\}' is printed as 'abc'.
This should only be used if the item names are single letters.

For Families of sets (\code{kmfamset} and sub classes), the states are
ordered by cardinality.

\code{inneropen} and \code{innerclose} are ignored for numeric vectors
(i.e. states/plain sets).

\code{mapsto} defines the 'mapsto' symbol for pretty-printing a surmise or
attribution function. It may contain more than one character, as in the
default.

For surmise and attribution relations, there are two special parameters.
\code{tuple} sets whether the relation shall be printed as set of ordered
pairs or whether it shall be printed as tuple of pairs with relation
symbol which can  be set with the \code{relation} parameter.

For \code{kmlearningpaths} lists, thre result should be printed through
\code{cat()} instead of \code{print()} because the latter does not execute
the line breaks.

Please note that the methods can also be used for sub classes, e.g. the
\code{kmprettyprinting.kmfamset()} method is also applicable for bases,
knowledge structures, and knowledge spaces.

\code{kmprettyprint()} was decidedly not implemented as \code{print()}
method. This way, one can \emph{print} KST objects in matrix or data frame
notation or \emph{prettyprint} them in set notation etc.
}
\examples{
kmprettyprint(xpl$basis)
kmprettyprint(xpl$basis, simplified=TRUE)
kmprettyprint(xpl$sr)
kmprettyprint(xpl$sf)

}
\seealso{
Other Functions for plotting and printing knowledge structures:
\code{\link[=kmcolors]{kmcolors()}},
\code{\link{plot}}
}
\concept{Functions for plotting and printing knowledge structures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmtrivial.R
\name{kmtrivial}
\alias{kmtrivial}
\alias{kmminimalspace}
\alias{kmmaximalspace}
\title{Create trivial knowledge spaces}
\usage{
kmminimalspace(noi)

kmmaximalspace(noi)
}
\arguments{
\item{noi}{Number of items}
}
\value{
A binary matrix representing the respective knowledge space
}
\description{
These functions create trivial knowledge spaces of a given item number. The minimal
space contains just the empty set and the full item set while the maximal space is
equal to the power set.
}
\details{
Please note that the computation time for creating large power sets can grow quite
large easily.
}
\examples{
kmminimalspace(3)
kmmaximalspace(3)

}
\seealso{
Other Utilities:
\code{\link[=binary_matrix_product]{binary_matrix_product()}},
\code{\link[=kmdoubleequal]{kmdoubleequal()}},
\code{\link[=kmminimalfamset]{kmminimalfamset()}},
\code{\link[=kmsetiselement]{kmsetiselement()}},
\code{\link[=kmsymmsetdiff]{kmsymmsetdiff()}}

Other Functions for working with knowledge structures:
\code{\link[=kmeqreduction]{kmeqreduction()}},
\code{\link[=kmexpand]{kmexpand()}},
\code{\link[=kmintersection]{kmintersection()}},
\code{\link[=kmmesh]{kmmesh()}},
\code{\link[=kmnotions]{kmnotions()}},
\code{\link[=kmsubstructure]{kmsubstructure()}},
\code{\link[=kmunion]{kmunion()}}
}
\concept{Functions for working with knowledge structures}
\concept{Utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmmesh.R
\name{kmmesh}
\alias{kmmesh}
\title{Determine the maximal mesh of two knowledge structures}
\usage{
kmmesh(x, y)
}
\arguments{
\item{x}{First structure}

\item{y}{Second structure}
}
\value{
Maximal mesh
}
\description{
If the two structures are not compatible, NULL is returned.
}
\references{
Falmagne J, Doignon J (1998). “Meshing Knowledge Structures.” In Dowling CE,
Roberts FS, Theuns P (eds.), \emph{Recent Progress in Mathematical Psychology,}
143–153. Lawrence Erlbaum Associates Ltd., Mahwah, NJ.
}
\seealso{
Other Functions for working with knowledge structures:
\code{\link[=kmeqreduction]{kmeqreduction()}},
\code{\link[=kmexpand]{kmexpand()}},
\code{\link[=kmintersection]{kmintersection()}},
\code{\link[=kmnotions]{kmnotions()}},
\code{\link[=kmsubstructure]{kmsubstructure()}},
\code{\link{kmtrivial}},
\code{\link[=kmunion]{kmunion()}}
}
\concept{Functions for working with knowledge structures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmvalidate.R
\name{kmvalidate}
\alias{kmvalidate}
\title{Validate a knowledge structure against a data set}
\usage{
kmvalidate(data, struct)
}
\arguments{
\item{data}{Binary matrix representing a set of response patterns}

\item{struct}{Binary matrix representing a knowledge structure}
}
\value{
A list with three elements:
\describe{
\item{dist}{Distance distribution vector}
\item{DI}{Discrepancy Index}
\item{DA}{Distance Agreement Coefficient}
}
}
\description{
\code{kmvalidate} returns a list with three elements,
a named vector (dist) with the frequencies of distances
between a set of response patterns and a knowledge structure, the
Discrepancy Index (DI), and the Distance Agreement Coefficient (DA).
}
\section{Warning}{

The DA computation can take quite some time for larger item sets as the
power set has to be computed. For item sets with around 30 items or more,
it may even crash the system due to huge memory requests.
}

\examples{
kmvalidate(xpl$data, xpl$space)

}
\seealso{
Other Functions for working with data:
\code{\link[=kmSRvalidate]{kmSRvalidate()}},
\code{\link[=kmdist]{kmdist()}},
\code{\link[=kmgenerate]{kmgenerate()}},
\code{\link[=kmiita2SR]{kmiita2SR()}},
\code{\link[=kmsimulate]{kmsimulate()}}
}
\concept{Functions for working with data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmsimulate.R
\name{kmsimulate}
\alias{kmsimulate}
\title{Simulate a set of response patterns according to the BLIM}
\usage{
kmsimulate(x, n, beta, eta)
}
\arguments{
\item{x}{Binary matrix representing a knowledge space}

\item{n}{Number of simulated response patterns}

\item{beta}{Careless error probability value or vector}

\item{eta}{Lucky guess probability value or vector}
}
\value{
Binary matrix representing the simulated data set
}
\description{
\code{kmsimulate} returns a data set of \code{n} simulated response patterns based
on the knowledge structure \code{x} given as a binary matrix. The simulation follows
the BLIM (Basic Local Independence Model; see Doigon & Falmagne, 1999).
}
\details{
The \code{beta} and \code{eta} parameters must be either single numericals
or vectors with a length identical to the number of rows in the \code{x} matrix.
A mixture is possible.

The \code{sample} function used by \code{kmsimulate} might work inaccurately for knowledge
structures \code{x} with 2^31 or more states.
}
\examples{
kmsimulate(xpl$space, 50, 0.2, 0.1)
kmsimulate(xpl$space, 50, c(0.2, 0.25, 0.15, 0.2), c(0.1, 0.15, 0.05, 0.1))
kmsimulate(xpl$space, 50, c(0.2, 0.25, 0.15, 0.2), 0)

}
\references{
Doignon, J.-P. & Falmagne, J.-C. (1999). \emph{Knowledge Spaces.}
Springer–Verlag, Berlin.
}
\seealso{
Other Functions for working with data:
\code{\link[=kmSRvalidate]{kmSRvalidate()}},
\code{\link[=kmdist]{kmdist()}},
\code{\link[=kmgenerate]{kmgenerate()}},
\code{\link[=kmiita2SR]{kmiita2SR()}},
\code{\link[=kmvalidate]{kmvalidate()}}
}
\concept{Functions for working with data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmdist.R
\name{kmdist}
\alias{kmdist}
\title{Compute the distance between a data set and a knowledge structure}
\usage{
kmdist(data, struct)
}
\arguments{
\item{data}{Binary matrix representing a set of response patterns}

\item{struct}{Binary matrix representing a knowledge structure}
}
\value{
Distance distribution vector
}
\description{
\code{kmdist} returns a named vector with the frequencies of distances
between a set of response patterns and a knowledge structure. This vector
can be used to compute, e.g., the Discrepancy Index (DI) or the
Distance Agreement Coefficient (DA).
}
\examples{
kmdist(xpl$data, xpl$space)

}
\seealso{
Other Functions for working with data:
\code{\link[=kmSRvalidate]{kmSRvalidate()}},
\code{\link[=kmgenerate]{kmgenerate()}},
\code{\link[=kmiita2SR]{kmiita2SR()}},
\code{\link[=kmsimulate]{kmsimulate()}},
\code{\link[=kmvalidate]{kmvalidate()}}
}
\concept{Functions for working with data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/binarymatrixproduct.R
\name{binary_matrix_product}
\alias{binary_matrix_product}
\title{Compute a binary matrix product}
\usage{
binary_matrix_product(m, n)
}
\arguments{
\item{m}{Binary matrix}

\item{n}{Binary matrix}
}
\value{
Boolean matrix product of m and n
}
\description{
\code{binary_matrix_product} expects two binary matrices and computes their Boolean product.
}
\examples{
binary_matrix_product(xpl$sr, kmsurmiserelation(kmmaximalspace(4)))
binary_matrix_product(xpl$sr, kmsurmiserelation(kmminimalspace(4)))


}
\seealso{
Other Utilities:
\code{\link[=kmdoubleequal]{kmdoubleequal()}},
\code{\link[=kmminimalfamset]{kmminimalfamset()}},
\code{\link[=kmsetiselement]{kmsetiselement()}},
\code{\link[=kmsymmsetdiff]{kmsymmsetdiff()}},
\code{\link{kmtrivial}}
}
\concept{Utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmassessinformative.R
\name{kmassessinformative}
\alias{kmassessinformative}
\title{Determine next question for probabilistic knowledge assessment}
\usage{
kmassessinformative(probs, ks, update, beta, eta, zeta0, zeta1)
}
\arguments{
\item{probs}{Probability distribution over the knowledge structure (vector)}

\item{ks}{Binary matrix of the knowledge structure}

\item{update}{Update rule ("Bayesian" or "multiplicative")}

\item{beta}{Careless error probabilities (vector)}

\item{eta}{Lucky guess probabilities (vector)}

\item{zeta0}{Vector of update parameters for wrong responses}

\item{zeta1}{Vector of update parameters for correct responses}
}
\value{
Number of the selected question
}
\description{
\code{kmassessinformative} determines the next question in a probabiliststic
assessment according to the informative rule.
}
\examples{
kmassessinformative(c(0.02, 0.1, 0.07, 0.01, 0.4, 0.17, 0.07, 0.08, 0.08),
                  xpl$space,
                  "Bayesian",
                  rep(0.3,4),
                  rep(0.2,4),
                  NULL,
                  NULL
                 )

}
\seealso{
Other Knowledge assessment functions:
\code{\link[=kmassess]{kmassess()}},
\code{\link[=kmassessbayesian]{kmassessbayesian()}},
\code{\link[=kmassesshalfsplit]{kmassesshalfsplit()}},
\code{\link[=kmassessmentsimulation]{kmassessmentsimulation()}},
\code{\link[=kmassessmultiplicative]{kmassessmultiplicative()}}
}
\concept{Knowledge assessment functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmsrvalidate.R
\name{kmSRvalidate}
\alias{kmSRvalidate}
\title{Validate a surmise relation against a data set}
\usage{
kmSRvalidate(data, sr)
}
\arguments{
\item{data}{Binary matrix representing a set of response patterns}

\item{sr}{Binary matrix representing a surmise relation}
}
\value{
A list with two elements:
\describe{
\item{gamma}{Goodman & Kruskal's gamma index}
\item{vc}{Violational Coefficient}
}
}
\description{
\code{kmSRvalidate} returns a list with two elements,
Goodman & Kruskal's gamma value and the violational
coefficient (VC).
}
\examples{
kmSRvalidate(xpl$data, xpl$sr)

}
\seealso{
Other Functions for working with data:
\code{\link[=kmdist]{kmdist()}},
\code{\link[=kmgenerate]{kmgenerate()}},
\code{\link[=kmiita2SR]{kmiita2SR()}},
\code{\link[=kmsimulate]{kmsimulate()}},
\code{\link[=kmvalidate]{kmvalidate()}}
}
\concept{Functions for working with data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmintersectionclosure.R
\name{kmintersectionclosure}
\alias{kmintersectionclosure}
\title{Close a knowledge space under intersection}
\usage{
kmintersectionclosure(x)
}
\arguments{
\item{x}{Knowledge space}
}
\value{
Quasi--ordinal knowledge space
}
\description{
Close a knowledge space under intersection
}
\examples{
kmintersectionclosure(xpl$space)

}
\seealso{
Other Closure operators:
\code{\link[=kmclosure]{kmclosure()}},
\code{\link[=kmunionclosure]{kmunionclosure()}}
}
\concept{Closure operators}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmexpand.R
\name{kmexpand}
\alias{kmexpand}
\title{Expand an attribution function to a larger set of items and close it to a surmise function}
\usage{
kmexpand(af, ni)
}
\arguments{
\item{af}{Attribution function}

\item{ni}{New (complete) item list}
}
\value{
Expanded surmise function
}
\description{
\code{kmexpand()} expands an attribution function by additional items and
closes it to a surmise function.
}
\examples{
xpl$sf
kmexpand(xpl$sf, c("a", "b", "c", "d", "e", "f"))


}
\seealso{
Other Functions for working with knowledge structures:
\code{\link[=kmeqreduction]{kmeqreduction()}},
\code{\link[=kmintersection]{kmintersection()}},
\code{\link[=kmmesh]{kmmesh()}},
\code{\link[=kmnotions]{kmnotions()}},
\code{\link[=kmsubstructure]{kmsubstructure()}},
\code{\link{kmtrivial}},
\code{\link[=kmunion]{kmunion()}}
}
\concept{Functions for working with knowledge structures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmheights.R
\name{kmheights}
\alias{kmheights}
\title{Compute the heights of items in a knowledge space specified through its basis}
\usage{
kmheights(b)
}
\arguments{
\item{b}{Basis}
}
\value{
Data.frame with a table of heights.
}
\description{
The height of an item in the knowledge space is defined as the minimal
cardinality of the states containing it minus 1, i.e. the minimal number
of prerequisites.
}
\examples{
kmheights(xpl$basis)

}
\seealso{
Other Properties of knowledge structures:
\code{\link[=kmiswellgraded]{kmiswellgraded()}},
\code{\link[=kmnotions]{kmnotions()}}
}
\concept{Properties of knowledge structures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmeqreduction.R
\name{kmeqreduction}
\alias{kmeqreduction}
\title{Reduce a family of knowledge states with respect to item equivalence}
\usage{
kmeqreduction(x)
}
\arguments{
\item{x}{Family of sets (\code{kmfamset} object)}
}
\value{
Binary matrix reduced by equivalences
}
\description{
\code{kmeqreduction} takes a family of knowledge states and returns its
reduction to non-equivalent items.
}
\examples{
kmeqreduction(xpl$space)

}
\seealso{
Other Functions for working with knowledge structures:
\code{\link[=kmexpand]{kmexpand()}},
\code{\link[=kmintersection]{kmintersection()}},
\code{\link[=kmmesh]{kmmesh()}},
\code{\link[=kmnotions]{kmnotions()}},
\code{\link[=kmsubstructure]{kmsubstructure()}},
\code{\link{kmtrivial}},
\code{\link[=kmunion]{kmunion()}}
}
\concept{Functions for working with knowledge structures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmassesshalfsplit.R
\name{kmassesshalfsplit}
\alias{kmassesshalfsplit}
\title{Determine next question for probabilistic knowledge assessment}
\usage{
kmassesshalfsplit(probs, ks)
}
\arguments{
\item{probs}{Probability distribution over the knowledge structure (vector)}

\item{ks}{Binary matrix of the knowledge structure}
}
\value{
Number of the selected question
}
\description{
\code{kmassesshalfsplit} determines the next question in a probabilistic
assessment according to the halfsplit rule.
}
\examples{
kmassesshalfsplit(c(0.02, 0.1, 0.07, 0.01, 0.4, 0.17, 0.07, 0.08, 0.08),
                  xpl$space)

}
\seealso{
Other Knowledge assessment functions:
\code{\link[=kmassess]{kmassess()}},
\code{\link[=kmassessbayesian]{kmassessbayesian()}},
\code{\link[=kmassessinformative]{kmassessinformative()}},
\code{\link[=kmassessmentsimulation]{kmassessmentsimulation()}},
\code{\link[=kmassessmultiplicative]{kmassessmultiplicative()}}
}
\concept{Knowledge assessment functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmassesssimulation.R
\name{kmassessmentsimulation}
\alias{kmassessmentsimulation}
\title{Simulate assessments for a set of response patterns}
\usage{
kmassessmentsimulation(
  respdata,
  ks,
  questioning,
  update,
  beta,
  eta,
  zeta0,
  zeta1,
  threshold
)
}
\arguments{
\item{respdata}{Data matrix}

\item{ks}{Knowledge structure}

\item{questioning}{Question rule}

\item{update}{Updating rule}

\item{beta}{Careless error probability}

\item{eta}{Lucky guess probability}

\item{zeta0}{Update parameter for wrong responses}

\item{zeta1}{Update parameter for correct responses}

\item{threshold}{Stopping criterion}
}
\value{
Assessment data as data frame
}
\description{
\code{kmassessmentsimulation} does a probabilistic knowledge assessment
for each response pattern in a data matrix and stores information about
the assessment.
}
\details{
\code{kmassessmentsimulation} applies the \code{kmsassess} function.
}
\examples{
kmassessmentsimulation(
  xpl$data,
  xpl$space,
  "halfsplit",
  "multiplicative",
  NULL,
  NULL,
  5,
  5,
  0.55
)

}
\seealso{
Other Knowledge assessment functions:
\code{\link[=kmassess]{kmassess()}},
\code{\link[=kmassessbayesian]{kmassessbayesian()}},
\code{\link[=kmassesshalfsplit]{kmassesshalfsplit()}},
\code{\link[=kmassessinformative]{kmassessinformative()}},
\code{\link[=kmassessmultiplicative]{kmassessmultiplicative()}}
}
\concept{Knowledge assessment functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phsg.R
\docType{data}
\name{phsg}
\alias{phsg}
\title{Knowledge space on linear functions}
\format{
A list containing the basis and the list of items
}
\usage{
phsg
}
\description{
Basis of a small knowledge space and list of items on linear functions
used in a manuscript by Steiner et al.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmbasisfringe.R
\name{kmbasisfringe}
\alias{kmbasisfringe}
\alias{kmbasisinnerfringe}
\alias{kmbasisouterfringe}
\title{Compute the fringe of a state within a knowledge structure using its basis}
\usage{
kmbasisfringe(state, basis)

kmbasisinnerfringe(state, basis)

kmbasisouterfringe(state, basis)
}
\arguments{
\item{state}{Binary vector representing a knowledge state}

\item{basis}{\code{kmbasis} object}
}
\value{
Binary vector representing the fringe
}
\description{
\code{kmbasisfringe} computes the fringe of a state
within a knowledge structure, i.e. the set of items by which
the state differs from its neighbours.
}
\examples{
kmbasisfringe(c(1,0,0,0), xpl$basis)

}
\references{
Hockemeyer C (1997). Using the Basis of a Knowledge Space for
Determining the Fringe of a Knowledge State.
\emph{Journal of Mathematical Psychology, 41,} 275–279.
}
\seealso{
Other Functions for fringes & neighbourhoods:
\code{\link[=kmbasisneighbourhood]{kmbasisneighbourhood()}},
\code{\link[=kmfringe]{kmfringe()}},
\code{\link[=kmneighbourhood]{kmneighbourhood()}}
}
\concept{Functions for fringes & neighbourhoods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cad.R
\docType{data}
\name{cad}
\alias{cad}
\title{Knowledge spaces on AutoCAD knowledge}
\format{
A list containing seven bases (cad1 to cad6, and cadmaj) in binary
matrix form. Each matrix has 28 columns representing the different knowledge
items and a varying number of rows containing the basis elements.
}
\usage{
cad
}
\description{
Bases of knowledge spaces on AutoCAD knowledge obtained from querying experts.
}
\details{
Six experts were queried about prerequisite relationships between 28 AutoCAD
knowledge items (Dowling, 1991; 1993). A seventh basis represents those
prerequisite relationships on which the majority (4 out of 6) of the experts
agree (Dowling & Hockemeyer, 1998).
}
\references{
Dowling, C. E. (1991). \emph{Constructing Knowledge Structures from
the Judgements of Experts.} Habilitationsschrift, Technische Universität
Carolo-Wilhelmina, Braunschweig, Germany.

Dowling, C. E. (1993). Applying the basis of a knowledge space for controlling
the questioning of an expert. \emph{Journal of Mathematical Psychology, 37,} 21–48.

Dowling, C. E. & Hockemeyer, C. (1998). Computing the intersection of knowledge
spaces using only their basis. In Cornelia E. Dowling, Fred S. Roberts, & Peter
Theuns, editors, \emph{Recent Progress in Mathematical Psychology,} pp. 133–141.
Lawrence Erlbaum Associates Ltd., Mahwah, NJ.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmneighbourhood.R
\name{kmneighbourhood}
\alias{kmneighbourhood}
\alias{kmnneighbourhood}
\title{Compute the neighbourhod of a state within a knowledge structure}
\usage{
kmneighbourhood(state, struct, include = FALSE)

kmnneighbourhood(state, struct, distance, include = FALSE)
}
\arguments{
\item{state}{Binary vector representing a knowledge state}

\item{struct}{Binary matrix representing a knowledge structure}

\item{include}{Boolean whether the original state should be included
(defaul FALSE)}

\item{distance}{Size of the n-neighbourhood}
}
\value{
Matrix containing the neighbouring states, one per row

Matrix containing the neighbouring states, one per row
}
\description{
\code{kmneighbourhood} computes the neighbourhood of a state
within a knowledge structure, i.e. the family of all other
states with a symmetric set difference of 1.

\code{kmnneighbourhood} computes the n-neighbourhood of a state
within a knowledge structure, i.e. the family of all other
states with a symmetric set difference maximal n.
}
\examples{
kmneighbourhood(c(1,1,0,0), xpl$space)

kmnneighbourhood(c(1,1,0,0), xpl$space, 2)

}
\seealso{
Other Functions for fringes & neighbourhoods:
\code{\link[=kmbasisfringe]{kmbasisfringe()}},
\code{\link[=kmbasisneighbourhood]{kmbasisneighbourhood()}},
\code{\link[=kmfringe]{kmfringe()}}
}
\concept{Functions for fringes & neighbourhoods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmlearningpaths.R
\name{kmlearningpaths}
\alias{kmlearningpaths}
\title{Determine all learning paths in a knowledge structure}
\usage{
kmlearningpaths(
  structure,
  through = NULL,
  from = NULL,
  to = NULL,
  simplified = FALSE
)
}
\arguments{
\item{structure}{Knowledge structure}

\item{through}{Knowledge state through which the paths should go}

\item{from}{Knowledge state where paths should start}

\item{to}{Knowledge state where paths should end}

\item{simplified}{Simplified printing of states within the paths (default FALSE)}
}
\value{
A list of learning paths where each learning path is a
list of states

If \code{through} is specified, only those learning paths are listed
which go through that state (binary vector). If \code{from} and
\code{to} are specified, the respective sub paths are returned.
}
\description{
Determine all learning paths in a knowledge structure
}
\examples{
kmlearningpaths(xpl$space)


}
\seealso{
Other Functions for wellgradedness & learning paths:
\code{\link[=kmgradations]{kmgradations()}},
\code{\link[=kmiswellgraded]{kmiswellgraded()}},
\code{\link[=kmlearningpathmatrices]{kmlearningpathmatrices()}}
}
\concept{Functions for wellgradedness & learning paths}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmlearningpathmatrices.R
\name{kmlearningpathmatrices}
\alias{kmlearningpathmatrices}
\title{Determine all learning paths in a knowledge structure}
\usage{
kmlearningpathmatrices(structure, through = NULL, from = NULL, to = NULL)
}
\arguments{
\item{structure}{Knowledge structure}

\item{through}{Knowledge state through which the paths should go}

\item{from}{Knowledge state where paths should start}

\item{to}{Knowledge state where paths should end}
}
\value{
A list of learning paths where each learning path matrix

If \code{through} is specified, only those learning paths are listed
which go through that state (binary vector). If \code{from} and
\code{to} are specified, the respective sub paths are returned.
}
\description{
Determine all learning paths in a knowledge structure
}
\examples{
kmlearningpathmatrices(xpl$space)


}
\seealso{
Other Functions for wellgradedness & learning paths:
\code{\link[=kmgradations]{kmgradations()}},
\code{\link[=kmiswellgraded]{kmiswellgraded()}},
\code{\link[=kmlearningpaths]{kmlearningpaths()}}
}
\concept{Functions for wellgradedness & learning paths}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmbasis.R, R/kmbasis.kmsurmisefunction.R,
%   R/kmbasis.kmsurmiserelation.R, R/kmbasis.matrix.R
\name{kmbasis}
\alias{kmbasis}
\alias{kmbasis.kmsurmisefunction}
\alias{kmbasis.kmsurmiserelation}
\alias{kmbasis.matrix}
\title{Compute the basis of a knowledge space}
\usage{
kmbasis(x)

\method{kmbasis}{kmsurmisefunction}(x)

\method{kmbasis}{kmsurmiserelation}(x)

\method{kmbasis}{matrix}(x)
}
\arguments{
\item{x}{Binary matrix representing a knowledge space}
}
\value{
Binary matrix representing the basis of the corresponding knowledge space.
}
\description{
\code{kmbasis.kmsurmisefunction} takes a surmise function and returns
the corresponding basis, i.e. effectively the collection of all clauses
in the surmise function.

\code{kmbasis.kmsurmiserelation} takes a surmise relation and returns the
corresponding basis.

\code{kmbasis.matrix} returns a matrix representing the basis of a
knowledge space. If \code{x} is a knowledge structure or an
arbitrary family of sets \code{kmbasis} returns the basis of
the smallest knowledge space containing \code{x}.
}
\examples{
kmbasis(xpl$sf)


kmbasis(xpl$sr)

kmbasis(xpl$space)

}
\seealso{
Other Different representations for knowledge structures:
\code{\link[=kmspace]{kmspace()}},
\code{\link[=kmsurmisefunction]{kmsurmisefunction()}},
\code{\link[=kmsurmiserelation]{kmsurmiserelation()}}
}
\concept{Different representations for knowledge structures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmsurmiserelation.R
\name{kmsurmiserelation}
\alias{kmsurmiserelation}
\title{Compute the surmise relation of a quasi-ordinal knowledge space}
\usage{
kmsurmiserelation(x)
}
\arguments{
\item{x}{Binary matrix representing a family of sets (class kmfamset)}
}
\value{
Binary matrix representing the surmise relation

Note: The columns of the surmise relation matrix describe the
minimal state for the respective item in the quasi-ordinal
knowledge space.
}
\description{
\code{kmsurmiserelation} returns a matrix representing the
surmise relation of a quasi-ordinal knowledge space. If \code{x}
is a general knowledge space, a knowledge structure or an
arbitrary family of sets, \code{kmsurmiserelation} returns the
surmise relation of the smallest quasi-ordinal knowledge space
containing \code{x}.
}
\examples{
kmsurmiserelation(xpl$space)

}
\seealso{
Other Different representations for knowledge structures:
\code{\link[=kmbasis]{kmbasis()}},
\code{\link[=kmspace]{kmspace()}},
\code{\link[=kmsurmisefunction]{kmsurmisefunction()}}
}
\concept{Different representations for knowledge structures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmsymmsetdiff.R
\name{kmsymmsetdiff}
\alias{kmsymmsetdiff}
\alias{kmsetdistance}
\title{Compute the symmetric set difference between two sets}
\usage{
kmsymmsetdiff(x, y)

kmsetdistance(x, y)
}
\arguments{
\item{x}{Binary vector representing a set}

\item{y}{Binary vector representing a set}
}
\value{
kmsymmsetdiff: Symmetric set difference between 'x' and 'y'

kmsetdistance: Distance between the sets 'x' and 'y', i.e. the cardinality
of the symmetric set difference
}
\description{
Compute the symmetric set difference between two sets
}
\examples{
kmsymmsetdiff(c(1,0,0), c(1,1,0))

kmsetdistance(c(1,0,0), c(1,1,0))

}
\seealso{
Other Utilities:
\code{\link[=binary_matrix_product]{binary_matrix_product()}},
\code{\link[=kmdoubleequal]{kmdoubleequal()}},
\code{\link[=kmminimalfamset]{kmminimalfamset()}},
\code{\link[=kmsetiselement]{kmsetiselement()}},
\code{\link{kmtrivial}}
}
\concept{Utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmcolors.R
\name{kmcolors}
\alias{kmcolors}
\title{Determine a color vector based on probabilities}
\usage{
kmcolors(prob, palette = cm.colors)
}
\arguments{
\item{prob}{Probability vector}

\item{palette}{Color palette (default = cm.colors)}
}
\description{
\code{kmcolors} takes a probabilty vector and a color palette and
creates a color vector to be used with \code{kstMatrix::plot}.
}
\seealso{
Other Functions for plotting and printing knowledge structures:
\code{\link[=kmprettyprint]{kmprettyprint()}},
\code{\link{plot}}
}
\concept{Functions for plotting and printing knowledge structures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmassessbayesian.R
\name{kmassessbayesian}
\alias{kmassessbayesian}
\title{Update probability distribution applying Bayesian update}
\usage{
kmassessbayesian(probs, ks, beta, eta, question, response)
}
\arguments{
\item{probs}{Probability distribution over the knowledge structure (vector)}

\item{ks}{Binary matrix of the knowledge structure}

\item{beta}{Vector of careless error probabilities}

\item{eta}{Vector of lucky guess probabilities}

\item{question}{Item that has been posed}

\item{response}{Correctness of received response (0 or 1)}
}
\value{
Updated probability vector
}
\description{
\code{kmassessbayesian} updates a probability distribution over a
knowledge structure according to the Bayesian update rule.
}
\examples{
kmassessbayesian(c(0.02, 0.1, 0.07, 0.01, 0.4, 0.17, 0.07, 0.08, 0.08),
                 xpl$space,
                 rep(0.2,4),
                 rep(0.1,4),
                 3,
                 1
                )

}
\seealso{
Other Knowledge assessment: 
\code{\link{kmassess}()},
\code{\link{kmassesshalfsplit}()},
\code{\link{kmassessinformative}()},
\code{\link{kmassessmentsimulation}()},
\code{\link{kmassessmultiplicative}()}
}
\concept{Knowledge assessment}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmminimalfamset.R
\name{kmminimalfamset}
\alias{kmminimalfamset}
\title{Determine subfamily of minimal states in a \code{kmfamset}}
\usage{
kmminimalfamset(x)
}
\arguments{
\item{x}{kmfamset}
}
\value{
kmfamset of minimal states

\code{kmminimalfamset()} determines the sub family of minimal sets
within a famset. If \code{x} is a knowledge structure, the result will
be a family containing (only) the empty set because the empty set is a
state in any knowledge structure and is its minimal state.

The concept of the minimal state of a family of sets should not be confused
with the concept of knowledge states minimal for some item.
}
\description{
Determine subfamily of minimal states in a \code{kmfamset}
}
\examples{
m <- matrix(c(1,0,0,0,1,0,1,0,1), ncol=3, byrow=TRUE)
m
kmminimalfamset(kmfamset(m))


}
\seealso{
Other Utilities:
\code{\link[=binary_matrix_product]{binary_matrix_product()}},
\code{\link[=kmdoubleequal]{kmdoubleequal()}},
\code{\link[=kmsetiselement]{kmsetiselement()}},
\code{\link[=kmsymmsetdiff]{kmsymmsetdiff()}},
\code{\link{kmtrivial}}
}
\concept{Utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmunion.R, R/kmunion.famset.R,
%   R/kmunion.kmbasis.R, R/kmunion.kmspace.R, R/kmunion.kmstructure.R,
%   R/kmunion.kmsurmisefunction.R, R/kmunion.kmsurmiserelation.R
\name{kmunion}
\alias{kmunion}
\alias{kmunion.kmfamset}
\alias{kmunion.kmbasis}
\alias{kmunion.kmspace}
\alias{kmunion.kmstructure}
\alias{kmunion.kmsurmisefunction}
\alias{kmunion.kmsurmiserelation}
\title{Compute the "union" of two KST objects}
\usage{
kmunion(x, y)

\method{kmunion}{kmfamset}(x, y)

\method{kmunion}{kmbasis}(x, y)

\method{kmunion}{kmspace}(x, y)

\method{kmunion}{kmstructure}(x, y)

\method{kmunion}{kmsurmisefunction}(x, y)

\method{kmunion}{kmsurmiserelation}(x, y)
}
\arguments{
\item{x}{First object}

\item{y}{Second object}
}
\value{
\emph{Union} of \code{x} and \code{y}
}
\description{
Both objects must be of the same kstMatrix class, the result will be
of the same class, i.e. a respective closure operator will be executed
automatically.
}
\details{
For surmise functions, the algorithm by Dowling & Hockemeyer (1998) is used.

Please note that, for families of sets, there are the functions
\code{kmunionclosure()} and \code{kmintersectionclosure()}.
}
\references{
Dowling CE, Hockemeyer C (1998). “Computing the Intersection
of Knowledge Spaces Using only their Basis.” In Dowling CE, Roberts FS,
Theuns P (eds.), \emph{Recent Progress in Mathematical Psychology,} 133–141.
Lawrence Erlbaum Associates Ltd., Mahwah, NJ.
}
\seealso{
Other Functions for working with knowledge structures:
\code{\link[=kmeqreduction]{kmeqreduction()}},
\code{\link[=kmexpand]{kmexpand()}},
\code{\link[=kmintersection]{kmintersection()}},
\code{\link[=kmmesh]{kmmesh()}},
\code{\link[=kmnotions]{kmnotions()}},
\code{\link[=kmsubstructure]{kmsubstructure()}},
\code{\link{kmtrivial}}
}
\concept{Functions for working with knowledge structures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmfringe.R
\name{kmfringe}
\alias{kmfringe}
\alias{kminnerfringe}
\alias{kmouterfringe}
\title{Compute the fringe of a state within a knowledge structure}
\usage{
kmfringe(state, struct)

kminnerfringe(state, struct)

kmouterfringe(state, struct)
}
\arguments{
\item{state}{Binary vector representing a knowledge state}

\item{struct}{Binary matrix representing a knowledge structure}
}
\value{
Binary vector representing the fringe
}
\description{
\code{kmfringe} computes the fringe of a state
within a knowledge structure, i.e. the set of items by which
the state differs from its neighbours.
}
\examples{
kmfringe(c(1,0,0,0), xpl$space)

}
\seealso{
Other Functions for fringes & neighbourhoods:
\code{\link[=kmbasisfringe]{kmbasisfringe()}},
\code{\link[=kmbasisneighbourhood]{kmbasisneighbourhood()}},
\code{\link[=kmneighbourhood]{kmneighbourhood()}}
}
\concept{Functions for fringes & neighbourhoods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmSF2basis.R
\name{kmSF2basis}
\alias{kmSF2basis}
\title{Derive a basis from a surmise function}
\usage{
kmSF2basis(sf)
}
\arguments{
\item{sf}{Surmise function}
}
\value{
Matrix representing the basis.
}
\description{
\code{kmSF2basis} expects a surmise function data frame
and returns the corresponding basis.
}
\seealso{
Other Different representations for knowledge spaces: 
\code{\link{kmSR2basis}()},
\code{\link{kmbasis}()},
\code{\link{kmsurmisefunction}()},
\code{\link{kmsurmiserelation}()},
\code{\link{kmunionclosure}()}
}
\concept{Different representations for knowledge spaces}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmassess.R, R/kmsassess.R
\name{kmassess}
\alias{kmassess}
\alias{Assessment}
\alias{kmsassess}
\title{Perform a probabilistic knowledge assessment}
\usage{
kmassess(
  r,
  pks,
  questioning,
  update,
  beta,
  eta,
  zeta0,
  zeta1,
  threshold,
  probdev = FALSE,
  directory = tempdir()
)

kmsassess(
  r,
  ks,
  questioning,
  update,
  beta,
  eta,
  zeta0,
  zeta1,
  threshold,
  probdev = FALSE,
  directory = NULL
)
}
\arguments{
\item{r}{Response pattern (binary vector)}

\item{pks}{Probabilistic knowledge structure: a data frame with a
probability distribution in the first columns and the structure
matrix in the subsequent columns.}

\item{questioning}{Questioning rule ("halfsplit" o "informative")}

\item{update}{Update rule ("Bayesian" or "multiplicative")}

\item{beta}{Careless error probability}

\item{eta}{Lucky guess probability}

\item{zeta0}{Update parameter for wrong responses}

\item{zeta1}{Update parameter for correct responses}

\item{threshold}{Probability threshold for stopping criterion}

\item{probdev}{Provide information on the probability development
including Hasse diagrams stored in \code{tempdir()}. Defaults to FALSE.}

\item{directory}{Where to store the Hasse diagrams.}

\item{ks}{Knowledge structure: a binary matrix}
}
\value{
A list with the following elements:
\describe{
\item{state}{Diagnosed knowledge state (binary vector)}
\item{probs}{Resulting probability distribution. If probdev is set to TRUE,
a list of probability distributions for each step is given
instead.}
\item{queried}{Sequence of items used in the assessment (list)}
\item{qtime}{Average time for finding a question}
\item{utime}{Average time for updating the probabilities}
}

A list with the following elements:
\describe{
\item{state}{Diagnosed knowledge state (binary vector)}
\item{probs}{Resulting probability distribution. If probdev is set to TRUE,
a list of probability distributions for each step is given
instead.}
\item{queried}{Sequence of items used in the assessment (list)}
\item{qtime}{Average time for finding a question}
\item{utime}{Average time for updating the probabilities}
}
}
\description{
\code{kmassess} performs a probabilistic knowledge assessment for a given
response vector, knowledge structure, and BLIM parameters.

\code{kmsassess} performs a simplified probabilistic knowledge assessment
for a given response vector, knowledge structure, and BLIM parameters. It
assumes an equal probability distribution over the knowledge structure
as starting point and identical beta and eta values for all items.
}
\details{
\code{kmassess} implements the stochastic assessment procedures according
to Doignon & Falmagne, 1999, chapter 10.

\code{kmassess} stops if the number of questions has reached twice the
number of items.
}
\section{Background}{

Doignon & Falmagne (1985, 1999) proposed knowledge space theory originally
with adaptive knowledge assessment in mind. The basic idea is to apply
prerequisite relationships between items for reducing the number of problems
to be posed to a learner in knowledge assessment.

Falmagne & Doignon (1988; Doignon & Falmange, 1999, chapte 10) proposed
a class of stochastic procdures for such adaptive assessment which take
into account that careless errors and lucky guesses may happen during the
assessment by estimating a probability distribution over the knowledge
structure. Such an assessment consists of three important parts
\itemize{
\item{Question rule}
\item{Update rule}
\item{Stopping criterion}
}

For the \strong{question rule,} they propose the \emph{halfsplit} and the \emph{infomrative}
rules, implemented in \code{kmassesshalfslit} and \code{kmassessinfomrative}.

For the \strong{update rule,} they again propose two possibilities there the
\emph{multiplicative rule} is a generalisation of the (classical) \emph{Bayesian
update rule} implemented here in \code{kmassessmultiplicative} and
\code{kmassessbayesian}, respectively.

As \strong{stopping criterion,} usually a threshold for the maximal probability for
one knowledge state is used. It is strongly recommended to keep this larger
than 0.5 in order to have one unequivocal resulting state (see also
Hockemeyer, 2002).
\subsection{Framework of assessment functions within the \pkg{kstMatrix} package:}{

The founding stones are the four aforementioned functions for finding
suitable questions and for updating the probability estimates, respectively.
They could also be used in an interactive system, e.g. a Shiny app, for
"real" adaptive assessment.

The remaining three assessment functions serve for mere simulation of
adaptive assessment. \code{kmassess} takes, among others, a full response
pattern as parameter and takes the responses for the selected questions
from this vector. \code{kmsassess} is a simplified version where the
update parameters (beta and eta for Bayesian or zeta0 and zeta1 for
multiplicative update, respectively) are identical for all items whereas
they are item-specific in \code{kmassess}. Finally,
\code{kmassesssimulation} takes a whole data set, i.e. a collection of
response patterns, and does an assessment for each of these patterns. Its
result is a data frame which should be suitable for further statistical
evaluation, especially if it is called several times with variant
parameters (e.g., structures, update parameters, update and question rules).

Both, \code{kmsassess} and \code{kmassesssimulation} call \code{kmassess}.
}

\subsection{Problems}{

In rare cases \code{kmassess} may flip forth and back between probability
distributions resulting in an endless loop. Therefore, it stops after
twice the number of items delivering a \code{NULL} result.
}
}

\examples{
kmassess(c(1, 1, 0, 0),
         cbind(as.data.frame(as.matrix(rep(1/9.0, 9), ncol=1)), xpl$space),
         "halfsplit",
         "Bayesian",
         rep(0.12, 4),
         rep(0.1, 4),
         NULL,
         NULL,
         0.55
        )

kmsassess(c(1,1,0,0), xpl$space, "halfsplit", "Bayesian", 0.1, 0.1, NULL, NULL, 0.55)

}
\references{
Doignon, J.-P. & Falmagne, J.-C. (1985). Spaces for the assessment of
knowledge. \emph{International Journal of Man-Machne-Studies, 23,} 175-196.
\doi{10.1016/S0020-7373(85)80031-6}.

Doignon, J.-P. & Falmagne, J.-C. (1999). \emph{Knowledge Spaces.} Springer Verlag,
Berlin. \doi{10.1007/978-3-642-58625-5}.

Falmagne, J.-C. & Doignon, J.-P. (1988). A class of stochastic procedures
for the assessment of knowledge. \emph{British Journal of Mathematical and
Statistical Psychology, 41,} 1-23. \doi{10.1111/j.2044-8317.1988.tb00884.x}.

Hoxkemeyer, C. (2002). A comparison of non-deterministic procedures for the
adaptive assessment of knowledge. \emph{Psychlogische Beiträge, 44(4),} 495-503.
}
\seealso{
Other Knowledge assessment functions:
\code{\link[=kmassessbayesian]{kmassessbayesian()}},
\code{\link[=kmassesshalfsplit]{kmassesshalfsplit()}},
\code{\link[=kmassessinformative]{kmassessinformative()}},
\code{\link[=kmassessmentsimulation]{kmassessmentsimulation()}},
\code{\link[=kmassessmultiplicative]{kmassessmultiplicative()}}
}
\concept{Knowledge assessment functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmunionclosure.R
\name{kmunionclosure}
\alias{kmunionclosure}
\title{Close a family of sets under union, i.e. determine the smallest knowledge space containing it}
\usage{
kmunionclosure(x)
}
\arguments{
\item{x}{Family of set}
}
\value{
Knowledge space

\code{kmunionclosure} implements the irredundant algorithm developed by Dowling (1993).
}
\description{
Close a family of sets under union, i.e. determine the smallest knowledge space containing it
}
\examples{
kmunionclosure(xpl$basis)


}
\references{
Dowling, C. E. (1993). On the irredundant construction of knowledge spaces.
\emph{Journal of Mathematical Psychology, 37,} 49–62.
}
\seealso{
Other Closure operators:
\code{\link[=kmclosure]{kmclosure()}},
\code{\link[=kmintersectionclosure]{kmintersectionclosure()}}
}
\concept{Closure operators}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Deprecated.R
\name{kmbasisdiagram}
\alias{kmbasisdiagram}
\title{DEPRECATED: \code{kmbasisdiagram()} has been replaced by a \code{plot()} method.}
\usage{
kmbasisdiagram(x, horizontal = TRUE, colors = NULL)
}
\arguments{
\item{x}{Basis}

\item{horizontal}{Boolean whether the diagram should be drawn horizontally or vertically (default TRUE)}

\item{colors}{Color vector (default NULL)}
}
\description{
DEPRECATED: \code{kmbasisdiagram()} has been replaced by a \code{plot()} method.
}
\seealso{
Other Deprecated functions and methods:
\code{\link[=kmSF2basis]{kmSF2basis()}},
\code{\link[=kmSR2basis]{kmSR2basis()}},
\code{\link[=kmSRdiagram]{kmSRdiagram()}},
\code{\link[=kmhasse]{kmhasse()}}
}
\concept{Deprecated functions and methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmgenerate.R
\name{kmgenerate}
\alias{kmgenerate}
\title{Generate a knowledge structure from a set of response patterns}
\usage{
kmgenerate(x, threshold = 0)
}
\arguments{
\item{x}{Binary matrix representing a data set}

\item{threshold}{Threshold for taking response patterns as knowledge states (default 0)}
}
\value{
Binary matrix representing the generated knowledge structure
}
\description{
\code{kmgenerate} returns a matrix representing a knowledge structure generated
from data. It uses a simplistic approach: patterns with a frequency above a
specified threshold are considered as knowledge states.
If the specified threshold is 0 (default) a real threshold is computed as
N (number of  response patterns) divided by 2^|Q|.
Please note that the number of response patterns should be much higher than the
size of the power set of the item set Q. A factor of art least 10 is recommended.
Currently, the number of items is limited to the number of bits in a C long
minus one (i.e. 31 under Windows and 63 otherwise). But we would probably run into
memory problems way earlier anyway.
}
\examples{
kmgenerate(xpl$sim, 15)

}
\seealso{
Other Functions for working with data:
\code{\link[=kmSRvalidate]{kmSRvalidate()}},
\code{\link[=kmdist]{kmdist()}},
\code{\link[=kmiita2SR]{kmiita2SR()}},
\code{\link[=kmsimulate]{kmsimulate()}},
\code{\link[=kmvalidate]{kmvalidate()}}
}
\concept{Functions for working with data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmspace.R, R/kmspace.kmdata.R,
%   R/kmspace.kmfamset.R, R/kmspace.kmsurmisefunction.R,
%   R/kmspace.kmsurmiserelation.R
\name{kmspace}
\alias{kmspace}
\alias{kmspace.kmdata}
\alias{kmspace.kmfamset}
\alias{kmspace.kmsurmisefunction}
\alias{kmspace.kmsurmiserelation}
\title{\code{kmspace} constructor method}
\usage{
kmspace(x)

\method{kmspace}{kmdata}(x)

\method{kmspace}{kmfamset}(x)

\method{kmspace}{kmsurmisefunction}(x)

\method{kmspace}{kmsurmiserelation}(x)
}
\arguments{
\item{x}{KST object}
}
\value{
Corresponding knowledge space
}
\description{
\code{kmspace()} determines the knowledge space corresponding to a KST object

Please note that a knowledge space delineated by a surmise relation is
quasi-ordinal. Therefore, the result is automatically marked as
\code{kmqspace} object.
}
\examples{
kmspace(xpl$basis)
kmspace(xpl$sf)

}
\seealso{
Other Constructors:
\code{\link[=kmfamset]{kmfamset()}},
\code{\link[=kmqspace]{kmqspace()}},
\code{\link[=kmstructure]{kmstructure()}}

Other Different representations for knowledge structures:
\code{\link[=kmbasis]{kmbasis()}},
\code{\link[=kmsurmisefunction]{kmsurmisefunction()}},
\code{\link[=kmsurmiserelation]{kmsurmiserelation()}}
}
\concept{Constructors}
\concept{Different representations for knowledge structures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmnotions.R
\name{kmnotions}
\alias{kmnotions}
\title{Determine the notions of a knowledge structure}
\usage{
kmnotions(x)
}
\arguments{
\item{x}{Binary matrix representing a knowledge structure}
}
\value{
Binary matrix representing notions in the knowledge structure

The matrix has a '1' in row 'i' and column 'j' if 'i' and 'j' belong
to the same notion (i.e. are equivalent). It is a symmetric matrix
with '1's in the main diagonal.
}
\description{
\code{kmnotions} returns a matrix representing the
notions of a knowledge structure.
}
\examples{
kmnotions(xpl$space)

}
\seealso{
Other Properties of knowledge structures:
\code{\link[=kmheights]{kmheights()}},
\code{\link[=kmiswellgraded]{kmiswellgraded()}}

Other Functions for working with knowledge structures:
\code{\link[=kmeqreduction]{kmeqreduction()}},
\code{\link[=kmexpand]{kmexpand()}},
\code{\link[=kmintersection]{kmintersection()}},
\code{\link[=kmmesh]{kmmesh()}},
\code{\link[=kmsubstructure]{kmsubstructure()}},
\code{\link{kmtrivial}},
\code{\link[=kmunion]{kmunion()}}
}
\concept{Functions for working with knowledge structures}
\concept{Properties of knowledge structures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmstructure.R
\name{kmstructure}
\alias{kmstructure}
\title{Convert a binary matrix to a \code{kmstructure} object}
\usage{
kmstructure(x)
}
\arguments{
\item{x}{Binary matrix representing a family of sets}
}
\value{
\code{kmstructure} object
}
\description{
\code{kmstructure()} returns a \code{kmstructure} object after checking that
the passed object is a binary matrix without double rows. The empty set
and the full item set are added if missing.
}
\examples{
m <- matrix(c(1,0,0,0,1,0,1,1,1), nrow=3, byrow=TRUE)
kmstructure(m)

}
\seealso{
Other Constructors:
\code{\link[=kmfamset]{kmfamset()}},
\code{\link[=kmqspace]{kmqspace()}},
\code{\link[=kmspace]{kmspace()}}
}
\concept{Constructors}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmclosure.R,
%   R/kmclosure.kmattributionfunction.R, R/kmclosure.kmattributionrelation.R,
%   R/kmclosure.kmdata.R, R/kmclosure.kmfamset.R
\name{kmclosure}
\alias{kmclosure}
\alias{kmclosure.kmattributionfunction}
\alias{kmclosure.kmattributionrelation}
\alias{kmclosure.kmdata}
\alias{kmclosure.kmfamset}
\title{Determine the closure of KST objects}
\usage{
kmclosure(x, ...)

\method{kmclosure}{kmattributionfunction}(x, ...)

\method{kmclosure}{kmattributionrelation}(x, ...)

\method{kmclosure}{kmdata}(x, ...)

\method{kmclosure}{kmfamset}(x, ..., closure = "union")
}
\arguments{
\item{x}{KST object}

\item{...}{Optional additional parameters}

\item{closure}{Type of closure

For families of sets (\code{kmfamset}), kmclosure is basically a wrapper
for \code{kmunionclosure()} and \code{kmintersectionclosure}.
\code{closure()} may take the values "union" (default) or "intersection".
The former requests a closure under union, the latter a closure under
union \strong{and} intersection.}
}
\value{
Corresponding closed KST object

An attribution relation or
attribution function is closed to the corresponding surmise relation
or surmise function, respectively.

A family of sets (including bases, knowledge structure, and knowledge space)
is closed to a knowledge space or quasi-ordinal knowledge space.
}
\description{
\code{kmclosure()} is an S3 method closing \code{kstMatrix} objects.
}
\seealso{
Other Closure operators:
\code{\link[=kmintersectionclosure]{kmintersectionclosure()}},
\code{\link[=kmunionclosure]{kmunionclosure()}}
}
\concept{Closure operators}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmassessmultiplicative.R
\name{kmassessmultiplicative}
\alias{kmassessmultiplicative}
\title{Update probability distribution applying multiplicative rule}
\usage{
kmassessmultiplicative(probs, ks, zeta0, zeta1, question, response)
}
\arguments{
\item{probs}{Probability distribution over the knowledge structure (vector)}

\item{ks}{Binary matrix of the knowledge structure}

\item{zeta0}{Vector of update parameters for wrong responses}

\item{zeta1}{Vector of update parameters for correct responses}

\item{question}{Item that has been posed}

\item{response}{Correctness of received response (0 or 1)}
}
\value{
Updated probability vector
}
\description{
\code{kmassessmultiplicative} updates a probability distribution on a
knowledge structure according to the multiplicative rule.
}
\examples{
kmassessmultiplicative(c(0.02, 0.1, 0.07, 0.01, 0.4, 0.17, 0.07, 0.08, 0.08),
                 xpl$space,
                 rep(1.2,4),
                 rep(2.1,4),
                 3,
                 1
                )

}
\seealso{
Other Knowledge assessment functions:
\code{\link[=kmassess]{kmassess()}},
\code{\link[=kmassessbayesian]{kmassessbayesian()}},
\code{\link[=kmassesshalfsplit]{kmassesshalfsplit()}},
\code{\link[=kmassessinformative]{kmassessinformative()}},
\code{\link[=kmassessmentsimulation]{kmassessmentsimulation()}}
}
\concept{Knowledge assessment functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmintersection.R, R/kmintersection.kmqspace.R,
%   R/kmintersection.kmspace.R, R/kmintersection.kmsurmisefunction.R,
%   R/kmintersection.kmsurmiserelation.R
\name{kmintersection}
\alias{kmintersection}
\alias{kmintersection.kmqspace}
\alias{kmintersection.kmspace}
\alias{kmintersection.kmsurmisefunction}
\alias{kmintersection.kmsurmiserelation}
\title{Compute the "intersection" of two KST objects}
\usage{
kmintersection(x, y)

\method{kmintersection}{kmqspace}(x, y)

\method{kmintersection}{kmspace}(x, y)

\method{kmintersection}{kmsurmisefunction}(x, y)

\method{kmintersection}{kmsurmiserelation}(x, y)
}
\arguments{
\item{x}{First KST object}

\item{y}{Second KST object}
}
\value{
\emph{Intersection} of \code{x} and \code{y}
}
\description{
Both objects must be of the same kstMatrix class, the result will be
of the same class, i.e. a respective closure operator will be executed
automatically.

Please note that the intersection of surmise relations is automatically
a surmise relation without any closure needs.
}
\examples{
sf <- kmsurmisefunction(kmbasis(xpl$sr))
kmintersection(xpl$sf, sf)

}
\seealso{
Other Functions for working with knowledge structures:
\code{\link[=kmeqreduction]{kmeqreduction()}},
\code{\link[=kmexpand]{kmexpand()}},
\code{\link[=kmmesh]{kmmesh()}},
\code{\link[=kmnotions]{kmnotions()}},
\code{\link[=kmsubstructure]{kmsubstructure()}},
\code{\link{kmtrivial}},
\code{\link[=kmunion]{kmunion()}}
}
\concept{Functions for working with knowledge structures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmsurmisefunction.R
\name{kmsurmisefunction}
\alias{kmsurmisefunction}
\title{Compute the surmise function for a knowledge space or basis}
\usage{
kmsurmisefunction(x, itemname = "Item")
}
\arguments{
\item{x}{Binary matrix representing a knowledge space or basis}

\item{itemname}{Name for the first column of the resulting data frame}
}
\value{
Data frame  representing the surmise unction of \code{x}.
}
\description{
\code{kmsurmisefunction} returns a data frame representing the
surmise function for a knowledge space or basis. The rows of the
data frame are ordered by item name.
}
\examples{
kmsurmisefunction(xpl$space)

}
\seealso{
Other Different representations for knowledge structures:
\code{\link[=kmbasis]{kmbasis()}},
\code{\link[=kmspace]{kmspace()}},
\code{\link[=kmsurmiserelation]{kmsurmiserelation()}}
}
\concept{Different representations for knowledge structures}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmfamset.R
\name{kmfamset}
\alias{kmfamset}
\title{Convert a binary matrix to a \code{kmfamset} object (family of sets)}
\usage{
kmfamset(x)
}
\arguments{
\item{x}{Binary matrix representing a family of sets}
}
\value{
\code{kmfamset} object
}
\description{
\code{kmfamset} returns a \code{kmfamset} object after checking that
the passed object is a binary matrix with all different rows. If the
passe object inherits the \code{kmfamset} property, nothing else is
changed.
}
\examples{
m <- as.matrix(c(1,0,0,0,1,0,1,1,1), nrow=3, byrow=TRUE)
kmfamset(m)

}
\seealso{
Other Constructors:
\code{\link[=kmqspace]{kmqspace()}},
\code{\link[=kmspace]{kmspace()}},
\code{\link[=kmstructure]{kmstructure()}}
}
\concept{Constructors}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmrefine.R
\name{kmrefine}
\alias{kmrefine}
\title{Refine a famset by dissolving a notion through a basis on the notion}
\usage{
kmrefine(x, b)
}
\arguments{
\item{x}{Family of sets}

\item{b}{Basis for refinement}
}
\value{
Refined family of sets
}
\description{
Refine a famset by dissolving a notion through a basis on the notion
}
\examples{
x <- kmfamset(matrix(c(1,0,0,0,0,1,0,0,1,1,1,1), ncol=4, byrow=TRUE))
colnames(x) <- c("a", "b", "c", "d")
x
b <- kmbasis(kmfamset(matrix(c(1,0,1,1), ncol=2, byrow=TRUE)))
colnames(b) <- c("c", "d")
b
kmrefine(x,b)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kmiita2SR.R
\name{kmiita2SR}
\alias{kmiita2SR}
\title{Convert an IITA result into a surmise relation matrix}
\usage{
kmiita2SR(ii, names = NULL, items = 0)
}
\arguments{
\item{ii}{\code{iita()} result}

\item{names}{Vector of item names (default NULL)}

\item{items}{Minimal number of items (default 0)}
}
\value{
Surmise relation matrix

The \code{iita()} function looses information on the item names and
uses consecutive numbers instead. The \code{implications} part of its
result does not give any hint on isolated items, i.e. items which
neither have a prerequisite nor are prerequisite of any other item.
Therefore, a minimal number of items can be passed to
\code{kmiita2SR()}. If the highest item number within
\code{implications} is higher, this \code{items} parameter is ignored.
}
\description{
\code{kmiita2SR} takes the result of a \code{DAKS::iita()} call and
delivers the matrix of the computed surmise relation.
}
\seealso{
Other Functions for working with data:
\code{\link[=kmSRvalidate]{kmSRvalidate()}},
\code{\link[=kmdist]{kmdist()}},
\code{\link[=kmgenerate]{kmgenerate()}},
\code{\link[=kmsimulate]{kmsimulate()}},
\code{\link[=kmvalidate]{kmvalidate()}}
}
\concept{Functions for working with data}
